@@ -1,11 +1,25 @@
 #ifndef CHAT_H
 #define CHAT_H
 
+#include <cstdlib>
 #include <deque>
+#include <list>
 
 #include "licq_packets.h"
+#include "licq_socket.h"
 class CICQDaemon;
 
+// Define for marking functions as deprecated
+#ifndef LICQ_DEPRECATED
+# if defined(__GNUC__) && !defined(__INTEL_COMPILER) && (__GNUC__ - 0 > 3 || (__GNUC__ - 0 == 3 && __GNUC_MINOR__ - 0 >= 2))
+#  define LICQ_DEPRECATED __attribute__ ((__deprecated__))
+# elif defined(_MSC_VER) && (_MSC_VER >= 1300)
+#  define LICQ_DEPRECATED __declspec(deprecated)
+# else
+#  define LICQ_DEPRECATED
+# endif
+#endif
+
 
 /*----------------------
  * Licq Chat Module
@@ -198,7 +212,6 @@ public:
 
   // Accessors
   const char *Name() { return m_szName; }
-  unsigned long Uin() { return m_nUin; }
   char *Id() { return m_szId; }
   unsigned long PPID() { return m_nPPID; }
   unsigned short Port() { return m_nPort; }
@@ -211,8 +224,10 @@ public:
 
   virtual ~CPChat_Color();
 
+  // Deprecated functions, to be removed
+  LICQ_DEPRECATED unsigned long Uin() { return strtoul(m_szId, NULL, 10); }
+
 protected:
-  unsigned long m_nUin;
   char *m_szId;
   unsigned long m_nPPID;
   char *m_szName;
@@ -233,7 +248,7 @@ class CChatClient
 {
 public:
   CChatClient();
-  CChatClient(ICQUser *);
+  CChatClient(const ICQUser* u);
   CChatClient(const CChatClient &);
   CChatClient& operator=(const CChatClient &);
   ~CChatClient();
@@ -247,7 +262,6 @@ public:
 
   unsigned long m_nVersion;
   unsigned short m_nPort;
-  unsigned long m_nUin;
   char *m_szId;
   unsigned long m_nPPID;
   unsigned long m_nIp;
@@ -289,7 +303,6 @@ public:
 
   // Accessors
   const char *Name() { return m_szName; }
-  unsigned long Uin() { return m_nUin; }
   char *Id() { return m_szId; }
   unsigned long PPID() { return m_nPPID; }
   unsigned short Session() { return m_nSession; }
@@ -311,8 +324,10 @@ public:
   unsigned char FontStyle() { return m_nFontStyle; }
   ChatClientList &ChatClients()  { return chatClients; }
 
+  // Deprecated functions, to be removed
+  LICQ_DEPRECATED unsigned long Uin() { return strtoul(m_szId, NULL, 10); }
+
 protected:
-  unsigned long m_nUin;
   char *m_szId;
   unsigned long m_nPPID;
   unsigned short m_nSession;
@@ -472,7 +487,6 @@ extern "C"
 class CChatUser
 {
 public:
-  unsigned long Uin()          { return uin; }
   char *Id()                   { return szId; }
   unsigned long PPID()         { return nPPID; }
   unsigned long ToKick()       { return nToKick; }
@@ -492,10 +506,12 @@ public:
 
   ~CChatUser();
 
+  // Deprecated functions, to be removed
+  LICQ_DEPRECATED unsigned long Uin()          { return strtoul(szId, NULL, 10); }
+
 protected:
   CChatUser();
 
-  unsigned long uin;
   char *szId;
   unsigned long nPPID;
   unsigned long nToKick;
@@ -596,8 +612,8 @@ public:
   void SendNewline();
   void SendBackspace();
   void SendCharacter(char);
-  void SendKick(unsigned long);
-  void SendKickNoVote(unsigned long);
+  void SendKick(const char* id);
+  void SendKickNoVote(const char* id);
   void SendVoteYes(unsigned long);
   void SendVoteNo(unsigned long);
 
@@ -618,7 +634,6 @@ protected:
 
   CICQDaemon *licqDaemon;
   int pipe_events[2], pipe_thread[2];
-  unsigned long m_nUin;
   char *m_szId;
   unsigned long m_nPPID;
   unsigned short m_nSession;
@@ -656,8 +671,7 @@ protected:
   void FinishKickVote(VoteInfoList::iterator, bool);
 
   void SendBuffer(CBuffer *, unsigned char,
-                  unsigned long _nUin = 0,
-                  bool bNotIter = true);
+      const char* id = NULL, bool bNotIter = true);
   bool SendBufferToClient(CBuffer *, unsigned char, CChatUser *);
   void SendBuffer_Raw(CBuffer *);
   //void SendPacket(CPacket *);
