@@ -54,9 +54,14 @@
  * which case the file will be saved as <filename>.<timestamp>
  *-------------------------------------------------------------------------*/
 
-#include <sys/time.h>
+#include <list>
+
+// Order of declaration is significant here!
+typedef std::list<char *> FileList;
+typedef std::list<const char *> ConstFileList;
 
 #include "licq_packets.h"
+#include "licq_socket.h"
 class CICQDaemon;
 
 
@@ -87,10 +92,13 @@ struct SFileReverseConnectInfo
 class CPacketFile : public CPacket
 {
 public:
-  virtual const unsigned short Sequence()    { return 0; };
-  virtual const unsigned short SubSequence() { return 0; };
-  virtual const unsigned short Command()     { return 0; };
-  virtual const unsigned short SubCommand()  { return 0; };
+  CPacketFile();
+  virtual ~CPacketFile();
+
+  virtual unsigned short Sequence()    { return 0; };
+  virtual unsigned short SubSequence() { return 0; };
+  virtual unsigned short Command()     { return 0; };
+  virtual unsigned short SubCommand()  { return 0; };
 protected:
    void InitBuffer()   { buffer = new CBuffer(m_nSize); };
 };
@@ -101,8 +109,9 @@ protected:
 class CPFile_InitClient : public CPacketFile
 {
 public:
-   CPFile_InitClient(char *_szLocalName, unsigned long _nNumFiles,
-                     unsigned long _nTotalSize);
+  CPFile_InitClient(char *_szLocalName, unsigned long _nNumFiles,
+                    unsigned long _nTotalSize);
+  virtual ~CPFile_InitClient();
 };
 
 
@@ -111,7 +120,8 @@ public:
 class CPFile_InitServer : public CPacketFile
 {
 public:
-   CPFile_InitServer(char *_szLocalName);
+  CPFile_InitServer(char *_szLocalName);
+  virtual ~CPFile_InitServer();
 };
 
 
@@ -121,20 +131,21 @@ public:
 class CPFile_Info : public CPacketFile
 {
 public:
-   CPFile_Info(const char *_szFileName);
-   virtual ~CPFile_Info();
-   bool IsValid()  { return m_bValid; };
-   unsigned long GetFileSize()
-     { return m_nFileSize; };
-   const char *GetFileName()
-     { return m_szFileName; }
-   const char *ErrorStr()
-     { return strerror(m_nError); }
+  CPFile_Info(const char *_szFileName);
+  virtual ~CPFile_Info();
+
+  bool IsValid()  { return m_bValid; };
+  unsigned long GetFileSize()
+    { return m_nFileSize; };
+  const char *GetFileName()
+    { return m_szFileName; }
+  const char *ErrorStr()
+    { return strerror(m_nError); }
 protected:
-   bool m_bValid;
-   int m_nError;
-   char *m_szFileName;
-   unsigned long m_nFileSize;
+  bool m_bValid;
+  int m_nError;
+  char *m_szFileName;
+  unsigned long m_nFileSize;
 };
 
 
@@ -143,7 +154,8 @@ protected:
 class CPFile_Start : public CPacketFile
 {
 public:
-   CPFile_Start(unsigned long nFilePos, unsigned long nFile);
+  CPFile_Start(unsigned long nFilePos, unsigned long nFile);
+  virtual ~CPFile_Start();
 };
 
 
@@ -152,7 +164,8 @@ public:
 class CPFile_SetSpeed : public CPacketFile
 {
 public:
-   CPFile_SetSpeed(unsigned long nSpeed);
+  CPFile_SetSpeed(unsigned long nSpeed);
+  virtual ~CPFile_SetSpeed();
 };
 
 
@@ -170,7 +183,7 @@ public:
   unsigned char Command() { return m_nCommand; }
   const char *Data() { return m_szData; }
 
-  ~CFileTransferEvent() { if (m_szData != NULL) free(m_szData); }
+  ~CFileTransferEvent();
 
 protected:
   CFileTransferEvent(unsigned char t, char *d = NULL);
@@ -181,11 +194,8 @@ friend class CFileTransferManager;
 };
 
 typedef std::list<CFileTransferEvent *> FileTransferEventList;
-typedef std::list<const char *> ConstFileList;
-typedef std::list<char *> FileList;
 typedef std::list<class CFileTransferManager *> FileTransferManagerList;
 
-
 class CFileTransferManager
 {
 public:
@@ -202,16 +212,17 @@ public:
   const char *LocalName()  { return m_szLocalName; }
   direction Direction() { return m_nDirection; }
   unsigned long Uin() { return m_nUin; }
+  const char* Id() const { return myId; }
 
   // Available after FT_STARTxBATCH
   const char *RemoteName()  { return m_szRemoteName; }
   unsigned short BatchFiles() { return m_nBatchFiles; }
   unsigned long BatchSize() { return m_nBatchSize; }
   time_t BatchStartTime() { return m_nBatchStartTime; }
-  
+
   // Available between FT_CONFIRMxFILE and FT_STATE_
 
-	// You must use this function to start receiving the incoming file, possibly
+  // You must use this function to start receiving the incoming file, possibly
   // giving it a different name on the local machine.
   bool StartReceivingFile(char *szFileName = NULL);
 
@@ -258,6 +269,7 @@ protected:
   unsigned char m_nResult;
   unsigned long m_nUin;
   unsigned short m_nSession, m_nSpeed, m_nState;
+  char myId[16];
 
   char m_szLocalName[64], m_szRemoteName[64];
   unsigned short m_nPort;
