@@ -2,11 +2,14 @@
 #define ICQPACKET_H
 
 #include "licq_user.h"
-#include "licq_buffer.h"
-#include "licq_socket.h"
 #include "licq_icq.h"
 
+class ICQUser;
 class CICQColor;
+class CBuffer;
+class INetSocket;
+
+typedef std::list<std::string> StringList;
 
 // values of extra info to identify plugin request
 const unsigned short DirectInfoPluginRequest     = 1;
@@ -32,37 +35,6 @@ unsigned short ReversePort(unsigned short p);
 unsigned short LengthField(const char *szField);
 char *PipeInput(char *m_szMessage);
 
-//
-// These classes, CPX_*, are general classes for different packets that do the
-// same function that may be sent through the server or directly to the client.
-// This gives the direct and server packets a multiple inheritence.
-//
-
-//-----FileTransfer------------------------------------------------------------
-class CPX_FileTransfer
-{
-public:
-  CPX_FileTransfer(ConstFileList &lFileList, const char *szFileName);
-  virtual ~CPX_FileTransfer();
-
-  bool IsValid()	{ return m_bValid; }
-  ConstFileList GetFileList()	{ return m_lFileList; }
-  const char *GetFilename()	{ return m_szFilename; }
-  const char *GetDescription() { return m_szDesc; }
-  unsigned long GetFileSize()	{ return m_nFileSize; }
-
-protected:
-  CPX_FileTransfer();
-
-  bool          m_bValid;
-  char          *m_szDesc;
-  char          *m_szFilename;
-  ConstFileList m_lFileList;
-  unsigned long m_nFileSize;
-};
-
-
-
 //=====Packet===================================================================
 
 class CPacket
@@ -105,6 +77,40 @@ protected:
    static char s_nMode;
 };
 
+// Order of inclusion is significant here!
+#include "licq_filetransfer.h"
+
+//
+// These classes, CPX_*, are general classes for different packets that do the
+// same function that may be sent through the server or directly to the client.
+// This gives the direct and server packets a multiple inheritence.
+//
+
+//-----FileTransfer------------------------------------------------------------
+class CPX_FileTransfer
+{
+public:
+  CPX_FileTransfer(ConstFileList &lFileList, const char *szFileName);
+  virtual ~CPX_FileTransfer();
+
+  bool IsValid()                { return m_bValid; }
+  ConstFileList GetFileList()   { return m_lFileList; }
+  const char *GetFilename()     { return m_szFilename; }
+  const char *GetDescription() { return m_szDesc; }
+  unsigned long GetFileSize()   { return m_nFileSize; }
+
+protected:
+  CPX_FileTransfer();
+
+  bool          m_bValid;
+  char          *m_szDesc;
+  char          *m_szFilename;
+  ConstFileList m_lFileList;
+  unsigned long m_nFileSize;
+};
+
+
+
 //=====ServerTCP===============================================================
 
 //-----SrvPacketTcp------------------------------------------------------------
@@ -134,7 +140,7 @@ protected:
   void InitBuffer();
 
   static bool s_bRegistered;
-  static unsigned short s_nSequence;
+  static unsigned short s_nSequence[32];
   static unsigned short s_nSubSequence;
   static pthread_mutex_t s_xMutex;
 
@@ -145,6 +151,7 @@ protected:
   unsigned short m_nSubType;
   unsigned short m_nSubCommand;
   unsigned short m_nExtraInfo;
+  unsigned short m_nService;
 
   char *m_szSequenceOffset;
 };
@@ -196,6 +203,12 @@ protected:
    static unsigned long  s_nSessionId;
 };
 
+class CPU_ConnectStart : public CSrvPacketTcp
+{
+public:
+  CPU_ConnectStart();
+};
+
 //-----Logon--------------------------------------------------------------------
 class CPU_Logon : public CSrvPacketTcp
 {
@@ -219,7 +232,7 @@ public:
 class CPU_SendCookie : public CSrvPacketTcp
 {
 public:
-  CPU_SendCookie(const char *, int len);
+  CPU_SendCookie(const char *, int len, unsigned short nService = 0);
   virtual ~CPU_SendCookie();
 };
 
@@ -237,10 +250,23 @@ protected:
 class CPU_GenericFamily : public CPU_CommonFamily
 {
 public:
-  CPU_GenericFamily(unsigned short Family, unsigned short SubType);
+  CPU_GenericFamily(unsigned short Family, unsigned short SubType,
+                    unsigned short nService = 0);
   virtual ~CPU_GenericFamily();
 };
 
+class CPU_RequestLogonSalt : public CPU_CommonFamily
+{
+public:
+  CPU_RequestLogonSalt(const std::string &);
+};
+
+class CPU_NewLogon : public CPU_CommonFamily
+{
+public:
+  CPU_NewLogon(const char *_szPassword, const char *_szUin, const char *_szMD5Salt);
+};
+
 #if ICQ_VERSION == 2 || ICQ_VERSION == 6
 //-----Register----------------------------------------------------------------
 // Doesn't actually descend from CPacketUdp in version 2 but we keep the
@@ -256,7 +282,7 @@ public:
   virtual unsigned short Command()  { return m_nCommand; }
   virtual unsigned short SubCommand() { return 0; }
 protected:
-  virtual unsigned long getSize()	{ return 1; }
+  virtual unsigned long getSize()       { return 1; }
 
   /* 02 00 FC 03 01 00 02 00 04 00 65 66 67 00 72 00 00 00 00 00 00 00 */
   unsigned short m_nVersion;
@@ -308,11 +334,20 @@ public:
   virtual ~CPU_SendVerification();
 };
 
+//-----ListRequestRights-----------------------------------------------------
+class CPU_ListRequestRights : public CPU_CommonFamily
+{
+public:
+  CPU_ListRequestRights();
+};
+
 //-----ImICQ------------------------------------------------------------------
 class CPU_ImICQ : public CPU_CommonFamily
 {
 public:
-    CPU_ImICQ();
+  CPU_ImICQ();
+  CPU_ImICQ(unsigned short VerArray[][2], unsigned short NumVer,
+            unsigned short nService);
 };
 
 //-----ICQMode------------------------------------------------------------------
@@ -333,7 +368,7 @@ public:
 class CPU_RateAck : public CPU_CommonFamily
 {
 public:
-  CPU_RateAck();
+  CPU_RateAck(unsigned short nService = 0);
 };
 
 //-----GenericUinList------------------------------------------------------------
@@ -341,7 +376,7 @@ class CPU_GenericUinList : public CPU_CommonFamily
 {
 public:
    CPU_GenericUinList(const char *szId, unsigned short Family, unsigned short Subtype);
-   CPU_GenericUinList(UserStringList &, unsigned short, unsigned short);
+   CPU_GenericUinList(const StringList& users, unsigned short, unsigned short);
    CPU_GenericUinList(unsigned long _nUin, unsigned short Family, unsigned short Subtype);
 };
 
@@ -367,14 +402,14 @@ public:
 class CPU_ExportToServerList : public CPU_CommonFamily
 {
 public:
-  CPU_ExportToServerList(UserStringList &, unsigned short);
+  CPU_ExportToServerList(const StringList& users, unsigned short);
 };
 
 //-----ExportGroupsToServerList-------------------------------------------------
 class CPU_ExportGroupsToServerList : public CPU_CommonFamily
 {
 public:
-  CPU_ExportGroupsToServerList(GroupList &);
+  CPU_ExportGroupsToServerList(const GroupNameMap& groups);
 };
 
 //-----AddPrivacyInfo-----------------------------------------------------------
@@ -419,7 +454,7 @@ public:
 class CPU_ClearServerList : public CPU_CommonFamily
 {
 public:
-  CPU_ClearServerList(UserStringList &, unsigned short);
+  CPU_ClearServerList(const StringList& users, unsigned short);
 };
 
 //-----UpdateToServerList-------------------------------------------------------
@@ -487,6 +522,8 @@ class CPU_ClientReady : public CPU_CommonFamily
 {
 public:
    CPU_ClientReady();
+   CPU_ClientReady(unsigned short VerArray[][4], unsigned short NumVer,
+                   unsigned short nService);
 };
 
 //-----ClientAckNameInfo--------------------------------------------------------
@@ -665,13 +702,13 @@ protected:
 class CPU_Type2Message : public CPU_CommonFamily
 {
 public:
-  CPU_Type2Message(ICQUser *u, bool _bAck, bool _bDirectInfo, const char *cap,
+  CPU_Type2Message(const ICQUser* u, bool _bAck, bool _bDirectInfo, const char *cap,
                    unsigned long nMsgID1 = 0,
                    unsigned long nMsgID2 = 0);
 protected:
   void InitBuffer();
 
-  ICQUser *m_pUser;
+  const ICQUser* m_pUser;
   bool m_bAck;
   bool m_bDirectInfo;
   unsigned long m_nMsgID[2];
@@ -683,7 +720,7 @@ protected:
 class CPU_ReverseConnect : public CPU_Type2Message
 {
 public:
-  CPU_ReverseConnect(ICQUser *u, unsigned long nLocalIP,
+  CPU_ReverseConnect(const ICQUser* u, unsigned long nLocalIP,
                      unsigned short nLocalPort, unsigned short nRemotePort);
 };
 
@@ -691,7 +728,7 @@ public:
 class CPU_ReverseConnectFailed : public CPU_CommonFamily
 {
 public:
-  CPU_ReverseConnectFailed(unsigned long nUin, unsigned long nMsgID1,
+  CPU_ReverseConnectFailed(const char* id, unsigned long nMsgID1,
                            unsigned long nMsgID2, unsigned short nFailedPort,
                            unsigned short nOurPort, unsigned long nConnectID);
 };
@@ -700,7 +737,7 @@ public:
 class CPU_PluginMessage : public CPU_Type2Message
 {
 public:
-  CPU_PluginMessage(ICQUser *u, bool bAck, const char *PluginGUID,
+  CPU_PluginMessage(const ICQUser* u, bool bAck, const char *PluginGUID,
                     unsigned long nMsgID1 = 0, unsigned long nMsgID2 = 0);
 
 protected:
@@ -713,7 +750,7 @@ protected:
 class CPU_InfoPluginReq : public CPU_PluginMessage
 {
 public:
-  CPU_InfoPluginReq(ICQUser *u, const char *GUID, unsigned long nTime);
+  CPU_InfoPluginReq(const ICQUser* u, const char *GUID, unsigned long nTime);
   virtual const char *RequestGUID() { return m_ReqGUID; }
   virtual unsigned short ExtraInfo() { return ServerInfoPluginRequest; }
 
@@ -725,7 +762,7 @@ protected:
 class CPU_StatusPluginReq : public CPU_PluginMessage
 {
 public:
-  CPU_StatusPluginReq(ICQUser *u, const char *GUID, unsigned long nTime);
+  CPU_StatusPluginReq(const ICQUser* u, const char *GUID, unsigned long nTime);
   virtual unsigned short ExtraInfo() { return ServerStatusPluginRequest; }
   virtual const char *RequestGUID() { return m_ReqGUID; }
 
@@ -737,7 +774,7 @@ protected:
 class CPU_AdvancedMessage : public CPU_Type2Message
 {
 public:
-  CPU_AdvancedMessage(ICQUser *u, unsigned short _nMsgType,
+  CPU_AdvancedMessage(const ICQUser* u, unsigned short _nMsgType,
                       unsigned short _nMsgFlags, bool _bAck,
                       unsigned short _nSequence,
                       unsigned long nID1 = 0,
@@ -755,14 +792,14 @@ class CPU_ChatRequest : public CPU_AdvancedMessage
 {
 public:
   CPU_ChatRequest(char *szReason, const char *szChatUsers, unsigned short nPort,
-                  unsigned short nLevel, ICQUser *pUser, bool bICBM);
+      unsigned short nLevel, const ICQUser* pUser, bool bICBM);
 };
 
 //-----FileTransfer------------------------------------------------------------
 class CPU_FileTransfer : public CPU_AdvancedMessage, public CPX_FileTransfer
 {
 public:
-  CPU_FileTransfer(ICQUser *, ConstFileList &lFileList, const char *_szFile,
+  CPU_FileTransfer(const ICQUser* u, ConstFileList &lFileList, const char *_szFile,
                    const char *_szDesc, unsigned short nLevel, bool bICBM);
 };
 
@@ -770,21 +807,21 @@ public:
 class CPU_NoManager : public CPU_CommonFamily
 {
 public:
-    CPU_NoManager(ICQUser *u, unsigned long nMsgID1, unsigned long nMsgID2);
+    CPU_NoManager(const ICQUser* u, unsigned long nMsgID1, unsigned long nMsgID2);
 };
 
 //-----AckThroughServer--------------------------------------------------------
 class CPU_AckThroughServer : public CPU_CommonFamily
 {
 public:
-    CPU_AckThroughServer(ICQUser *u, unsigned long msgid1,
+    CPU_AckThroughServer(const ICQUser* u, unsigned long msgid1,
                          unsigned long msgid2, unsigned short sequence,
                          unsigned short msgType, bool bAccept,
                          unsigned short nLevel, const char *GUID);
 protected:
   void InitBuffer();
 
-  unsigned long m_nUin, m_nMsgID[2];
+  unsigned long m_nMsgID[2];
   unsigned short m_nSequence, m_nMsgType, m_nStatus, m_nUinLen, m_nLevel;
   char m_szUin[13];
   char *m_szMessage;
@@ -795,7 +832,7 @@ protected:
 class CPU_AckGeneral : public CPU_AckThroughServer
 {
 public:
-  CPU_AckGeneral(ICQUser *u, unsigned long nMsgID1,
+  CPU_AckGeneral(const ICQUser* u, unsigned long nMsgID1,
                  unsigned long nMsgID2, unsigned short nSequence,
                  unsigned short nMsgType, bool bAccept, unsigned short nLevel);
 };
@@ -804,7 +841,7 @@ public:
 class CPU_AckFileAccept : public CPU_AdvancedMessage
 {
 public:
-  CPU_AckFileAccept(ICQUser *u, unsigned long nMsgID[],
+  CPU_AckFileAccept(const ICQUser* u, const unsigned long nMsgID[],
                     unsigned short nSequence, unsigned short nPort,
                     const char *szDesc, const char *szFile,
                     unsigned long nFileSize);
@@ -814,7 +851,7 @@ public:
 class CPU_AckFileRefuse : public CPU_AckThroughServer
 {
 public:
-  CPU_AckFileRefuse(ICQUser *u, unsigned long nMsgID[],
+  CPU_AckFileRefuse(const ICQUser* u, const unsigned long nMsgID[],
                     unsigned short nSequence, const char *msg);
 };
 
@@ -822,7 +859,7 @@ public:
 class CPU_AckChatAccept : public CPU_AdvancedMessage
 {
 public:
-  CPU_AckChatAccept(ICQUser *u, const char *szClients, unsigned long nMsgID[],
+  CPU_AckChatAccept(const ICQUser* u, const char *szClients, const unsigned long nMsgID[],
                     unsigned short nSequence, unsigned short nPort);
 };
 
@@ -830,7 +867,7 @@ public:
 class CPU_AckChatRefuse : public CPU_AckThroughServer
 {
 public:
-  CPU_AckChatRefuse(ICQUser *u, unsigned long nMsgID[],
+  CPU_AckChatRefuse(const ICQUser* u, const unsigned long nMsgID[],
                     unsigned short nSequence, const char *msg);
 };
 
@@ -838,7 +875,7 @@ public:
 class CPU_PluginError : public CPU_AckThroughServer
 {
 public:
-  CPU_PluginError(ICQUser *u, unsigned long nMsgID1, unsigned long nMsgID2,
+  CPU_PluginError(const ICQUser* u, unsigned long nMsgID1, unsigned long nMsgID2,
                   unsigned short nSequence, const char *GUID);
 };
 
@@ -846,7 +883,7 @@ public:
 class CPU_InfoPluginListResp : public CPU_AckThroughServer
 {
 public:
-  CPU_InfoPluginListResp(ICQUser *u, unsigned long nMsgID1,
+  CPU_InfoPluginListResp(const ICQUser* u, unsigned long nMsgID1,
                          unsigned long nMsgID2, unsigned short nSequence);
 };
 
@@ -854,7 +891,7 @@ public:
 class CPU_InfoPhoneBookResp : public CPU_AckThroughServer
 {
 public:
-  CPU_InfoPhoneBookResp(ICQUser *u, unsigned long nMsgID1,
+  CPU_InfoPhoneBookResp(const ICQUser* u, unsigned long nMsgID1,
                         unsigned long nMsgID2, unsigned short nSequence);
 };
 
@@ -862,7 +899,7 @@ public:
 class CPU_InfoPictureResp : public CPU_AckThroughServer
 {
 public:
-  CPU_InfoPictureResp(ICQUser *u, unsigned long nMsgID1,
+  CPU_InfoPictureResp(const ICQUser* u, unsigned long nMsgID1,
                       unsigned long nMsgID2, unsigned short nSequence);
 };
 
@@ -870,7 +907,7 @@ public:
 class CPU_StatusPluginListResp : public CPU_AckThroughServer
 {
 public:
-  CPU_StatusPluginListResp(ICQUser *u, unsigned long nMsgID1,
+  CPU_StatusPluginListResp(const ICQUser* u, unsigned long nMsgID1,
                            unsigned long nMsgID2, unsigned short nSequence);
 };
 
@@ -878,7 +915,7 @@ public:
 class CPU_StatusPluginResp : public CPU_AckThroughServer
 {
 public:
-  CPU_StatusPluginResp(ICQUser *u, unsigned long nMsgID1,
+  CPU_StatusPluginResp(const ICQUser* u, unsigned long nMsgID1,
                        unsigned long nMsgID2, unsigned short nSequence,
                        unsigned long nStatus);
 };
@@ -908,17 +945,14 @@ protected:
 class CPU_RequestAuth : public CPU_CommonFamily
 {
 public:
-  CPU_RequestAuth(unsigned long, const char *);
+  CPU_RequestAuth(const char* id, const char *);
 };
 
 //-----Authorize----------------------------------------------------------------
-class CPU_Authorize : public CPacketUdp
+class CPU_Authorize : public CPU_CommonFamily
 {
 public:
-   CPU_Authorize(unsigned long _nAuthorizeUin);
-protected:
-   /* 02 00 56 04 05 00 50 A5 82 00 A7 B8 19 00 08 00 01 00 00 */
-   unsigned long m_nAuthorizeUin;
+   CPU_Authorize(const char *szId);
 };
 
 
@@ -1129,6 +1163,21 @@ public:
   CPU_RequestInfo(const char *szId);
 };
 
+//-----RequestBuddyIcon------------------------------------------------------
+class CPU_RequestBuddyIcon : public CPU_CommonFamily
+{
+public:
+  CPU_RequestBuddyIcon(const char *szId, unsigned short _nBuddyIconType,
+                       char _nBuddyIconHashType, const char *_szBuddyIconHash,
+                       unsigned short nService);
+};
+
+class CPU_RequestService : public CPU_CommonFamily
+{
+public:
+  CPU_RequestService(unsigned short nFam);
+};
+
 //-----AIMFetchAwayMessage--------------------------------------------------
 class CPU_AIMFetchAwayMessage : public CPU_CommonFamily
 {
@@ -1142,7 +1191,7 @@ class CPU_SetPassword : public CPU_CommonFamily
 public:
   CPU_SetPassword(const char *szPassword);
   virtual unsigned short SubCommand()  { return m_nMetaCommand; }
-  unsigned long Uin()	{ return 0; }
+  unsigned long Uin()           { return 0; }
 protected:
   unsigned short m_nMetaCommand;
   char *m_szPassword;
