@@ -1,17 +1,14 @@
+#ifndef PROXY_H
+#define PROXY_H
+
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 
-#include <sys/types.h>
-#include <sys/socket.h>
 #include <netinet/in.h>
-#include <unistd.h>
-#include <strings.h>
-
-#include "licq_constants.h"
 
 // proxy types
-#define	PROXY_TYPE_HTTP	1
+#define PROXY_TYPE_HTTP 1
 
 typedef enum ProxyError_et_
 {
@@ -68,3 +65,5 @@ public:
   bool HTTPInitProxy();
   bool HTTPOpenProxyConnection(const char *_szRemoteName, unsigned short _nRemotePort);
 };
+
+#endif
