@@ -2,18 +2,19 @@
 #include "config.h"
 #endif
 
+#include <cctype>
+#include <climits>
+#include <cstdio>
+#include <cstdlib>
+#include <cstring>
 #include <sys/select.h>
 #include <sys/time.h>
 #include <sys/types.h>
 #include <unistd.h>
-#include <string.h>
-#include <ctype.h>
-#include <stdio.h>
-#include <stdlib.h>
 #include <sys/wait.h>
 #include <signal.h>
 #ifdef HAVE_ERRNO_H
-#include <errno.h>
+#include <cerrno>
 #else
 extern int errno;
 #endif
@@ -29,7 +30,6 @@ extern int errno;
 #include "licq_icqd.h"
 #include "licq_file.h"
 #include "licq_user.h"
-#include "licq_socket.h"
 #include "licq_constants.h"
 
 extern "C" { const char *LP_Version(); }
@@ -42,7 +42,6 @@ const unsigned short SUBJ_CHARS = 20;
  *-------------------------------------------------------------------------*/
 CLicqAutoReply::CLicqAutoReply(bool _bEnable, bool _bDelete, char *_szStatus)
 {
-  tcp = new TCPSocket;
   m_bExit = false;
   m_bEnabled = _bEnable;
   m_bDelete = _bDelete;
@@ -55,7 +54,6 @@ CLicqAutoReply::CLicqAutoReply(bool _bEnable, bool _bDelete, char *_szStatus)
  *-------------------------------------------------------------------------*/
 CLicqAutoReply::~CLicqAutoReply()
 {
-  delete tcp;
 }
 
 /*---------------------------------------------------------------------------
@@ -96,9 +94,9 @@ int CLicqAutoReply::Run(CICQDaemon *_licqDaemon)
   if (m_szStatus != NULL)
   {
     unsigned long s = StringToStatus(m_szStatus);
-    ICQOwner *o = gUserManager.FetchOwner(LOCK_R);
+    ICQOwner* o = gUserManager.FetchOwner(LICQ_PPID, LOCK_R);
     bool b = o->StatusOffline();
-    gUserManager.DropOwner();
+    gUserManager.DropOwner(o);
     if (s == INT_MAX)
       gLog.Warn("%sInvalid startup status.\n", L_AUTOREPxSTR);
     else
@@ -210,7 +208,7 @@ void CLicqAutoReply::ProcessSignal(CICQSignal *s)
  *-------------------------------------------------------------------------*/
 void CLicqAutoReply::ProcessEvent(ICQEvent *e)
 {
-  CUserEvent *user_event;
+  const CUserEvent* user_event;
 
   if (e->Result() != EVENT_ACKED)
   {
@@ -219,7 +217,7 @@ void CLicqAutoReply::ProcessEvent(ICQEvent *e)
          e->SubCommand() != ICQ_CMDxSUB_FILE))
     {
 	    user_event = e->UserEvent();
-      licqDaemon->icqSendMessage(e->Uin(), user_event->Text(), !m_bSendThroughServer,
+      licqDaemon->icqSendMessage(e->Id(), user_event->Text(), !m_bSendThroughServer,
         ICQ_TCPxMSG_URGENT); //urgent, because, hey, he asked us, right?
     }
   }
