@@ -2,7 +2,11 @@
 #include "config.h"
 #endif
 
+#include <sys/select.h>
+#include <sys/time.h>
+#include <sys/types.h>
 #include <unistd.h>
+
 #include <cstring>
 #include <cctype>
 #include <vector>
@@ -86,35 +90,35 @@ struct SVariable aVariables[NUM_VARIABLES] =
 const unsigned short NUM_COLORMAPS = 15;
 const struct SColorMap aColorMaps[NUM_COLORMAPS] =
   {
-    { "green", 24, A_NORMAL }
+    { "green", COLOR_GREEN, A_NORMAL }
     ,          // 0
-    { "red", 16, A_NORMAL }
+    { "red", COLOR_RED, A_NORMAL }
     ,              // 1
-    { "cyan", 56, A_NORMAL }
+    { "cyan", COLOR_CYAN, A_NORMAL }
     ,            // 2
-    { "white", 8, A_NORMAL }
+    { "white", COLOR_WHITE, A_NORMAL }
     ,          // 3
-    { "magenta", 48, A_NORMAL }
+    { "magenta", COLOR_MAGENTA, A_NORMAL }
     ,      // 4
-    { "blue", 40, A_NORMAL }
+    { "blue", COLOR_BLUE, A_NORMAL }
     ,            // 5
-    { "yellow", 32, A_NORMAL }
+    { "yellow", COLOR_YELLOW, A_NORMAL }
     ,        // 6
-    { "black", 8, A_NORMAL }
+    { "black", COLOR_BLACK, A_NORMAL }
     ,          // 7
-    { "bright_green", 24, A_BOLD }
+    { "bright_green", COLOR_GREEN, A_BOLD }
     ,     // 8
-    { "bright_red", 16, A_BOLD }
+    { "bright_red", COLOR_RED, A_BOLD }
     ,         // 9
-    { "bright_cyan", 56, A_BOLD }
+    { "bright_cyan", COLOR_CYAN, A_BOLD }
     ,       // 10
-    { "bright_white", 8, A_BOLD }
+    { "bright_white", COLOR_WHITE, A_BOLD }
     ,     // 11
-    { "bright_magenta", 48, A_BOLD }
+    { "bright_magenta", COLOR_MAGENTA, A_BOLD }
     , // 12
-    { "bright_blue", 40, A_BOLD }
+    { "bright_blue", COLOR_BLUE, A_BOLD }
     ,       // 13
-    { "bright_yellow", 32, A_BOLD }    // 14
+    { "bright_yellow", COLOR_YELLOW, A_BOLD }    // 14
   };
 
 const char MLE_HELP[] =
@@ -123,7 +127,7 @@ const char MLE_HELP[] =
 /*---------------------------------------------------------------------------
  * CLicqConsole::Constructor
  *-------------------------------------------------------------------------*/
-CLicqConsole::CLicqConsole(int argc, char **argv)
+CLicqConsole::CLicqConsole(int /* argc */, char** /* argv */)
 {
   CWindow::StartScreen();
 
@@ -161,6 +165,7 @@ CLicqConsole::CLicqConsole(int argc, char **argv)
   licqConf.ReadStr("AwayFormat", m_szAwayFormat, "%a [%S]");
   licqConf.ReadStr("OfflineFormat", m_szOfflineFormat, "%a");
   licqConf.ReadStr("CommandCharacter", m_szCommandChar, "/");
+  licqConf.ReadNum("Backspace", m_nBackspace, KEY_BACKSPACE);
 
   if (licqConf.SetSection("macros"))
   {
@@ -209,6 +214,7 @@ CLicqConsole::CLicqConsole(int argc, char **argv)
   aVariables[i++].pData = m_szCommandChar;
 
   m_bExit = false;
+  cdkUserList = 0;
 }
 
 
@@ -243,18 +249,17 @@ int CLicqConsole::Run(CICQDaemon *_licqDaemon)
 {
   // Register with the daemon, we want to receive all signals
   m_nPipe = _licqDaemon->RegisterPlugin(SIGNAL_ALL);
-  m_bExit = false; 
-  _licqDaemon->AddProtocolPlugins();
+  m_bExit = false;
   licqDaemon = _licqDaemon;
-  m_nCurrentGroup = gUserManager.DefaultGroup();
 
   // Create the windows
   for (unsigned short i = 0; i <= MAX_CON; i++)
   {
-    winCon[i] = new CWindow(LINES - 5, COLS - USER_WIN_WIDTH - 1, 2, USER_WIN_WIDTH,
+    winCon[i] = new CWindow(LINES - 5, COLS - USER_WIN_WIDTH - 1, 2, USER_WIN_WIDTH + 1,
                             SCROLLBACK_BUFFER, true);
     scrollok(winCon[i]->Win(), true);
     winCon[i]->fProcessInput = &CLicqConsole::InputCommand;
+    winCon[i]->data = NULL;
   }
   winCon[0]->fProcessInput = &CLicqConsole::InputLogWindow;
   winStatus = new CWindow(2, COLS, LINES - 3, 0, false);
@@ -290,14 +295,14 @@ int CLicqConsole::Run(CICQDaemon *_licqDaemon)
   }
   else
   {
-    ICQOwner *o = gUserManager.FetchOwner(LOCK_R);
+    ICQOwner *o = gUserManager.FetchOwner(LICQ_PPID, LOCK_R);
     if (o->Password()[0] == '\0')
     {
-      gUserManager.DropOwner();
+      gUserManager.DropOwner(o);
       UserSelect();
     }
     else
-      gUserManager.DropOwner();
+      gUserManager.DropOwner(o);
   }
 
   //fd_set fdSet;
@@ -352,7 +357,13 @@ int CLicqConsole::Run(CICQDaemon *_licqDaemon)
       {
         if (FD_ISSET((*iter)->Pipe(), &fdSet))
         {
-          ProcessFile(iter);
+          if (!ProcessFile(*iter))
+          {
+            delete *iter;
+#undef erase
+            m_lFileStat.erase(iter);
+          }
+
           break;
         }
       }
@@ -393,6 +404,7 @@ void CLicqConsole::DoneOptions()
   licqConf.WriteStr("AwayFormat", m_szAwayFormat);
   licqConf.WriteStr("OfflineFormat", m_szOfflineFormat);
   licqConf.WriteStr("CommandCharacter", m_szCommandChar);
+  licqConf.WriteNum("Backspace", (unsigned long)m_nBackspace);
 
   licqConf.SetSection("macros");
   char sz[32];
@@ -422,29 +434,29 @@ void CLicqConsole::ProcessLog()
   switch (log->NextLogType())
   {
   case L_WARN:
-    cp = 32;
+    cp = COLOR_YELLOW;
     break;
   case L_ERROR:
-    cp = 16;
+    cp = COLOR_RED;
     break;
   case L_PACKET:
-    cp = 40;
+    cp = COLOR_BLUE;
     break;
   case L_UNKNOWN:
-    cp = 48;
+    cp = COLOR_MAGENTA;
     break;
   case L_INFO:
   default:
-    cp = 8;
+    cp = COLOR_WHITE;
     break;
   }
   char *p = log->NextLogMsg();
   char *l = &p[LOG_PREFIX_OFFSET];
   p[LOG_PREFIX_OFFSET - 1] = '\0';
-  winLog->wprintf("%C%s %C%s", 24, p, cp, l);
+  winLog->wprintf("%C%s %C%s", COLOR_GREEN, p, cp, l);
   if (log->NextLogType() == L_ERROR)
   {
-    winMain->wprintf("%C%s %C%s", 24, p, cp, l);
+    winMain->wprintf("%C%s %C%s", COLOR_GREEN, p, cp, l);
     winMain->RefreshWin();
   }
   log->ClearLog();
@@ -504,8 +516,8 @@ void CLicqConsole::ProcessSignal(CICQSignal *s)
     {
       for (unsigned short i = 0; i < MAX_CON; i++)
       {
-        if (s->Uin() == winCon[i]->nLastUin)
-          winCon[i]->nLastUin = 0;
+        if (s->Id() == winCon[i]->myLastId)
+          winCon[i]->myLastId.clear();
       }
     }
     PrintStatus();
@@ -514,18 +526,20 @@ void CLicqConsole::ProcessSignal(CICQSignal *s)
     break;
   case SIGNAL_UPDATExUSER:
     {
-      if (gUserManager.FindOwner(s->Id(), s->PPID()) != NULL && s->SubSignal() == USER_STATUS
+      if ((gUserManager.FindOwner(s->Id(), s->PPID()) != NULL && s->SubSignal() == USER_STATUS)
           || s->SubSignal() == USER_EVENTS)
         PrintStatus();
       ICQUser *u = gUserManager.FetchUser(s->Id(), s->PPID(), LOCK_R);
       if (u != NULL)
       {
-        if (u->GetInGroup(m_nGroupType, m_nCurrentGroup))
+        bool isInGroup = u->GetInGroup(m_nGroupType, m_nCurrentGroup);
+        gUserManager.DropUser(u);
+
+        if (isInGroup || (m_nGroupType == GROUPS_USER && m_nCurrentGroup == 0))
         {
           CreateUserList();
           PrintUsers();
         }
-        gUserManager.DropUser(u);
       }
 
       break;
@@ -535,7 +549,7 @@ void CLicqConsole::ProcessSignal(CICQSignal *s)
     PrintStatus();
     break;
   case SIGNAL_ADDxSERVERxLIST:
-    licqDaemon->icqRenameUser(s->Uin());
+    licqDaemon->icqRenameUser(s->Id());
     break;
   case SIGNAL_NEWxPROTO_PLUGIN:
     //ignore for now
@@ -554,7 +568,7 @@ void CLicqConsole::ProcessSignal(CICQSignal *s)
 /*---------------------------------------------------------------------------
  * CLicqConsole::AddEventTag
  *-------------------------------------------------------------------------*/
-void CLicqConsole::AddEventTag(char *_szId, unsigned long _nPPID, unsigned long _nEventTag)
+void CLicqConsole::AddEventTag(const char* _szId, unsigned long _nPPID, unsigned long _nEventTag)
 {
   if (!_szId || !_nPPID || !_nEventTag)
     return;
@@ -564,7 +578,7 @@ void CLicqConsole::AddEventTag(char *_szId, unsigned long _nPPID, unsigned long
   for (i = 1; i <= MAX_CON; i++)
   {
     data = (CData *)winCon[i]->data;
-    if (strcmp(data->szId, _szId) == 0 && data->nPPID == _nPPID)
+    if (data && strcmp(data->szId, _szId) == 0 && data->nPPID == _nPPID)
     {
       winCon[i]->event = _nEventTag;
       break;
@@ -618,14 +632,14 @@ void CLicqConsole::ProcessEvent(ICQEvent *e)
   case MAKESNAC(ICQ_SNACxFAM_BUDDY, ICQ_SNACxBDY_ADDxTOxLIST):
   case ICQ_CMDxSND_LOGON:
     if (e->Result() != EVENT_SUCCESS)
-      winMain->wprintf("%CLogon failed.  See the log console for details.\n", 16);
+      winMain->wprintf("%CLogon failed.  See the log console for details.\n", COLOR_RED);
     break;
 
   case ICQ_CMDxSND_REGISTERxUSER:
     // Needs to be better dealt with...
     // How's this then?
-    winMain->wprintf("Registration complete!\nYour UIN is %ld\n",
-                     gUserManager.OwnerUin());
+      winMain->wprintf("Registration complete!\nYour UIN is %s\n",
+          gUserManager.OwnerId(LICQ_PPID).c_str());
     winMain->fProcessInput = &CLicqConsole::InputCommand;
     PrintStatus();
     break;
@@ -641,36 +655,36 @@ void CLicqConsole::ProcessEvent(ICQEvent *e)
 /*---------------------------------------------------------------------------
  * CLicqConsole::ProcessFile
  *-------------------------------------------------------------------------*/
-void CLicqConsole::ProcessFile(list<CFileTransferManager *>::iterator iter)
+bool CLicqConsole::ProcessFile(CFileTransferManager *ftman)
 {
   char buf[32];
   bool bCloseFT = false;
-  read((*iter)->Pipe(), buf, 32);
+  read(ftman->Pipe(), buf, 32);
 
   CFileTransferEvent *e = NULL;
 
-  while ((e = (*iter)->PopFileTransferEvent()) != NULL)
+  while ((e = ftman->PopFileTransferEvent()) != NULL)
   {
     switch(e->Command())
     {
     case FT_ERRORxCONNECT:
       winMain->wprintf("%C%AFile transfer could not connect.  See network "
                        "window for details.%C%Z\n",
-                       16, A_BOLD, 8, A_BOLD);
+                       COLOR_RED, A_BOLD, COLOR_WHITE, A_BOLD);
       bCloseFT = true;
       break;
 
     case FT_ERRORxBIND:
       winMain->wprintf("%C%AFile transfer could not bind to a port.  See "
                        "network window for details.%C%Z\n",
-                       16, A_BOLD, 8, A_BOLD);
+                       COLOR_RED, A_BOLD, COLOR_WHITE, A_BOLD);
       bCloseFT = true;
       break;
 
     case FT_ERRORxRESOURCES:
       winMain->wprintf("%C%AFile transfer unable to create new thread.  See "
                        "network window for details.%C%Z\n",
-                       16, A_BOLD, 8, A_BOLD);
+                       COLOR_RED, A_BOLD, COLOR_WHITE, A_BOLD);
       bCloseFT = true;
       break;
            
@@ -679,41 +693,44 @@ void CLicqConsole::ProcessFile(list<CFileTransferManager *>::iterator iter)
 
     case FT_DONExBATCH:
       winMain->wprintf("%C%AFile transfer successfuly finished.%C%Z\n",
-                       24, A_BOLD, 8, A_BOLD);
+                       COLOR_GREEN, A_BOLD, COLOR_WHITE, A_BOLD);
       bCloseFT = true;
       break;
 
     case FT_ERRORxCLOSED:
       winMain->wprintf("%C%AFile transfer closed.%C%Z\n",
-                       16, A_BOLD, 8, A_BOLD);
+                       COLOR_RED, A_BOLD, COLOR_WHITE, A_BOLD);
       bCloseFT = true;
       break;
 
     case FT_ERRORxFILE:
       winMain->wprintf("%C%AFile transfer I/O error.%C%Z\n",
-                       16, A_BOLD, 8, A_BOLD);
+                       COLOR_RED, A_BOLD, COLOR_WHITE, A_BOLD);
       bCloseFT = true;
       break;
 
     case FT_ERRORxHANDSHAKE:
       winMain->wprintf("%C%AFile transfer handshake error.%C%Z\n",
-                       16, A_BOLD, 8, A_BOLD);
+                       COLOR_RED, A_BOLD, COLOR_WHITE, A_BOLD);
       bCloseFT = true;
       break;
+
+    case FT_CONFIRMxFILE:
+      ftman->StartReceivingFile(const_cast<char *>(ftman->FileName()));
+      break;
     }
 
     if (bCloseFT)
     {
-      (*iter)->CloseFileTransfer();
-      delete *iter;
-#undef erase
-      m_lFileStat.erase(iter);
+      ftman->CloseFileTransfer();
       delete e;
-      return;
+      return false;
     }
 
     delete e;
   }
+
+  return true;
 }
 
 
@@ -744,7 +761,7 @@ void CLicqConsole::ProcessDoneEvent(ICQEvent *e)
 
   if (e == NULL)
   {
-    win->wprintf("%A%Cerror\n", A_BOLD, 16);
+    win->wprintf("%A%Cerror\n", A_BOLD, COLOR_RED);
   }
   else
   {
@@ -780,7 +797,7 @@ void CLicqConsole::ProcessDoneEvent(ICQEvent *e)
          e->SubCommand() == ICQ_CMDxSUB_FILE) )
     {
       win->wprintf("%C%ADirect send failed, send through server (y/N)? %C%Z",
-                   m_cColorQuery->nColor, m_cColorQuery->nAttr, 8,
+                   m_cColorQuery->nColor, m_cColorQuery->nAttr, COLOR_WHITE,
                    A_BOLD);
       win->state = STATE_QUERY;
       win->data->nPos = 0;
@@ -794,24 +811,24 @@ void CLicqConsole::ProcessDoneEvent(ICQEvent *e)
     case ICQ_CMDxTCP_START:
       {
         ICQUser *u = NULL;
-        CUserEvent *ue = e->UserEvent();
+        const CUserEvent* ue = e->UserEvent();
         if (e->SubResult() == ICQ_TCPxACK_RETURN)
         {
-          u = gUserManager.FetchUser(e->Uin(), LOCK_R);
+          u = gUserManager.FetchUser(e->Id(), e->PPID(), LOCK_R);
           win->wprintf("%s is in %s mode:\n%s\n[Send \"urgent\" ('.u') to ignore]\n",
                        u->GetAlias(), u->StatusStr(), u->AutoResponse());
           gUserManager.DropUser(u);
         }
         else if (e->SubResult() == ICQ_TCPxACK_REFUSE)
         {
-          u = gUserManager.FetchUser(e->Uin(), LOCK_R);
+          u = gUserManager.FetchUser(e->Id(), e->PPID(), LOCK_R);
           win->wprintf("%s refused %s.\n",
                        u->GetAlias(), ue->Description());
           gUserManager.DropUser(u);
         }
         else if(e->SubCommand() == ICQ_CMDxSUB_FILE)
         {
-          CExtendedAck *ea = e->ExtendedAck();
+          const CExtendedAck* ea = e->ExtendedAck();
 
           if( ea == NULL || ue == NULL)
           {
@@ -821,7 +838,7 @@ void CLicqConsole::ProcessDoneEvent(ICQEvent *e)
 
           if(!ea->Accepted())
           {
-            u = gUserManager.FetchUser(e->Uin(), LOCK_R);
+            u = gUserManager.FetchUser(e->Id(), e->PPID(), LOCK_R);
             win->wprintf("%s refused file: %s\n",
                          u->GetAlias(), ea->Response());
             gUserManager.DropUser(u);
@@ -831,7 +848,7 @@ void CLicqConsole::ProcessDoneEvent(ICQEvent *e)
           {
             // For now don't check for a chat subcommand..
             // Invoke a file transfer manager here
-            CEventFile *f = (CEventFile *)ue;
+            const CEventFile* f = dynamic_cast<const CEventFile *>(ue);
             CFileTransferManager *ftman = new CFileTransferManager(licqDaemon,
                                           e->Uin());
             m_lFileStat.push_back(ftman);
@@ -888,7 +905,7 @@ void CLicqConsole::ProcessDoneEvent(ICQEvent *e)
             } // if file or chat*/
         else
         {
-          u = gUserManager.FetchUser(e->Uin(), LOCK_R);
+          u = gUserManager.FetchUser(e->Id(), e->PPID(), LOCK_R);
           if (u != NULL && u->Away() && u->ShowAwayMsg())
           {
             win->wprintf("%s\n", u->AutoResponse());
@@ -948,7 +965,7 @@ void CLicqConsole::ProcessDoneSearch(ICQEvent *e)
   if (e->SearchAck() != NULL && e->SearchAck()->Uin() != 0)
   {
     win->wprintf("%C%s%A,%Z %s %s %A(%Z%s%A) -%Z %lu %A(%Z%s%A)\n",
-                 8,
+                 COLOR_WHITE,
                  e->SearchAck()->Alias(),
                  A_BOLD, A_BOLD,
                  e->SearchAck()->FirstName(),
@@ -973,7 +990,7 @@ void CLicqConsole::ProcessDoneSearch(ICQEvent *e)
     {
       win->wprintf("%A%CSearch complete.\n", m_cColorInfo->nAttr, m_cColorInfo->nColor);
     }
-    else if (e->SearchAck()->More() == -1)
+    else if (static_cast<long>(e->SearchAck()->More()) == -1)
     {
       win->wprintf("%A%CSearch complete.  More users found, narrow search.\n",
                    m_cColorInfo->nAttr, m_cColorInfo->nColor);
@@ -986,7 +1003,7 @@ void CLicqConsole::ProcessDoneSearch(ICQEvent *e)
   }
   else
   {
-    win->wprintf("%CSearch failed.\n", 16);
+    win->wprintf("%CSearch failed.\n", COLOR_RED);
   }
 
   win->fProcessInput = &CLicqConsole::InputCommand;
@@ -1022,7 +1039,7 @@ void CLicqConsole::SwitchToCon(unsigned short nCon)
     winConStatus->wprintf("%A%Clog console", A_BOLD, 5);
   winConStatus->wprintf("%A%C) ]", A_BOLD, 29);
   wclrtoeol(winConStatus->Win());
-  wbkgdset(winConStatus->Win(), COLOR_PAIR(8));
+  wbkgdset(winConStatus->Win(), COLOR_PAIR(COLOR_WHITE));
   mvwhline(winConStatus->Win(), 1, 0, ACS_HLINE, COLS);
   //mvwaddch(winConStatus->Win(), 1, COLS - USER_WIN_WIDTH - 1, ACS_TTEE);
   winConStatus->RefreshWin();
@@ -1059,6 +1076,14 @@ void CLicqConsole::ProcessStdin()
     return;
   }
 
+  // Some operating systems don't have a consistent set of settings for
+  // what key should erase characters.  In particular, FC3 sets the terminal
+  // erase character (kbs) for xterm to ^H instead of 127, even though the
+  // rest of the operating system follows the Debian conventions.
+  // This allows the user to override whatever the OS has chosen.
+  if (cIn == m_nBackspace)
+    cIn = KEY_BACKSPACE;
+
   (this->*(winMain->fProcessInput))(cIn);
 }
 
@@ -1196,6 +1221,10 @@ void CLicqConsole::InputCommand(int cIn)
         break;
       }
 
+      // If we haven't typed anything, why should we tab?
+      if (!nPos)
+        break;
+
       szIn[nPos] = '\0';
       char *szArg = strchr(szIn, ' ');
       unsigned short nArgPos = 0;
@@ -1245,7 +1274,7 @@ void CLicqConsole::InputCommand(int cIn)
       if (sTabCompletion.vszPartialMatch.size() == 0)
       { // No matches
         Beep();
-        winMain->wprintf("%CNo matches.\n", 16);
+        winMain->wprintf("%CNo matches.\n", COLOR_RED);
         wprintw(winPrompt->Win(), "");
         break;
       }
@@ -1262,7 +1291,8 @@ void CLicqConsole::InputCommand(int cIn)
       char *szMatch = sTabCompletion.szPartialMatch;
 
       if (sTabCompletion.vszPartialMatch.size() == 1)
-      { // Only one match
+      {
+        // Only one match
         // Check if there is a space in the match
         if (strchr(szMatch, ' ') != NULL)
         {
@@ -1280,7 +1310,9 @@ void CLicqConsole::InputCommand(int cIn)
         nTabs = 1;
       }
       else
-      { // Multiple matches
+      { 
+        if (szMatch == 0) break;
+        // Multiple matches
         Beep();
         // Check if there is a space in the match
         if (strchr(szMatch, ' ') != NULL)
@@ -1298,6 +1330,7 @@ void CLicqConsole::InputCommand(int cIn)
         // Set nTabs to 2 so the array is saved for the next tab hit
         nTabs = 2;
       }
+      free(szMatch);
       winPrompt->RefreshWin();
       break;
     }
@@ -1421,7 +1454,7 @@ bool CLicqConsole::ParseMacro(char *szMacro)
 
   if (iter == listMacros.end())
   {
-    winMain->wprintf("%CNo such macro \"%A%s%Z\"\n", 16, A_BOLD,
+    winMain->wprintf("%CNo such macro \"%A%s%Z\"\n", COLOR_RED, A_BOLD,
                      szMacro, A_BOLD);
     szMacro[0] = '\0';
     return false;
@@ -1430,8 +1463,21 @@ bool CLicqConsole::ParseMacro(char *szMacro)
   return true;
 }
 
-
-
+/*---------------------------------------------------------------------------
+ * CLicqConsole::SaveLastUser
+ *-------------------------------------------------------------------------*/
+void CLicqConsole::SaveLastUser(const string& id, unsigned long nPPID)
+{
+  // Save this as the last user
+  if (winMain->sLastContact.szId.empty() ||
+      !(id == winMain->sLastContact.szId &&
+      nPPID == winMain->sLastContact.nPPID))
+  {
+    winMain->sLastContact.nPPID = nPPID;
+    winMain->sLastContact.szId = id;
+    PrintStatus();
+  }
+}
 
 /*---------------------------------------------------------------------------
  * CLicqConsole::CurrentGroupName
@@ -1446,12 +1492,12 @@ char *CLicqConsole::CurrentGroupName()
       strcpy(szGroupName, "All Users");
     else
     {
-      GroupList *g = gUserManager.LockGroupList(LOCK_R);
-      if (m_nCurrentGroup > g->size())
+      LicqGroup* group = gUserManager.FetchGroup(m_nCurrentGroup, LOCK_R);
+      if (group == NULL)
         strcpy(szGroupName, "Invalid Group");
       else
-        strcpy(szGroupName, (*g)[m_nCurrentGroup - 1]);
-      gUserManager.UnlockGroupList();
+        strcpy(szGroupName, group->name().c_str());
+      gUserManager.DropGroup(group);
     }
   }
   else
@@ -1484,7 +1530,7 @@ void CLicqConsole::UserCommand_Info(const char *szId, unsigned long nPPID, char
                    "(U)pdate Info\n"
                    "for %s (%s)? %C%Z",
                    m_cColorQuery->nColor, m_cColorQuery->nAttr,
-                   u->GetAlias(), szId, 8, A_BOLD);
+                   u->GetAlias(), szId, COLOR_WHITE, A_BOLD);
   winMain->RefreshWin();
   gUserManager.DropUser(u);
 }
@@ -1526,7 +1572,7 @@ void CLicqConsole::InputInfo(int cIn)
     case '\r':
       break;
     default:
-      winMain->wprintf("%CInvalid key.\n", 16);
+      winMain->wprintf("%CInvalid key.\n", COLOR_RED);
     }
 
     winMain->fProcessInput = &CLicqConsole::InputCommand;
@@ -1542,7 +1588,7 @@ void CLicqConsole::InputInfo(int cIn)
     break;
 
   default:
-    winMain->wprintf("%CInvalid state: %A%d%Z.\n", 16, A_BOLD, A_BOLD);
+    winMain->wprintf("%CInvalid state: %A%d%Z.\n", COLOR_RED, A_BOLD, A_BOLD);
   }
 
 }
@@ -1584,8 +1630,7 @@ void CLicqConsole::UserCommand_View(const char *szId, unsigned long nPPID, char
 
     // Do we want to accept the file transfer?
     if (e->SubCommand() == ICQ_CMDxSUB_FILE)
-      //FIXME
-      //FileChatOffer(e->Sequence(), u->Uin());
+      FileChatOffer(e, szId, nPPID);
 
     delete e;
     gUserManager.DropUser(u);
@@ -1618,7 +1663,7 @@ void CLicqConsole::UserCommand_Remove(const char *szId, unsigned long nPPID, cha
 
   winMain->wprintf("%C%ARemove %s (%s) from contact list (y/N)? %C%Z",
                    m_cColorQuery->nColor, m_cColorQuery->nAttr,
-                   u->GetAlias(), szId, 8, A_BOLD);
+                   u->GetAlias(), szId, COLOR_WHITE, A_BOLD);
   winMain->RefreshWin();
   gUserManager.DropUser(u);
 }
@@ -1659,7 +1704,7 @@ void CLicqConsole::InputRemove(int cIn)
     break;
 
   default:
-    winMain->wprintf("%CInvalid state: %A%d%Z.\n", 16, A_BOLD, A_BOLD);
+    winMain->wprintf("%CInvalid state: %A%d%Z.\n", COLOR_RED, A_BOLD, A_BOLD);
   }
 
 }
@@ -1749,11 +1794,11 @@ void CLicqConsole::UserCommand_History(const char *szId, unsigned long nPPID, ch
     gUserManager.DropUser(u);
     return;
   }
-  char szFrom[32];
+  char *szFrom;
   if (gUserManager.FindOwner(szId, nPPID))
-    strcpy(szFrom, "Server");
+    szFrom = strdup("Server");
   else
-    strcpy(szFrom, u->GetAlias());
+    szFrom = strdup(u->GetAlias());
   gUserManager.DropUser(u);
 
   unsigned short nLast = lHistory.size();
@@ -1765,12 +1810,13 @@ void CLicqConsole::UserCommand_History(const char *szId, unsigned long nPPID, ch
     if (nLast > 0 )
     {
       winMain->wprintf("%CYou must specify an event number. (1-%d)\n",
-                       16, nLast);
+                       COLOR_RED, nLast);
     }
     else
     {
-      winMain->wprintf("%CNo System Events.\n", 8);
+      winMain->wprintf("%CNo System Events.\n", COLOR_WHITE);
     }
+    free(szFrom);
     return;
   }
 
@@ -1785,14 +1831,16 @@ void CLicqConsole::UserCommand_History(const char *szId, unsigned long nPPID, ch
   nStart = StrToRange(szStart, nLast, winMain->nLastHistory);
   if (nStart == -1)
   {
-    winMain->wprintf("%CInvalid start range: %A%s\n", 16,
+    winMain->wprintf("%CInvalid start range: %A%s\n", COLOR_RED,
                      A_BOLD, szStart);
+    free(szFrom);
     return;
   }
   else if (nStart > nLast || nStart < 1)
   {
     winMain->wprintf("%CStart value out of range, history contains %d events.\n",
-                     16, nLast);
+                     COLOR_RED, nLast);
+    free(szFrom);
     return;
   }
 
@@ -1801,14 +1849,16 @@ void CLicqConsole::UserCommand_History(const char *szId, unsigned long nPPID, ch
     nEnd = StrToRange(szEnd, nLast, nStart);
     if (nEnd == -1)
     {
-      winMain->wprintf("%CInvalid end range: %A%s\n", 16,
+      winMain->wprintf("%CInvalid end range: %A%s\n", COLOR_RED,
                        A_BOLD, szEnd);
+      free(szFrom);
       return;
     }
     else if (nEnd > nLast || nEnd < 1)
     {
       winMain->wprintf("%CEnd value out of range, history contains %d events.\n",
-                       16, nLast);
+                       COLOR_RED, nLast);
+      free(szFrom);
       return;
     }
   }
@@ -1819,6 +1869,7 @@ void CLicqConsole::UserCommand_History(const char *szId, unsigned long nPPID, ch
 
   winMain->nLastHistory = nEnd;
   PrintHistory(lHistory, nStart - 1, nEnd - 1, szFrom);
+  free(szFrom);
 }
 
 
@@ -1903,8 +1954,9 @@ void CLicqConsole::InputMessage(int cIn)
                          m_cColorInfo->nAttr);
         return;
       }
+      sz--;
       *sz = '\0';
-      sz++;
+      sz += 2;
       bool bDirect = SendDirect(data->szId, data->nPPID, *sz);
       winMain->wprintf("%C%ASending message %s...", m_cColorInfo->nColor,
                        m_cColorInfo->nAttr,
@@ -1938,7 +1990,7 @@ void CLicqConsole::InputMessage(int cIn)
     break;
 
   default:
-    winMain->wprintf("%CInvalid state: %A%d%Z.\n", 16, A_BOLD, A_BOLD);
+    winMain->wprintf("%CInvalid state: %A%d%Z.\n", COLOR_RED, A_BOLD, A_BOLD);
   }
 
 }
@@ -2043,8 +2095,8 @@ void CLicqConsole::InputSendFile(int cIn)
       ConstFileList lFileList;
       lFileList.push_back(strdup(data->szFileName));
 
-      winMain->event = licqDaemon->icqFileTransfer(strtoul(data->szId, (char **)NULL, 10),
-		      data->szFileName, data->szDescription, lFileList, ICQ_TCPxMSG_NORMAL,
+      winMain->event = licqDaemon->icqFileTransfer(data->szId,
+              data->szFileName, data->szDescription, lFileList, ICQ_TCPxMSG_NORMAL,
                        !bDirect);
       break;
     }
@@ -2058,7 +2110,7 @@ void CLicqConsole::InputSendFile(int cIn)
 /*---------------------------------------------------------------------------
  * CLicqConsole::UserCommand_SetAutoResponse
  *-------------------------------------------------------------------------*/
-void CLicqConsole::UserCommand_SetAutoResponse(const char *szId, unsigned long nPPID, char *)
+void CLicqConsole::UserCommand_SetAutoResponse(const char* /* szId */, unsigned long /* nPPID */, char *)
 {
   // First put this console into edit mode
   winMain->fProcessInput = &CLicqConsole::InputAutoResponse;
@@ -2094,9 +2146,9 @@ void CLicqConsole::InputAutoResponse(int cIn)
     else
     {
       *sz = '\0';
-      ICQOwner *o = gUserManager.FetchOwner(LOCK_W);
+      ICQOwner* o = gUserManager.FetchOwner(LICQ_PPID, LOCK_W);
       o->SetAutoResponse(data->szRsp);
-      gUserManager.DropOwner();
+      gUserManager.DropOwner(o);
       winMain->wprintf("%C%AAuto-response set.\n",
                        m_cColorInfo->nColor, m_cColorInfo->nAttr);
     }
@@ -2107,7 +2159,7 @@ void CLicqConsole::InputAutoResponse(int cIn)
     break;
 
   default:
-    winMain->wprintf("%CInvalid state: %A%d%Z.\n", 16, A_BOLD, A_BOLD);
+    winMain->wprintf("%CInvalid state: %A%d%Z.\n", COLOR_RED, A_BOLD, A_BOLD);
   }
 
 }
@@ -2214,7 +2266,7 @@ void CLicqConsole::InputUrl(int cIn)
     break;
 
   default:
-    winMain->wprintf("%CInvalid state: %A%d%Z.\n", 16, A_BOLD, A_BOLD);
+    winMain->wprintf("%CInvalid state: %A%d%Z.\n", COLOR_RED, A_BOLD, A_BOLD);
   }
 
 }
@@ -2279,15 +2331,15 @@ void CLicqConsole::InputSms(int cIn)
       ICQUser *u = gUserManager.FetchUser(data->szId, data->nPPID, LOCK_R);
       winMain->wprintf("%C%ASending SMS to %s ...", m_cColorInfo->nColor,
                        m_cColorInfo->nAttr, u->GetCellularNumber());
-      winMain->event = licqDaemon->icqSendSms(u->GetCellularNumber(),
-                       data->szMsg, strtoul(data->szId, (char **)NULL, 0));
+        winMain->event = licqDaemon->icqSendSms(data->szId, LICQ_PPID,
+            u->GetCellularNumber(), data->szMsg);
       gUserManager.DropUser(u);
       winMain->state = STATE_PENDING;
       break;
     }
 
   default:
-    winMain->wprintf("%CInvalid state: %A%d%Z.\n", 16, A_BOLD, A_BOLD);
+    winMain->wprintf("%CInvalid state: %A%d%Z.\n", COLOR_RED, A_BOLD, A_BOLD);
   }
 
 }
@@ -2349,7 +2401,7 @@ void CLicqConsole::InputAuthorize(int cIn)
     }
 
   default:
-    winMain->wprintf("%CInvalid state: %A%d%Z.\n", 16, A_BOLD, A_BOLD);
+    winMain->wprintf("%CInvalid state: %A%d%Z.\n", COLOR_RED, A_BOLD, A_BOLD);
   }
 
 }
@@ -2648,7 +2700,7 @@ void CLicqConsole::InputSearch(int cIn)
                              "1 %A-%Z Female\n"
                              "2 %A-%Z Male\n"
                              "%A%CGender: ",
-                             8, A_BOLD, A_BOLD, A_BOLD, A_BOLD,
+                             COLOR_WHITE, A_BOLD, A_BOLD, A_BOLD, A_BOLD,
                              A_BOLD, A_BOLD, m_cColorQuery->nAttr, m_cColorQuery->nColor);
             return;
           }
@@ -2678,7 +2730,7 @@ void CLicqConsole::InputSearch(int cIn)
             for (unsigned short i = 0; i < NUM_LANGUAGES; i++)
             {
               winMain->wprintf("%C%s %A(%Z%d%A)%s%Z",
-                               8, gLanguages[i].szName,
+                               COLOR_WHITE, gLanguages[i].szName,
                                A_BOLD, A_BOLD, gLanguages[i].nCode,
                                A_BOLD,
                                i == NUM_LANGUAGES - 1 ? "\n" : ", ",
@@ -2736,7 +2788,7 @@ void CLicqConsole::InputSearch(int cIn)
             for (unsigned short i = 0; i < NUM_COUNTRIES; i++)
             {
               winMain->wprintf("%C%s %A(%Z%d%A)%s%Z",
-                               8, gCountries[i].szName,
+                               COLOR_WHITE, gCountries[i].szName,
                                A_BOLD, A_BOLD, gCountries[i].nCode,
                                A_BOLD,
                                i == NUM_COUNTRIES - 1 ? "\n" : ", ",
@@ -2848,7 +2900,7 @@ void CLicqConsole::RegistrationWizard()
   winMain->data = new DataRegWizard();
 
   winMain->wprintf("%A%CWelcome to the Licq Console Registration Wizard\n\nPress 1 to register a new UIN\nPress 2 if you have a UIN and password\n\n",
-                   A_BOLD, 24);
+                   A_BOLD, COLOR_GREEN);
 
   return;
 }
@@ -2932,8 +2984,8 @@ void CLicqConsole::InputRegistrationWizard(int cIn)
               return;
             }
 
-	    winMain->state = STATE_QUERY;
-	    winMain->wprintf("\nSave password? (y/N) ");
+            winMain->state = STATE_QUERY;
+            winMain->wprintf("\nSave password? (y/N) ");
           }
           break;
         }
@@ -2980,14 +3032,15 @@ void CLicqConsole::InputRegistrationWizard(int cIn)
             }
 
             // Passwords match if we are this far, now set up the new user
-	    winMain->wprintf("Registration complete for user %s\n",data->szUin);
-            gUserManager.SetOwnerUin(atol(data->szUin));
-            ICQOwner *owner = gUserManager.FetchOwner(LOCK_W);
+            winMain->wprintf("Registration complete for user %s\n",data->szUin);
+            gUserManager.AddOwner(data->szUin, LICQ_PPID);
+            ICQOwner* owner = gUserManager.FetchOwner(LICQ_PPID, LOCK_W);
             owner->SetPassword(data->szPassword1);
-            gUserManager.DropOwner();
+            gUserManager.DropOwner(owner);
+            licqDaemon->SaveConf();
 
-	    winMain->wprintf("Save password? (y/N) ");
-	    winMain->state = STATE_QUERY;
+            winMain->wprintf("Save password? (y/N) ");
+            winMain->state = STATE_QUERY;
           }
           break;
         }
@@ -2997,13 +3050,13 @@ void CLicqConsole::InputRegistrationWizard(int cIn)
       }
       break;
     }
-  
+
   case STATE_QUERY:
   {
-    ICQOwner *o = gUserManager.FetchOwner(LOCK_W);
+    ICQOwner* o = gUserManager.FetchOwner(LICQ_PPID, LOCK_W);
     o->SetSavePassword(tolower(cIn) == 'y');
-    gUserManager.DropOwner();
-    
+    gUserManager.DropOwner(o);
+
     if (data->szOption[0] == '1')
     {
       winMain->wprintf("\nRegistering you as a new user...\n");
@@ -3027,14 +3080,15 @@ void CLicqConsole::InputRegistrationWizard(int cIn)
 /*---------------------------------------------------------------------------
  * CLicqConsole::FileChatOffer
  *-------------------------------------------------------------------------*/
-void CLicqConsole::FileChatOffer(unsigned long _nSequence, const char *_szId, unsigned long _nPPID)
+void CLicqConsole::FileChatOffer(CUserEvent *e, const char *_szId, unsigned long _nPPID)
 {
+  CEventFile *f = (CEventFile *)e;
   // Get y or n
   winMain->fProcessInput = &CLicqConsole::InputFileChatOffer;
   winMain->state = STATE_QUERY;
-  winMain->data = new DataFileChatOffer(_nSequence, _szId, _nPPID);
+  winMain->data = new DataFileChatOffer(f, _szId, _nPPID);
   winMain->wprintf("%C%ADo you wish to accept this request? (y/N) %C%Z",
-                   m_cColorQuery->nColor, m_cColorQuery->nAttr, 8, A_BOLD);
+                   m_cColorQuery->nColor, m_cColorQuery->nAttr, COLOR_WHITE, A_BOLD);
   winMain->RefreshWin();
 
   return;
@@ -3047,6 +3101,7 @@ void CLicqConsole::InputFileChatOffer(int cIn)
 {
   DataFileChatOffer *data = (DataFileChatOffer *)winMain->data;
   char *sz;
+  CEventFile *f = data->f;
 
   switch(winMain->state)
   {
@@ -3056,7 +3111,7 @@ void CLicqConsole::InputFileChatOffer(int cIn)
       {
       case 'y':
         {
-          winMain->wprintf("%C%A\nAccepting file\n", 24, A_BOLD);
+          winMain->wprintf("%C%A\nAccepting file\n", COLOR_GREEN, A_BOLD);
 
           // Make the ftman
           CFileTransferManager *ftman = new CFileTransferManager(licqDaemon,
@@ -3064,13 +3119,16 @@ void CLicqConsole::InputFileChatOffer(int cIn)
           ftman->SetUpdatesEnabled(1);
           m_lFileStat.push_back(ftman);
 
+          // Now watch the file pipe
+          FD_SET(ftman->Pipe(), &fdSet);
+
+
           // Accept the file
           const char *home = getenv("HOME");
           ftman->ReceiveFiles(home);
-          // XXX hack
-          unsigned long dummy[2] = { 0, 0};
-          licqDaemon->icqFileTransferAccept(strtoul(data->szId, (char **)NULL, 10), ftman->LocalPort(),
-                                            data->nSequence, dummy, true);
+          licqDaemon->icqFileTransferAccept(data->szId, ftman->LocalPort(),
+                                            f->Sequence(), f->MessageID(), f->IsDirect(),
+                                            f->FileDescription(), f->Filename(), f->FileSize());
           winMain->fProcessInput = &CLicqConsole::InputCommand;
 
           if(winMain->data)
@@ -3101,8 +3159,8 @@ void CLicqConsole::InputFileChatOffer(int cIn)
 
       // XXX hack
       unsigned long dummy[2] = { 0, 0 };
-      licqDaemon->icqFileTransferRefuse(strtoul(data->szId, (char **)NULL, 10), data->szReason,
-                                        data->nSequence, dummy, true);
+      licqDaemon->icqFileTransferRefuse(data->szId, data->szReason,
+                                        f->Sequence(), dummy, true);
 
       // We are done now
       winMain->wprintf("%ARefusing file from %s with reason: %Z%s\n",
@@ -3130,14 +3188,14 @@ void CLicqConsole::UserCommand_Secure(const char *szId, unsigned long nPPID, cha
   if(!licqDaemon->CryptoEnabled())
   {
     winMain->wprintf("%CYou need to recompile Licq with OpenSSL for this "
-                     "feature to work!\n", 16);
+                     "feature to work!\n", COLOR_RED);
     return;
   }
 
   if(u->SecureChannelSupport() != SECURE_CHANNEL_SUPPORTED)
   {
     winMain->wprintf("%CThe remote end is not using a supported client.  "
-                     "This may not work!\n", 16);
+                     "This may not work!\n", COLOR_RED);
   }
 
   bool bOpen = u->Secure();
@@ -3162,24 +3220,20 @@ void CLicqConsole::UserCommand_Secure(const char *szId, unsigned long nPPID, cha
     winMain->wprintf("%ARequest secure channel with %s ... ", A_BOLD,
                      u->GetAlias());
     gUserManager.DropUser(u);
-    char *p;
-    unsigned long uin = strtoul(szId, &p,10);
-    if( (p == 0 || (p && !*p)) && nPPID == LICQ_PPID )
-      winMain->event = licqDaemon->icqOpenSecureChannel(uin);
+    if (nPPID == LICQ_PPID )
+      winMain->event = licqDaemon->icqOpenSecureChannel(szId);
   }
   else if(strcasecmp(szStatus, "close") == 0)
   {
     winMain->wprintf("%AClose secure channel with %s ... ", A_BOLD,
                      u->GetAlias());
     gUserManager.DropUser(u);
-    char *p;
-    unsigned long uin = strtoul(szId, &p,10);
-    if( (p == 0 || (p && !*p)) && nPPID == LICQ_PPID )
-      winMain->event = licqDaemon->icqCloseSecureChannel(uin);
+    if (nPPID == LICQ_PPID )
+      winMain->event = licqDaemon->icqCloseSecureChannel(szId);
   }
   else
   {
-    winMain->wprintf("%C<user> secure <open | close | (blank)>\n", 16);
+    winMain->wprintf("%C<user> secure <open | close | (blank)>\n", COLOR_RED);
   }
 
   if(u)
@@ -3195,16 +3249,13 @@ void CLicqConsole::UserSelect()
   winMain->fProcessInput = &CLicqConsole::InputUserSelect;
   winMain->state = STATE_LE;
 
-  char sz[20];
   //TODO which owner
-  sprintf(sz, "%lu", gUserManager.OwnerUin());
-  
-  winMain->data = new DataUserSelect(sz, LICQ_PPID); 
-  
-  ICQOwner *o = gUserManager.FetchOwner(LOCK_R);
-  winMain->wprintf("%A%CEnter your password for %s (%lu):%C%Z\n", A_BOLD,
-                   24, o->GetAlias(), o->Uin(), 8, A_BOLD);
-  gUserManager.DropOwner();
+  winMain->data = new DataUserSelect(gUserManager.OwnerId(LICQ_PPID).c_str(), LICQ_PPID);
+
+  ICQOwner* o = gUserManager.FetchOwner(LICQ_PPID, LOCK_R);
+  winMain->wprintf("%A%CEnter your password for %s (%s):%C%Z\n", A_BOLD,
+                   COLOR_GREEN, o->GetAlias(), o->IdString(), COLOR_WHITE, A_BOLD);
+  gUserManager.DropOwner(o);
 }
 
 /*------------------------------------------------------------------------
@@ -3225,17 +3276,17 @@ void CLicqConsole::InputUserSelect(int cIn)
       data->nPos = 0;
 
       // Next stage is saving the password possibly
-      winMain->wprintf("%C%ASave password? (y/N) %C%Z", 24, A_BOLD,
-                       8, A_BOLD);
+      winMain->wprintf("%C%ASave password? (y/N) %C%Z", COLOR_GREEN, A_BOLD,
+                       COLOR_WHITE, A_BOLD);
       winMain->state = STATE_QUERY;
       break;
     
     case STATE_QUERY:
     {
-      ICQOwner *o = gUserManager.FetchOwner(LOCK_W);
+      ICQOwner* o = gUserManager.FetchOwner(LICQ_PPID, LOCK_W);
       o->SetSavePassword(tolower(cIn) == 'y');
       o->SetPassword(data->szPassword);
-      gUserManager.DropOwner();
+      gUserManager.DropOwner(o);
 
       if (winMain->data)
       {
