@@ -1,7 +1,11 @@
 #include "console.h"
+
+#include <cctype>
+#include <string>
+
 #include "event_data.h"
 
-#include <ctype.h>
+using namespace std;
 
 const unsigned short NUM_COMMANDS = 24;
 const struct SCommand aCommands[NUM_COMMANDS] =
@@ -110,7 +114,7 @@ const struct SCommand aCommands[NUM_COMMANDS] =
  *
  * Callback function for when SPACE is pressed on the cdkUserList
  *-------------------------------------------------------------------------*/
-int CLicqConsole::MenuPopupWrapper(EObjectType cdktype, void *object, void *clientData, chtype key)
+int CLicqConsole::MenuPopupWrapper(EObjectType /* cdktype */, void* /* object */, void* clientData, chtype /* key */)
 {
   CLicqConsole *me = (CLicqConsole *)clientData;
   me->MenuPopup(me->cdkUserList->currentItem);
@@ -177,7 +181,7 @@ void CLicqConsole::MenuSwitchConsole(char *_szArg)
 /*---------------------------------------------------------------------------
  * CLicqConsole::MenuList
  *-------------------------------------------------------------------------*/
-void CLicqConsole::MenuList(char *_szArg)
+void CLicqConsole::MenuList(char* /* _szArg */)
 {
   int userSelected;
   UserListHighlight(A_REVERSE);
@@ -254,7 +258,7 @@ void CLicqConsole::MenuQuit(char *)
 /*---------------------------------------------------------------------------
  * CLicqConsole::MenuPlugins
  *-------------------------------------------------------------------------*/
-void CLicqConsole::MenuPlugins(char *_szArg)
+void CLicqConsole::MenuPlugins(char* /* _szArg */)
 {
   PluginsList l;
   PluginsListIter it;
@@ -262,7 +266,7 @@ void CLicqConsole::MenuPlugins(char *_szArg)
   ProtoPluginsList p1;
   ProtoPluginsListIter pit;
   licqDaemon->ProtoPluginList(p1);
-  PrintBoxTop("Plugins", 40, 70);
+  PrintBoxTop("Plugins", COLOR_BLUE, 70);
   for (it = l.begin(); it != l.end(); it++)
   {
     PrintBoxLeft();
@@ -315,7 +319,7 @@ void CLicqConsole::MenuDefine(char *szArg)
         return;
       }
     }
-    winMain->wprintf("%CNo such macro \"%A%s%Z\"\n", 16, A_BOLD,
+    winMain->wprintf("%CNo such macro \"%A%s%Z\"\n", COLOR_RED, A_BOLD,
       szArg, A_BOLD);
     return;
   }
@@ -373,7 +377,7 @@ void CLicqConsole::MenuGroup(char *_szArg)
 
     if (nCurrentGroup > NUM_GROUPS_SYSTEM || nCurrentGroup == 0)
     {
-      winMain->wprintf("%CInvalid group number (0 - %d)\n", 16,
+      winMain->wprintf("%CInvalid group number (0 - %d)\n", COLOR_RED,
                        NUM_GROUPS_SYSTEM);
       return;
     }
@@ -388,21 +392,20 @@ void CLicqConsole::MenuGroup(char *_szArg)
   {
     nGroupType = GROUPS_USER;
     nCurrentGroup = atol(_szArg);
+    LicqGroup* group = gUserManager.FetchGroup(nCurrentGroup, LOCK_R);
 
-    if (nCurrentGroup > gUserManager.NumGroups())
+    if (nCurrentGroup != 0 && group == NULL)
     {
-      winMain->wprintf("%CInvalid group number (0 - %d)\n", 16,
-                       gUserManager.NumGroups());
+      winMain->wprintf("%CInvalid group number\n", COLOR_RED);
       return;
     }
     m_nCurrentGroup = nCurrentGroup;
     m_nGroupType = nGroupType;
-    GroupList *g = gUserManager.LockGroupList(LOCK_R);
     winMain->wprintf("%C%ASwitching to group %d (%s).\n",
                      m_cColorInfo->nColor, m_cColorInfo->nAttr,
                      m_nCurrentGroup,
-                     m_nCurrentGroup == 0 ? "All Users" : (*g)[m_nCurrentGroup - 1]);
-    gUserManager.UnlockGroupList();
+                     m_nCurrentGroup == 0 ? "All Users" : group->name().c_str());
+    gUserManager.DropGroup(group);
   }
 
   PrintStatus();
@@ -419,7 +422,7 @@ void CLicqConsole::MenuAdd(char *szArg)
 {
   if (szArg == NULL)
   {
-    winMain->wprintf("%CSpecify a UIN to add.\n", 16);
+    winMain->wprintf("%CSpecify a UIN to add.\n", COLOR_RED);
     return;
   }
 
@@ -437,7 +440,7 @@ void CLicqConsole::MenuAdd(char *szArg)
   if (!licqDaemon->AddUserToList(szArg, LICQ_PPID))
   {
     winMain->wprintf("%CAdding user %s failed (duplicate user or invalid uin).\n",
-     16, szArg);
+     COLOR_RED, szArg);
     return;
   }
 
@@ -447,11 +450,9 @@ void CLicqConsole::MenuAdd(char *szArg)
 
   if (bAlert)
   {
-    int nUin = atol(szArg);
-    licqDaemon->icqAlertUser(nUin);
-    winMain->wprintf("%C%AAlerted user %ld they were added.\n",
-                     m_cColorInfo->nColor, m_cColorInfo->nAttr,
-                     nUin);
+    licqDaemon->icqAlertUser(szArg, LICQ_PPID);
+    winMain->wprintf("%C%AAlerted user %s they were added.\n",
+        m_cColorInfo->nColor, m_cColorInfo->nAttr, szArg);
   }
 
 }
@@ -464,7 +465,7 @@ void CLicqConsole::MenuAuthorize(char *szArg)
 {
   if (szArg == NULL)
   {
-    winMain->wprintf("%CSpecify \"grant/refuse\" and a UIN/Screen Name to authorize.\n", 16);
+    winMain->wprintf("%CSpecify \"grant/refuse\" and a UIN/Screen Name to authorize.\n", COLOR_RED);
     return;
   }
 
@@ -505,7 +506,7 @@ void CLicqConsole::MenuStatus(char *_szArg)
 
   if (_szArg == NULL)
   {
-    winMain->wprintf("%CSpecify status.\n", 16);
+    winMain->wprintf("%CSpecify status.\n", COLOR_RED);
     return;
   }
 
@@ -527,7 +528,7 @@ void CLicqConsole::MenuStatus(char *_szArg)
   // Check that we found it
   if (i == NUM_STATUS)
   {
-    winMain->wprintf("%CInvalid status: %A%s\n", 16, A_BOLD, _szArg);
+    winMain->wprintf("%CInvalid status: %A%s\n", COLOR_RED, A_BOLD, _szArg);
     return;
   }
 
@@ -541,7 +542,7 @@ void CLicqConsole::MenuStatus(char *_szArg)
     ICQOwner *o = gUserManager.FetchOwner(nPPID, LOCK_R);
     if (nStatus == ICQ_STATUS_OFFLINE)
     {
-      gUserManager.DropOwner(nPPID);
+      gUserManager.DropOwner(o);
       licqDaemon->ProtoLogoff(nPPID);
       continue;
     }
@@ -550,7 +551,7 @@ void CLicqConsole::MenuStatus(char *_szArg)
 
     // call the right function
     bool b = o->StatusOffline();
-    gUserManager.DropOwner(nPPID);
+    gUserManager.DropOwner(o);
     if (b)
     {
        licqDaemon->ProtoLogon(nPPID, nStatus);
@@ -593,10 +594,10 @@ void CLicqConsole::MenuUins(char *)
 /*---------------------------------------------------------------------------
  * CLicqConsole::GetUinFromArg
  *-------------------------------------------------------------------------*/
-unsigned long CLicqConsole::GetUinFromArg(char **p_szArg)
+string CLicqConsole::GetUserFromArg(char** p_szArg)
 {
   char *szAlias, *szCmd;
-  unsigned long nUin = 0;
+  string id;
   bool bCheckUin = true;
   char *szArg = *p_szArg;
 
@@ -612,8 +613,8 @@ unsigned long CLicqConsole::GetUinFromArg(char **p_szArg)
     szCmd = strchr(&szArg[1], '"');
     if (szCmd == NULL)
     {
-      winMain->wprintf("%CUnbalanced quotes.\n", 16);
-      return (unsigned long)-1;
+      winMain->wprintf("%CUnbalanced quotes.\n", COLOR_RED);
+      return "-";
     }
     *szCmd++ = '\0';
     szCmd = strchr(szCmd, ' ');
@@ -621,12 +622,12 @@ unsigned long CLicqConsole::GetUinFromArg(char **p_szArg)
   else if (szArg[0] == '#')
   {
     *p_szArg = NULL;
-    return gUserManager.OwnerUin();
+    return gUserManager.OwnerId(LICQ_PPID);
   }
   else if (szArg[0] == '$')
   {
     *p_szArg = NULL;
-    return winMain->nLastUin;
+    return winMain->myLastId;
   }
   else
   {
@@ -641,49 +642,31 @@ unsigned long CLicqConsole::GetUinFromArg(char **p_szArg)
   }
   *p_szArg = szCmd;
 
-  // Find the user
-  // See if all the chars are digits
-  if (bCheckUin)
+  FOR_EACH_PROTO_USER_START(LICQ_PPID, LOCK_R)
   {
-    char *sz = szAlias;
-    while (isdigit(*sz)) sz++;
-    if (*sz == '\0') nUin = atol(szAlias);
+    if (strcasecmp(szAlias, pUser->GetAlias()) == 0 ||
+        (bCheckUin && strcasecmp(szAlias, pUser->IdString()) == 0))
+    {
+      id = pUser->IdString();
+      FOR_EACH_USER_BREAK;
+    }
   }
+  FOR_EACH_PROTO_USER_END
 
-  if (nUin == 0)
+  if (id.empty())
   {
-    FOR_EACH_PROTO_USER_START(LICQ_PPID, LOCK_R)
-    {
-      if (strcasecmp(szAlias, pUser->GetAlias()) == 0)
-      {
-        nUin = pUser->Uin();
-        FOR_EACH_USER_BREAK;
-      }
-    }
-    FOR_EACH_PROTO_USER_END
-    if (nUin == 0)
-    {
-      winMain->wprintf("%CInvalid user: %A%s\n", 16, A_BOLD, szAlias);
-      return (unsigned long)-1;
-    }
-  }
-  else
-  {
-    if (!gUserManager.IsOnList(nUin))
-    {
-      winMain->wprintf("%CInvalid uin: %A%lu\n", 16, A_BOLD, nUin);
-      return (unsigned long)-1;
-    }
+    winMain->wprintf("%CInvalid user: %A%s\n", COLOR_RED, A_BOLD, szAlias);
+    return "-";
   }
 
   // Save this as the last user
-  if (winMain->nLastUin != nUin)
+  if (winMain->myLastId != id)
   {
-    winMain->nLastUin = nUin;
+    winMain->myLastId = id;
     PrintStatus();
   }
 
-  return nUin;
+  return id;
 }
 
 /*---------------------------------------------------------------------------
@@ -695,7 +678,6 @@ struct SContact CLicqConsole::GetContactFromArg(char **p_szArg)
   char *szArg = *p_szArg;
   unsigned long nPPID = 0;
   struct SContact scon;
-  scon.szId = NULL;
   scon.nPPID = 0;
 
   if (szArg == NULL) {
@@ -731,7 +713,7 @@ struct SContact CLicqConsole::GetContactFromArg(char **p_szArg)
     szCmd = strchr(&szArg[1], '"');
     if (szCmd == NULL)
     {
-      winMain->wprintf("%CUnbalanced quotes.\n", 16);
+      winMain->wprintf("%CUnbalanced quotes.\n", COLOR_RED);
       return scon;
     }
     *szCmd++ = '\0';
@@ -740,10 +722,10 @@ struct SContact CLicqConsole::GetContactFromArg(char **p_szArg)
   else if (szArg[0] == '#')
   {
     *p_szArg = NULL;
-    ICQOwner *o = gUserManager.FetchOwner(LOCK_R);
+    ICQOwner* o = gUserManager.FetchOwner(LICQ_PPID, LOCK_R);
     scon.szId = o->IdString();
-    scon.nPPID = o->PPID(); 
-    gUserManager.DropOwner();
+    scon.nPPID = o->PPID();
+    gUserManager.DropOwner(o);
     return scon;
   }
   else if (szArg[0] == '$')
@@ -782,10 +764,10 @@ struct SContact CLicqConsole::GetContactFromArg(char **p_szArg)
     }
   }
   FOR_EACH_USER_END
-  if (scon.szId == NULL)
+  if (scon.szId.empty())
   {
-    winMain->wprintf("%CInvalid user: %A%s\n", 16, A_BOLD, szAlias);
-    scon.szId = NULL;
+    winMain->wprintf("%CInvalid user: %A%s\n", COLOR_RED, A_BOLD, szAlias);
+    scon.szId.clear();
     scon.nPPID = (unsigned long)-1;
     return scon;
   }
@@ -803,10 +785,10 @@ void CLicqConsole::MenuMessage(char *szArg)
   char *sz = szArg;
   struct SContact scon = GetContactFromArg(&sz);
 
-  if (!scon.szId && scon.nPPID != (unsigned long)-1)
-    winMain->wprintf("%CYou must specify a user to send a message to.\n", 16);
+  if (scon.szId.empty() && scon.nPPID != (unsigned long)-1)
+    winMain->wprintf("%CYou must specify a user to send a message to.\n", COLOR_RED);
   else if (scon.nPPID != (unsigned long)-1)
-    UserCommand_Msg(scon.szId, scon.nPPID, sz);
+    UserCommand_Msg(scon.szId.c_str(), scon.nPPID, sz);
 }
 
 
@@ -817,17 +799,14 @@ void CLicqConsole::MenuMessage(char *szArg)
 void CLicqConsole::MenuInfo(char *szArg)
 {
   char *sz = szArg;
-  unsigned long nUin = GetUinFromArg(&sz);
+  string id = GetUserFromArg(&sz);
 
-  if (nUin == gUserManager.OwnerUin())
-    winMain->wprintf("%CSetting personal info not implemented yet.\n", 16);
-  else if (nUin == 0) {
-    char szUin[24];
-    sprintf(szArg, "%lu", gUserManager.OwnerUin());
-    UserCommand_Info(szUin, LICQ_PPID, sz);
-  } else if (nUin != (unsigned long)-1)
-    sprintf(szArg, "%lu", nUin);
-    UserCommand_Info(szArg, LICQ_PPID, sz);
+  if (gUserManager.FindOwner(id.c_str(), LICQ_PPID) != NULL)
+    winMain->wprintf("%CSetting personal info not implemented yet.\n", COLOR_RED);
+  else if (id.empty())
+    UserCommand_Info(gUserManager.OwnerId(LICQ_PPID).c_str(), LICQ_PPID, sz);
+  else if (id != "-")
+    UserCommand_Info(id.c_str(), LICQ_PPID, sz);
 }
 
 
@@ -838,16 +817,14 @@ void CLicqConsole::MenuInfo(char *szArg)
 void CLicqConsole::MenuUrl(char *szArg)
 {
   char *sz = szArg;
-  unsigned long nUin = GetUinFromArg(&sz);
-
-  if (nUin == gUserManager.OwnerUin())
-    winMain->wprintf("%CYou can't send URLs to yourself!\n", 16);
-  else if (nUin == 0)
-    winMain->wprintf("%CYou must specify a user to send a URL to.\n", 16);
-  else if (nUin != (unsigned long)-1)
-    sprintf(szArg, "%lu", nUin);
-    UserCommand_Url(szArg, LICQ_PPID, sz);
+  string id = GetUserFromArg(&sz);
 
+  if (gUserManager.FindOwner(id.c_str(), LICQ_PPID) != NULL)
+    winMain->wprintf("%CYou can't send URLs to yourself!\n", COLOR_RED);
+  else if (id.empty())
+    winMain->wprintf("%CYou must specify a user to send a URL to.\n", COLOR_RED);
+  else if (id != "-")
+    UserCommand_Url(id.c_str(), LICQ_PPID, sz);
 }
 
 
@@ -858,15 +835,12 @@ void CLicqConsole::MenuUrl(char *szArg)
 void CLicqConsole::MenuSms(char *szArg)
 {
   char *sz = szArg;
-  unsigned long nUin = GetUinFromArg(&sz);
+  string id = GetUserFromArg(&sz);
 
-  if (nUin == 0)
-    winMain->wprintf("%CInvalid user\n", 16);
-  else if (nUin != (unsigned long)-1)
-  {
-    sprintf(szArg, "%lu", nUin);
-    UserCommand_Sms(szArg, LICQ_PPID, sz);
-  }
+  if (id.empty())
+    winMain->wprintf("%CInvalid user\n", COLOR_RED);
+  else if (id != "-")
+    UserCommand_Sms(id.c_str(), LICQ_PPID, sz);
 }
 
 
@@ -877,11 +851,11 @@ void CLicqConsole::MenuView(char *szArg)
 {
 
   char *sz = szArg;
-  char *szId = NULL;
+  string szId;
   unsigned long nPPID = (unsigned long)-1;
   struct SContact scon = GetContactFromArg(&sz);
-  
-  if (scon.szId == 0)
+
+  if (scon.szId.empty())
   {
     // Do nothing if there are no events pending
     if (ICQUser::getNumUserEvents() == 0) return;
@@ -889,13 +863,11 @@ void CLicqConsole::MenuView(char *szArg)
     // Do icq system messages first
     ICQOwner *o = gUserManager.FetchOwner(LICQ_PPID, LOCK_R);
     unsigned short nNumMsg = o->NewMessages();
-    gUserManager.DropOwner();
+    gUserManager.DropOwner(o);
     if (nNumMsg > 0)
     {
       //TODO which owner?
-      char szUin[24];
-      sprintf(szUin, "%lu", gUserManager.OwnerUin());
-      UserCommand_View(szUin, LICQ_PPID, NULL);
+      UserCommand_View(gUserManager.OwnerId(LICQ_PPID).c_str(), LICQ_PPID, NULL);
       return;
     }
 
@@ -910,14 +882,14 @@ void CLicqConsole::MenuView(char *szArg)
       }
     }
     FOR_EACH_USER_END
-    if (szId != NULL)
+    if (!szId.empty())
     {
-      UserCommand_View(szId, nPPID, NULL);
+      UserCommand_View(szId.c_str(), nPPID, NULL);
     }
   }
-  else if (scon.szId != NULL)
+  else if (!scon.szId.empty())
   {
-    UserCommand_View(scon.szId, scon.nPPID, sz);
+    UserCommand_View(scon.szId.c_str(), scon.nPPID, sz);
   }
 
 }
@@ -929,17 +901,14 @@ void CLicqConsole::MenuView(char *szArg)
 void CLicqConsole::MenuSecure(char *szArg)
 {
   char *sz = szArg;
-  unsigned long nUin = GetUinFromArg(&sz);
+  string id = GetUserFromArg(&sz);
 
-  if (nUin == gUserManager.OwnerUin())
-    winMain->wprintf("%CYou can't establish a secure connection to yourself!\n", 16);
-  else if (nUin == 0)
-    winMain->wprintf("%CYou must specify a user to talk to.\n", 16);
-  else if (nUin != (unsigned long)-1)
-  {
-    sprintf(szArg, "%lu", nUin);
-    UserCommand_Secure(szArg, LICQ_PPID, sz);
-  }
+  if (gUserManager.FindOwner(id.c_str(), LICQ_PPID) != NULL)
+    winMain->wprintf("%CYou can't establish a secure connection to yourself!\n", COLOR_RED);
+  else if (id.empty())
+    winMain->wprintf("%CYou must specify a user to talk to.\n", COLOR_RED);
+  else if (id != "-")
+    UserCommand_Secure(id.c_str(), LICQ_PPID, sz);
 }
 
 
@@ -949,11 +918,11 @@ void CLicqConsole::MenuSecure(char *szArg)
 void CLicqConsole::MenuFile(char *szArg)
 {
   char *sz = szArg;
-  unsigned long nUin = GetUinFromArg(&sz);
+  string id = GetUserFromArg(&sz);
 
-  if (nUin == gUserManager.OwnerUin())
-    winMain->wprintf("%CYou can't send files to yourself!\n", 16);
-  else if (nUin == 0)
+  if (gUserManager.FindOwner(id.c_str(), LICQ_PPID) != NULL)
+    winMain->wprintf("%CYou can't send files to yourself!\n", COLOR_RED);
+  else if (id.empty())
   {
     bool bNum = false;
 
@@ -972,11 +941,8 @@ void CLicqConsole::MenuFile(char *szArg)
        m_cColorInfo->nColor);
     }
   }
-  else if (nUin != (unsigned long)-1)
-  {
-    sprintf(szArg, "%lu", nUin);
-    UserCommand_SendFile(szArg, LICQ_PPID, sz);
-  }
+  else if (id != "-")
+    UserCommand_SendFile(id.c_str(), LICQ_PPID, sz);
 }
 
 
@@ -986,9 +952,9 @@ void CLicqConsole::MenuFile(char *szArg)
 void CLicqConsole::MenuAutoResponse(char *szArg)
 {
   char *sz = szArg;
-  unsigned long nUin = GetUinFromArg(&sz);
+  string id = GetUserFromArg(&sz);
 
-  if (nUin == gUserManager.OwnerUin())
+  if (gUserManager.FindOwner(id.c_str(), LICQ_PPID) != NULL)
   {
     wattron(winMain->Win(), A_BOLD);
     for (unsigned short i = 0; i < winMain->Cols() - 10; i++)
@@ -996,8 +962,8 @@ void CLicqConsole::MenuAutoResponse(char *szArg)
     waddch(winMain->Win(), '\n');
     ICQOwner *o = gUserManager.FetchOwner(LICQ_PPID, LOCK_R);
     winMain->wprintf("%B%CAuto response:\n%b%s\n",
-                     8, o->AutoResponse());
-    gUserManager.DropOwner();
+                     COLOR_WHITE, o->AutoResponse());
+    gUserManager.DropOwner(o);
     wattron(winMain->Win(), A_BOLD);
     for (unsigned short i = 0; i < winMain->Cols() - 10; i++)
       waddch(winMain->Win(), ACS_HLINE);
@@ -1005,9 +971,9 @@ void CLicqConsole::MenuAutoResponse(char *szArg)
     winMain->RefreshWin();
     wattroff(winMain->Win(), A_BOLD);
   }
-  else if (nUin == 0)
+  else if (id.empty())
     UserCommand_SetAutoResponse(NULL, LICQ_PPID, sz);
-  else if (nUin != (unsigned long)-1)
+  else if (id != "-")
     UserCommand_FetchAutoResponse(NULL, LICQ_PPID, sz);
 }
 
@@ -1020,12 +986,12 @@ void CLicqConsole::MenuRemove(char *szArg)
   char *sz = szArg;
   struct SContact scon = GetContactFromArg(&sz);
 
-  if (gUserManager.FindOwner(scon.szId, scon.nPPID))
-    winMain->wprintf("%CYou can't remove yourself!\n", 16);
-  else if (!scon.szId && scon.nPPID != (unsigned long)-1)
-    winMain->wprintf("%CYou must specify a user to remove.\n", 16);
+  if (gUserManager.FindOwner(scon.szId.c_str(), scon.nPPID))
+    winMain->wprintf("%CYou can't remove yourself!\n", COLOR_RED);
+  else if (scon.szId.empty() && scon.nPPID != (unsigned long)-1)
+    winMain->wprintf("%CYou must specify a user to remove.\n", COLOR_RED);
   else
-    UserCommand_Remove(scon.szId, scon.nPPID, sz);
+    UserCommand_Remove(scon.szId.c_str(), scon.nPPID, sz);
 }
 
 
@@ -1037,10 +1003,10 @@ void CLicqConsole::MenuHistory(char *szArg)
   char *sz = szArg;
   struct SContact scon = GetContactFromArg(&sz);
 
-  if (!scon.szId && scon.nPPID != (unsigned long)-1)
-    winMain->wprintf("%CYou must specify a user to view history.\n", 16);
+  if (scon.szId.empty() && scon.nPPID != (unsigned long)-1)
+    winMain->wprintf("%CYou must specify a user to view history.\n", COLOR_RED);
   else if (scon.nPPID != (unsigned long)-1)
-    UserCommand_History(scon.szId, scon.nPPID, sz);
+    UserCommand_History(scon.szId.c_str(), scon.nPPID, sz);
 }
 
 
@@ -1080,7 +1046,7 @@ void CLicqConsole::MenuSet(char *_szArg)
   }
   if (i == NUM_VARIABLES)
   {
-    winMain->wprintf("%CNo such variable: %A%s\n", 16, A_BOLD, szVariable);
+    winMain->wprintf("%CNo such variable: %A%s\n", COLOR_RED, A_BOLD, szVariable);
     return;
   }
 
@@ -1110,7 +1076,7 @@ void CLicqConsole::MenuSet(char *_szArg)
     }
     if (i == NUM_COLORMAPS)
     {
-      winMain->wprintf("%CNo such color: %A%s\n", 16, A_BOLD, szValue);
+      winMain->wprintf("%CNo such color: %A%s\n", COLOR_RED, A_BOLD, szValue);
       break;
     }
 
@@ -1133,7 +1099,7 @@ void CLicqConsole::MenuSet(char *_szArg)
   case STRING:
     if (szValue[0] != '"' || szValue[strlen(szValue) - 1] != '"')
     {
-      winMain->wprintf("%CString values must be enclosed by double quotes (\").\n", 16);
+      winMain->wprintf("%CString values must be enclosed by double quotes (\").\n", COLOR_RED);
       return;
     }
     szValue[strlen(szValue) - 1] = '\0';
