@@ -1,10 +1,13 @@
 #include "console.h"
-#include "licq_countrycodes.h"
-#include "licq_languagecodes.h"
-#include "licq_filetransfer.h"
 
-#include <string.h>
-#include <time.h>
+#include <cstring>
+#include <ctime>
+
+#include <licq_countrycodes.h>
+#include <licq_languagecodes.h>
+#include <licq_filetransfer.h>
+
+using namespace std;
 
 //======Utilities============================================================
 char *EncodeFileSize(unsigned long nSize)
@@ -130,11 +133,11 @@ void CLicqConsole::PrintStatus()
   werase(winStatus->Win());
 
   unsigned short nNumOwnerEvents = 0;
-  ICQOwner *o = gUserManager.FetchOwner(LOCK_R);
+  ICQOwner* o = gUserManager.FetchOwner(LICQ_PPID, LOCK_R);
   if (o)
   {
     nNumOwnerEvents = o->NewMessages();
-    gUserManager.DropOwner();
+    gUserManager.DropOwner(o);
   }
 
   unsigned short nNumUserEvents = ICQUser::getNumUserEvents() - nNumOwnerEvents;
@@ -145,9 +148,9 @@ void CLicqConsole::PrintStatus()
   else
     strcpy(szMsgStr, "No Messages");
 
-  if (winMain->sLastContact.szId != 0)
+  if (!winMain->sLastContact.szId.empty())
   {
-    ICQUser *u = gUserManager.FetchUser(winMain->sLastContact.szId,
+    ICQUser* u = gUserManager.FetchUser(winMain->sLastContact.szId.c_str(),
       winMain->sLastContact.nPPID, LOCK_R);
     if (u == NULL)
       szLastUser = strdup("<Removed>");
@@ -160,7 +163,7 @@ void CLicqConsole::PrintStatus()
   else
     szLastUser = strdup("<None>");
 
-  o = gUserManager.FetchOwner(LOCK_R);
+  o = gUserManager.FetchOwner(LICQ_PPID, LOCK_R);
   wbkgdset(winStatus->Win(), COLOR_PAIR(COLOR_WHITE));
   mvwhline(winStatus->Win(), 0, 0, ACS_HLINE, COLS);
   //mvwaddch(winStatus->Win(), 0, COLS - USER_WIN_WIDTH - 1, ACS_BTEE);
@@ -168,16 +171,16 @@ void CLicqConsole::PrintStatus()
   wbkgdset(winStatus->Win(), COLOR_PAIR(32));
   if (o)
   {
-    winStatus->wprintf("%C%A[ %C%s %C(%C%ld%C) - S: %C%s %C- G: %C%s %C- M: %C%s %C- L: %C%s %C]", 29,
+    winStatus->wprintf("%C%A[ %C%s %C(%C%s%C) - S: %C%s %C- G: %C%s %C- M: %C%s %C- L: %C%s %C]", 29,
                        A_BOLD, 5,  o->GetAlias(), 29,
-                       5, o->Uin(), 29,
+                       5, o->IdString(), 29,
                        53, o->StatusStr(), 29,
                        53, CurrentGroupName(), 29,
                        53, szMsgStr, 29, 53,
                        szLastUser, 29);
-    gUserManager.DropOwner();
+    gUserManager.DropOwner(o);
   }
-  
+
   wclrtoeol(winStatus->Win());
   winStatus->RefreshWin();
   free(szLastUser);
@@ -203,16 +206,16 @@ void CLicqConsole::PrintGroups()
   waddch(winMain->Win(), ACS_RTEE);
   waddch(winMain->Win(), '\n');
 
-  GroupList *g = gUserManager.LockGroupList(LOCK_R);
-  for (GroupList::iterator i = g->begin(); i != g->end(); i++, j++)
+  FOR_EACH_GROUP_START_SORTED(LOCK_R)
   {
     PrintBoxLeft();
     winMain->wprintf("%A%C%3d. %-19s",
         m_cColorGroupList->nAttr,
-        m_cColorGroupList->nColor, j, *i);
+        m_cColorGroupList->nColor, j, pGroup->name().c_str());
     PrintBoxRight(26);
+    ++j;
   }
-  gUserManager.UnlockGroupList();
+  FOR_EACH_GROUP_END
 
   waddch(winMain->Win(), ACS_LTEE);
   for (k = 0; k < 24; k++) waddch(winMain->Win(), ACS_HLINE);
@@ -286,7 +289,8 @@ void CLicqConsole::CreateUserList()
   FOR_EACH_USER_START(LOCK_R)
   {
     // Only show users on the current group and not on the ignore list
-    if (!pUser->GetInGroup(m_nGroupType, m_nCurrentGroup) ||
+    if ((!pUser->GetInGroup(m_nGroupType, m_nCurrentGroup) &&
+        (m_nGroupType != GROUPS_USER || m_nCurrentGroup != 0)) ||
         (pUser->IgnoreList() && m_nGroupType != GROUPS_SYSTEM && m_nCurrentGroup != GROUP_IGNORE_LIST) )
       FOR_EACH_USER_CONTINUE
 
@@ -372,8 +376,8 @@ void CLicqConsole::CreateUserList()
  *-------------------------------------------------------------------------*/
 void CLicqConsole::PrintUsers()
 {
-  char *title = "<C></B/40>Contacts";
-  char *ulist[1024];
+  const char* title = "<C></B/40>Contacts";
+  const char* ulist[1024];
   int i = 0;
   struct SScrollUser *s = NULL;
   
@@ -402,12 +406,12 @@ void CLicqConsole::PrintUsers()
   if (!cdkUserList)
   {
     cdkUserList = newCDKScroll(winUsers->CDKScreen(), 0, 0, 
-                               RIGHT, LINES - 5, USER_WIN_WIDTH, title,
-                               ulist, i, FALSE, A_NORMAL, TRUE, TRUE);
+                               RIGHT, LINES - 5, USER_WIN_WIDTH, const_cast<char*>(title),
+                               const_cast<char**>(ulist), i, FALSE, A_NORMAL, TRUE, TRUE);
   }
   else
   {
-    setCDKScrollItems (cdkUserList, ulist, i, FALSE);
+    setCDKScrollItems (cdkUserList, const_cast<char**>(ulist), i, FALSE);
   }
   
   bindCDKObject(vSCROLL, cdkUserList, SPACE, CLicqConsole::MenuPopupWrapper, this);
@@ -451,7 +455,7 @@ void CLicqConsole::UserListHighlight(chtype type, chtype input)
 /*---------------------------------------------------------------------------
  * CLicqConsole::UserListCallback
  *-------------------------------------------------------------------------*/
-int CLicqConsole::UserListCallback(EObjectType cdktype, void *object, void *clientData, chtype input)
+int CLicqConsole::UserListCallback(EObjectType /* cdktype */, void* /* object */, void *clientData, chtype input)
 {
   CLicqConsole *me = (CLicqConsole *)clientData; 
   me->UserListHighlight(A_REVERSE, input);
@@ -461,11 +465,11 @@ int CLicqConsole::UserListCallback(EObjectType cdktype, void *object, void *clie
 /*---------------------------------------------------------------------------
  * CLicqConsole::PrintContactPopup
  *-------------------------------------------------------------------------*/
-void CLicqConsole::PrintContactPopup(char *_szAlias)
+void CLicqConsole::PrintContactPopup(const char* _szAlias)
 {
   char title[256];
   snprintf(title, 256, "<C></B/40>%s", _szAlias);
-  char *list[10000];
+  const char* list[10000];
   int i = 0;
 
   list[i++] = "Message";
@@ -473,9 +477,9 @@ void CLicqConsole::PrintContactPopup(char *_szAlias)
 
   cdkContactPopup = newCDKScroll(winMain->CDKScreen(), 0, 0,
                                  RIGHT, 10, 20, title,
-                                 list, i, FALSE, A_REVERSE, TRUE, TRUE);
+                                 const_cast<char**>(list), i, FALSE, A_REVERSE, TRUE, TRUE);
 
-  setCDKScrollBackgroundColor(cdkContactPopup, "</40>");
+  setCDKScrollBackgroundColor(cdkContactPopup, const_cast<char*>("</40>"));
   drawCDKScroll(cdkContactPopup, true);
   winMain->RefreshWin();
 }
@@ -538,7 +542,7 @@ void CLicqConsole::PrintHistory(HistoryList &lHistory, unsigned short nStart,
 {
   HistoryListIter it = lHistory.begin();
   unsigned short n = 0;
-  for (n = 0; n < nStart && it != lHistory.end(); n++, it++);
+  for (n = 0; n < nStart && it != lHistory.end(); n++, it++) ;
   while (n <= nEnd && it != lHistory.end())
   {
     wattron(winMain->Win(), A_BOLD);
@@ -590,8 +594,8 @@ void CLicqConsole::PrintInfo_General(const char *szId, unsigned long nPPID)
   waddch(winMain->Win(), '\n');
   wattroff(winMain->Win(), A_BOLD);
 
-  winMain->wprintf("%s %A(%Z%ld%A) General Info - %Z%s\n", u->GetAlias(), A_BOLD,
-                   A_BOLD, u->Uin(), A_BOLD, A_BOLD, u->StatusStr());
+  winMain->wprintf("%s %A(%Z%s%A) General Info - %Z%s\n", u->GetAlias(), A_BOLD,
+                   A_BOLD, u->IdString(), A_BOLD, A_BOLD, u->StatusStr());
 
   winMain->wprintf("%C%AName: %Z%s %s\n", COLOR_WHITE, A_BOLD, A_BOLD,
                    u->GetFirstName(), u->GetLastName());
@@ -657,8 +661,8 @@ void CLicqConsole::PrintInfo_More(const char *szId, unsigned long nPPID)
   waddch(winMain->Win(), '\n');
   wattroff(winMain->Win(), A_BOLD);
 
-  winMain->wprintf("%s %A(%Z%ld%A) More Info - %Z%s\n", u->GetAlias(), A_BOLD,
-                   A_BOLD, u->Uin(), A_BOLD, A_BOLD, u->StatusStr());
+  winMain->wprintf("%s %A(%Z%s%A) More Info - %Z%s\n", u->GetAlias(), A_BOLD,
+                   A_BOLD, u->IdString(), A_BOLD, A_BOLD, u->StatusStr());
 
   if (u->GetAge() == AGE_UNSPECIFIED)
     winMain->wprintf("%C%AAge: %ZUnspecified\n", COLOR_WHITE, A_BOLD, A_BOLD);
@@ -703,8 +707,8 @@ void CLicqConsole::PrintInfo_Work(const char *szId, unsigned long nPPID)
   waddch(winMain->Win(), '\n');
   wattroff(winMain->Win(), A_BOLD);
 
-  winMain->wprintf("%s %A(%Z%ld%A) Work Info - %Z%s\n", u->GetAlias(), A_BOLD,
-                   A_BOLD, u->Uin(), A_BOLD, A_BOLD, u->StatusStr());
+  winMain->wprintf("%s %A(%Z%s%A) Work Info - %Z%s\n", u->GetAlias(), A_BOLD,
+                   A_BOLD, u->IdString(), A_BOLD, A_BOLD, u->StatusStr());
 
   winMain->wprintf("%C%ACompany Name: %Z%s\n", COLOR_WHITE, A_BOLD, A_BOLD, u->GetCompanyName());
   winMain->wprintf("%C%ACompany Department: %Z%s\n", COLOR_WHITE, A_BOLD, A_BOLD, u->GetCompanyDepartment());
@@ -753,8 +757,8 @@ void CLicqConsole::PrintInfo_About(const char *szId, unsigned long nPPID)
   waddch(winMain->Win(), '\n');
   wattroff(winMain->Win(), A_BOLD);
 
-  winMain->wprintf("%s %A(%Z%ld%A) About Info - %Z%s\n", u->GetAlias(), A_BOLD,
-                    A_BOLD, u->Uin(), A_BOLD, A_BOLD, u->StatusStr());
+  winMain->wprintf("%s %A(%Z%s%A) About Info - %Z%s\n", u->GetAlias(), A_BOLD,
+                    A_BOLD, u->IdString(), A_BOLD, A_BOLD, u->StatusStr());
 
   winMain->wprintf("%s\n", u->GetAbout());
 
@@ -774,15 +778,15 @@ void CLicqConsole::PrintInfo_About(const char *szId, unsigned long nPPID)
 void CLicqConsole::PrintFileStat(CFileTransferManager *ftman)
 {
   // Get the user's name
-  ICQUser *u = gUserManager.FetchUser(ftman->Uin(), LOCK_R);
-  const char *szAlias = u->GetAlias();
-  gUserManager.DropUser(u);
+  ICQUser* u = gUserManager.FetchUser(ftman->Id(), LICQ_PPID, LOCK_R);
 
   // Make the title
   char szTitle[30];
   ftman->Direction() == D_RECEIVER ? strcpy(szTitle, "File from ") :
     strcpy(szTitle, "File to ");
-  strcat(szTitle, szAlias);
+  strcat(szTitle, u->GetAlias());
+
+  gUserManager.DropUser(u);
 
   // Current file name and Current File # slash Total Batch Files
   PrintBoxTop(szTitle, COLOR_WHITE, 48);
