@@ -67,7 +67,7 @@ CWindow::CWindow(int _rows, int _cols, int _y, int _x, int _scrollback, int _use
   }
 
   keypad(win, TRUE);
-  wbkgd(win, COLOR_PAIR(8));
+  wbkgd(win, COLOR_PAIR(COLOR_WHITE));
 
   cdkscreen = NULL;
   if (_useCDK)
@@ -81,16 +81,13 @@ CWindow::CWindow(int _rows, int _cols, int _y, int _x, int _scrollback, int _use
     }
   }
 
-  nLastUin = 0;
-  sLastContact.szId = 0;
+  myLastId.clear();
   sLastContact.nPPID = 0;
   nLastHistory = 1;
 }
 
 CWindow::~CWindow()
 {
-   if (sLastContact.szId)
-     free(sLastContact.szId);
    delwin(win);
 }
 
@@ -168,7 +165,7 @@ CWindow& CWindow::operator<<(unsigned short d)
   return(*this << s);
 }
 
-void CWindow::wprintf(char *formatIn, ...)
+void CWindow::wprintf(const char *formatIn, ...)
 {
    va_list argp;
    char formatOut[1024], out[1024];
