@@ -32,7 +32,7 @@ protected:
   unsigned short m_nSMTPPort;
   char m_szSMTPHost[256], m_szSMTPTo[256], m_szSMTPFrom[256],
        m_szSMTPDomain[256];
-  unsigned long m_nUINTo;
+  char myUserId[16];
   unsigned short m_nForwardType;
 
   CICQDaemon *licqDaemon;
@@ -43,7 +43,7 @@ public:
   void ProcessSignal(CICQSignal *);
   void ProcessEvent(ICQEvent *);
 
-  void ProcessUserEvent(unsigned long, unsigned long);
+  void ProcessUserEvent(const char *, unsigned long, unsigned long);
   bool ForwardEvent(ICQUser *, CUserEvent *);
   bool ForwardEvent_ICQ(ICQUser *, CUserEvent *);
   bool ForwardEvent_Email(ICQUser *, CUserEvent *);
