@@ -144,8 +144,17 @@ void CMSN::ProcessSBPacket(char *szUser, CMSNBuffer *packet, int nSock)
       {
         packet->SkipRN();
         packet->ParseHeaders();
-        
-        string strType = packet->GetValue("Application-Name");
+
+        string application = packet->GetValue("Application-Name");
+        string cookie = packet->GetValue("Invitation-Cookie");
+        string command = packet->GetValue("Invitation-Command");
+
+        if (command == "INVITE")
+        {
+          // Invitation for unknown application, tell inviter that we don't have it
+          gLog.Info("%sInvitation from %s for unknown application (%s).\n", L_MSNxSTR, strUser.c_str(), application.c_str());
+          pReply = new CPS_MSNCancelInvite(cookie, "REJECT_NOT_INSTALLED");
+        }
       }
       else if (strncmp(strType.c_str(), "application/x-msnmsgrp2p", 24) == 0)
       {
@@ -180,6 +189,10 @@ void CMSN::ProcessSBPacket(char *szUser, CMSNBuffer *packet, int nSock)
 	  }
 	}
       }
+      else
+      {
+        gLog.Info("%sMessage from %s with unknown content type (%s).\n", L_MSNxSTR, strUser.c_str(), strType.c_str());
+      }
     }
     else if (strCmd == "ACK")
     {
@@ -197,7 +210,7 @@ void CMSN::ProcessSBPacket(char *szUser, CMSNBuffer *packet, int nSock)
             for (int i = 0; i < pConv->NumUsers(); i++)
             {
               string strUser = pConv->GetUser(i);
-              ICQUser *u = gUserManager.FetchUser(strUser.c_str(), MSN_PPID, LOCK_R);
+              ICQUser* u = gUserManager.FetchUser(strUser.c_str(), MSN_PPID, LOCK_W);
               if (u)
               {
                 e->m_pUserEvent->AddToHistory(u, MSN_PPID, D_SENDER);
@@ -210,7 +223,7 @@ void CMSN::ProcessSBPacket(char *szUser, CMSNBuffer *packet, int nSock)
           }
           else
           {
-            ICQUser *u = gUserManager.FetchUser(e->m_szId, e->m_nPPID, LOCK_R);
+            ICQUser* u = gUserManager.FetchUser(e->m_szId, e->m_nPPID, LOCK_W);
             if (u != NULL)
             {
               e->m_pUserEvent->AddToHistory(u, MSN_PPID, D_SENDER);
@@ -375,9 +388,9 @@ void CMSN::ProcessSBPacket(char *szUser, CMSNBuffer *packet, int nSock)
   //delete packet;
 }
 
-void CMSN::Send_SB_Packet(string &strUser, CMSNPacket *p, int nSocket, bool bDelete)
+void CMSN::Send_SB_Packet(const string &strUser, CMSNPacket *p, int nSocket, bool bDelete)
 {
-  ICQUser *u = gUserManager.FetchUser(const_cast<char *>(strUser.c_str()), MSN_PPID, LOCK_R);
+  const ICQUser* u = gUserManager.FetchUser(strUser.c_str(), MSN_PPID, LOCK_R);
   if (!u) return;
 
   int nSock = nSocket != -1 ? nSocket : u->SocketDesc(ICQ_CHNxNONE);
@@ -397,7 +410,7 @@ void CMSN::Send_SB_Packet(string &strUser, CMSNPacket *p, int nSocket, bool bDel
     m_pDaemon->RemoveUserConversation(nSock, strUser.c_str());
     CConversation *pConv = m_pDaemon->FindConversation(nSock);
 
-    ICQUser *u = gUserManager.FetchUser(const_cast<char *>(strUser.c_str()), MSN_PPID, LOCK_W);
+    ICQUser* u = gUserManager.FetchUser(strUser.c_str(), MSN_PPID, LOCK_W);
     if (u)
     {
       u->ClearSocketDesc(ICQ_CHNxNONE);
@@ -423,7 +436,7 @@ void CMSN::Send_SB_Packet(string &strUser, CMSNPacket *p, int nSocket, bool bDel
     delete p;
 }
 
-bool CMSN::MSNSBConnectStart(string &strServer, string &strCookie)
+bool CMSN::MSNSBConnectStart(const string &strServer, const string &strCookie)
 {
   const char *szParam = strServer.c_str();
   char szServer[16];
@@ -485,8 +498,8 @@ bool CMSN::MSNSBConnectStart(string &strServer, string &strCookie)
   return true;
 }
 
-bool CMSN::MSNSBConnectAnswer(string &strServer, string &strSessionId, string &strCookie,
-                              string &strUser)
+bool CMSN::MSNSBConnectAnswer(const string& strServer, const string& strSessionId,
+    const string& strCookie, const string& strUser)
 {
   const char *szParam = strServer.c_str();
   char szServer[16];
@@ -540,9 +553,9 @@ bool CMSN::MSNSBConnectAnswer(string &strServer, string &strSessionId, string &s
   return true;
 }
 
-void CMSN::MSNSendInvitation(char *_szUser, CMSNPacket *_pPacket)
+void CMSN::MSNSendInvitation(const char* _szUser, CMSNPacket* _pPacket)
 {
-  //ICQUser *u = gUserManager.FetchUser(_szUser, MSN_PPID, LOCK_R);
+  //const ICQUser* u = gUserManager.FetchUser(_szUser, MSN_PPID, LOCK_R);
   //if (!u) return;
   //gUserManager.DropUser(u);
 
@@ -564,7 +577,8 @@ void CMSN::MSNSendInvitation(char *_szUser, CMSNPacket *_pPacket)
   SendPacket(pSB);
 }
 
-void CMSN::MSNSendMessage(char *_szUser, char *_szMsg, pthread_t _tPlugin, unsigned long _nCID)
+void CMSN::MSNSendMessage(const char* _szUser, const char* _szMsg,
+    pthread_t _tPlugin, unsigned long _nCID)
 {
   string strUser(_szUser);
   int nSocket = -1;
@@ -575,8 +589,8 @@ void CMSN::MSNSendMessage(char *_szUser, char *_szMsg, pthread_t _tPlugin, unsig
     if (pConv)
       nSocket = pConv->Socket();
   } 
-  
-  ICQUser *u = gUserManager.FetchUser(_szUser, MSN_PPID, LOCK_R);
+
+  const ICQUser* u = gUserManager.FetchUser(_szUser, MSN_PPID, LOCK_R);
   if (!u) return;
   gUserManager.DropUser(u);
   
@@ -618,7 +632,7 @@ void CMSN::MSNSendMessage(char *_szUser, char *_szMsg, pthread_t _tPlugin, unsig
   }  
 }
 
-void CMSN::MSNSendTypingNotification(char *_szUser, unsigned long _nCID)
+void CMSN::MSNSendTypingNotification(const char* _szUser, unsigned long _nCID)
 {
   string strUser(_szUser);
   CMSNPacket *pSend = new CPS_MSNTypingNotification(m_szUserName);
