@@ -104,7 +104,7 @@ void CMSN::ProcessServerPacket(CMSNBuffer *packet)
         // Set our alias here
         ICQOwner *o = gUserManager.FetchOwner(MSN_PPID, LOCK_W);
         o->SetAlias(strDecodedNick.c_str());
-        gUserManager.DropOwner(MSN_PPID);
+        gUserManager.DropOwner(o);
          
         // This cookie doesn't work anymore now that we are online
         if (m_szCookie)
@@ -219,12 +219,12 @@ void CMSN::ProcessServerPacket(CMSNBuffer *packet)
         ICQOwner *o = gUserManager.FetchOwner(MSN_PPID, LOCK_W);
         if (m_pDaemon->AddUserEvent(o, e))
         {
-          gUserManager.DropOwner(MSN_PPID);
+          gUserManager.DropOwner(o);
           e->AddToHistory(NULL, MSN_PPID, D_RECEIVER);
           m_pDaemon->m_xOnEventManager.Do(ON_EVENT_SYSMSG, NULL);
         }
         else
-          gUserManager.DropOwner(MSN_PPID);
+          gUserManager.DropOwner(o);
       }
       else
       {
@@ -267,7 +267,7 @@ void CMSN::ProcessServerPacket(CMSNBuffer *packet)
         ICQOwner *o = gUserManager.FetchOwner(MSN_PPID, LOCK_W);
         string strDecodedNick = Decode(strNick);
         o->SetAlias(strDecodedNick.c_str());
-        gUserManager.DropOwner(MSN_PPID);
+        gUserManager.DropOwner(o);
       }
       
       gLog.Info("%s%s renamed successfully.\n", L_MSNxSTR, strUser.c_str());
@@ -276,7 +276,7 @@ void CMSN::ProcessServerPacket(CMSNBuffer *packet)
     {
       packet->SkipParameter(); // seq
       string strStatus = packet->GetParameter();
-      ICQUser *o = gUserManager.FetchOwner(MSN_PPID, LOCK_W);
+      ICQOwner* o = gUserManager.FetchOwner(MSN_PPID, LOCK_W);
       unsigned long nStatus;
       bool bHidden = false;
       
@@ -295,7 +295,7 @@ void CMSN::ProcessServerPacket(CMSNBuffer *packet)
       m_pDaemon->ChangeUserStatus(o, nStatus);
       m_nStatus = nStatus;
       gLog.Info("%sServer says we are now: %s\n", L_MSNxSTR, ICQUser::StatusToStatusStr(o->Status(), bHidden));
-      gUserManager.DropOwner(MSN_PPID);
+      gUserManager.DropOwner(o);
     }
     else if (strCmd == "ILN" || strCmd == "NLN")
     {
@@ -441,12 +441,12 @@ void CMSN::ProcessServerPacket(CMSNBuffer *packet)
         ICQOwner *o = gUserManager.FetchOwner(MSN_PPID, LOCK_W);
         if (m_pDaemon->AddUserEvent(o, pEmailAlert))
         {
-          gUserManager.DropOwner(MSN_PPID);
+          gUserManager.DropOwner(o);
           pEmailAlert->AddToHistory(NULL, MSN_PPID, D_RECEIVER);
           m_pDaemon->m_xOnEventManager.Do(ON_EVENT_SYSMSG, NULL);
         }
         else
-          gUserManager.DropOwner(MSN_PPID);
+          gUserManager.DropOwner(o);
       }
     }
     else if (strCmd == "QNG")
@@ -530,7 +530,7 @@ void CMSN::MSNLogon(const char *_szServer, int _nPort, unsigned long _nStatus)
   if (_nStatus == ICQ_STATUS_OFFLINE)
     return;
 
-  ICQOwner *o = gUserManager.FetchOwner(MSN_PPID, LOCK_R);
+  const ICQOwner* o = gUserManager.FetchOwner(MSN_PPID, LOCK_R);
   if (!o)
   {
     gLog.Error("%sNo owner set.\n", L_MSNxSTR);
@@ -538,8 +538,8 @@ void CMSN::MSNLogon(const char *_szServer, int _nPort, unsigned long _nStatus)
   }
   m_szUserName = strdup(o->IdString());
   m_szPassword = strdup(o->Password());
-  gUserManager.DropOwner(MSN_PPID);
-  
+  gUserManager.DropOwner(o);
+
   SrvSocket *sock = new SrvSocket(m_szUserName, MSN_PPID);
   sock->SetRemoteAddr(_szServer, _nPort);
   char ipbuf[32];
@@ -607,11 +607,11 @@ void CMSN::MSNLogoff(bool bDisconnected)
     
   ICQOwner *o = gUserManager.FetchOwner(MSN_PPID, LOCK_W);      
   m_pDaemon->ChangeUserStatus(o, ICQ_STATUS_OFFLINE);
-  gUserManager.DropOwner(MSN_PPID);  
+  gUserManager.DropOwner(o);
   //m_pDaemon->PushPluginSignal(new CICQSignal(SIGNAL_LOGOFF, 0, 0));   
 }
 
-void CMSN::MSNAddUser(char *szUser)
+void CMSN::MSNAddUser(const char* szUser)
 {
   ICQUser *u = gUserManager.FetchUser(szUser, MSN_PPID, LOCK_W);
   u->SetEnableSave(false);
@@ -624,32 +624,31 @@ void CMSN::MSNAddUser(char *szUser)
   SendPacket(pSend);
 }
 
-void CMSN::MSNRemoveUser(char *szUser)
+void CMSN::MSNRemoveUser(const char* szUser)
 {
   CMSNPacket *pSend = new CPS_MSNRemoveUser(szUser, CONTACT_LIST);
   SendPacket(pSend);
 }
 
-void CMSN::MSNRenameUser(char *szUser)
+void CMSN::MSNRenameUser(const char* szUser)
 {
-  ICQUser *u = gUserManager.FetchUser(szUser, MSN_PPID, LOCK_R);
+  const ICQUser* u = gUserManager.FetchUser(szUser, MSN_PPID, LOCK_R);
   if (!u) return;
-  char *szNewNick = u->GetAlias();
+  string strNick = u->GetAlias();
   gUserManager.DropUser(u);
 
-  string strNick(szNewNick);
   string strEncodedNick = Encode(strNick);
   CMSNPacket *pSend = new CPS_MSNRenameUser(szUser, strEncodedNick.c_str());
   SendPacket(pSend);
 }
 
-void CMSN::MSNGrantAuth(char *szUser)
+void CMSN::MSNGrantAuth(const char* szUser)
 {
   CMSNPacket *pSend = new CPS_MSNAddUser(szUser, ALLOW_LIST);
   SendPacket(pSend);
 }
 
-void CMSN::MSNUpdateUser(char *szAlias)
+void CMSN::MSNUpdateUser(const char* szAlias)
 {
   string strNick(szAlias);
   string strEncodedNick = Encode(strNick);
@@ -657,7 +656,7 @@ void CMSN::MSNUpdateUser(char *szAlias)
   SendPacket(pSend);
 }
 
-void CMSN::MSNBlockUser(char *szUser)
+void CMSN::MSNBlockUser(const char* szUser)
 {
   ICQUser *u = gUserManager.FetchUser(szUser, MSN_PPID, LOCK_W);
   if (u)
@@ -676,7 +675,7 @@ void CMSN::MSNBlockUser(char *szUser)
   SendPacket(pAdd);
 }
 
-void CMSN::MSNUnblockUser(char *szUser)
+void CMSN::MSNUnblockUser(const char* szUser)
 {
   ICQUser *u = gUserManager.FetchUser(szUser, MSN_PPID, LOCK_W);
   if (u)
@@ -701,7 +700,7 @@ void CMSN::MSNGetDisplayPicture(const string &strUser, const string &strMSNObjec
   if (m_nStatus & ICQ_STATUS_FxPRIVATE)
     return;
 
-  char *szUser = const_cast<char *>(strUser.c_str());
+  const char *szUser = const_cast<const char *>(strUser.c_str());
   CMSNPacket *pGetMSNDP = new CPS_MSNInvitation(szUser,
 						m_szUserName,
 						const_cast<char *>(strMSNObject.c_str()));
@@ -739,7 +738,7 @@ void *MSNPing_tep(void *p)
       gLog.Info("%sPing timeout. Reconnecting...\n", L_MSNxSTR);
       pMSN->SetWaitingPingReply(false);
       pMSN->MSNLogoff();
-      pMSN->MSNLogon("messenger.hotmail.com", 1863);
+      pMSN->MSNLogon(pMSN->serverAddress().c_str(), pMSN->serverPort());
       pthread_mutex_unlock(&(pMSN->mutex_ServerSocket));
     }
     else if (pMSN->CanSendPing())
