@@ -20,28 +20,30 @@
 #include "config.h"
 #endif
 
-#include "msn.h"
-#include "msnpacket.h"
+#include <cctype>
+#include <cstdio>
+#include <list>
+#include <openssl/md5.h>
+#include <pthread.h>
+#include <sys/select.h>
+#include <sys/time.h>
+#include <sys/types.h>
+#include <string>
+#include <unistd.h>
+#include <vector>
+
 #include "licq_log.h"
 #include "licq_message.h"
 
-#include <openssl/md5.h>
-
-#include <string>
-#include <list>
-#include <vector>
+#include "msn.h"
+#include "msnpacket.h"
 
 using namespace std;
 
 #ifndef HAVE_STRNDUP
 
-#include <stdlib.h>
-#include <pthread.h>
-
-#include <sys/select.h>
-#include <sys/time.h>
-#include <sys/types.h>
-#include <unistd.h>
+#include <cstdlib>
+#include <cstring>
 
 char *strndup(const char *s, size_t n)
 {
@@ -93,11 +95,15 @@ CMSN::CMSN(CICQDaemon *_pDaemon, int _nPipe) : m_vlPacketBucket(211)
     fclose(f);
     msnConf.LoadFile(szFileName);
   }  
-  
+
+  char tmpStr[MAX_LINE_LEN];
+
   msnConf.SetSection("network");
-  
   msnConf.ReadNum("ListVersion", m_nListVersion, 0);
-  
+  msnConf.ReadStr("MsnServerAddress", tmpStr, MSN_DEFAULT_SERVER_ADDRESS);
+  myServerAddress = tmpStr;
+  msnConf.ReadNum("MsnServerPort", myServerPort, MSN_DEFAULT_SERVER_PORT);
+
   msnConf.CloseFile();
 
   // pthread stuff now
@@ -141,7 +147,7 @@ void CMSN::StorePacket(SBuffer *_pBuf, int _nSock)
   }
 }
 
-void CMSN::RemovePacket(string _strUser, int _nSock, int nSize)
+void CMSN::RemovePacket(const string& _strUser, int _nSock, int nSize)
 {
   pthread_mutex_lock(&mutex_Bucket);
   BufferList &b = m_vlPacketBucket[HashValue(_nSock)];
@@ -241,11 +247,17 @@ void CMSN::HandlePacket(int _nSocket, CMSNBuffer &packet, const char* _szUser)
     if ((szNeedle = strstr((char *)pBuf->m_pBuf->getDataStart(),
                            "\r\n")))
     {
-      if (memcmp(pBuf->m_pBuf->getDataStart(), "MSG", 3) == 0)
+      bool isMSG = (memcmp(pBuf->m_pBuf->getDataStart(), "MSG", 3) == 0);
+
+      if (/*memcmp(pBuf->m_pBuf->getDataStart(), "MSG", 3) == 0*/ isMSG ||
+          memcmp(pBuf->m_pBuf->getDataStart(), "NOT", 3) == 0)
       {
-        pBuf->m_pBuf->SkipParameter(); // MSG
-        pBuf->m_pBuf->SkipParameter(); // Hotmail
-        pBuf->m_pBuf->SkipParameter(); // Hotmail
+        pBuf->m_pBuf->SkipParameter(); // MSG, NOT
+        if (isMSG)
+        {
+          pBuf->m_pBuf->SkipParameter(); // Hotmail
+          pBuf->m_pBuf->SkipParameter(); // Hotmail
+        }
         string strSize = pBuf->m_pBuf->GetParameter();
         int nSize = atoi(strSize.c_str());
 
@@ -452,7 +464,7 @@ void CMSN::Run()
             m_nServerSocket = -1;
             gSocketMan.DropSocket(sock);
             gSocketMan.CloseSocket(nSD);
-            MSNLogon("messenger.hotmail.com", 1863, m_nStatus);
+            MSNLogon(myServerAddress.c_str(), myServerPort, m_nStatus);
           }
         }
         
@@ -553,7 +565,7 @@ void CMSN::ProcessSignal(CSignal *s)
       if (m_nServerSocket < 0)
       {
         CLogonSignal *sig = static_cast<CLogonSignal *>(s);
-        MSNLogon("messenger.hotmail.com", 1863, sig->LogonStatus());
+        MSNLogon(myServerAddress.c_str(), myServerPort, sig->LogonStatus());
       }
       break;
     }
@@ -617,13 +629,13 @@ void CMSN::ProcessSignal(CSignal *s)
 
     case PROTOxSENDxREFUSExAUTH:
     {
-      CRefuseAuthSignal *sig = static_cast<CRefuseAuthSignal *>(s);
+//      CRefuseAuthSignal *sig = static_cast<CRefuseAuthSignal *>(s);
       break;
     }
 
     case PROTOxREQUESTxINFO:
     {
-      CRequestInfo *sig = static_cast<CRequestInfo *>(s);
+//      CRequestInfo *sig = static_cast<CRequestInfo *>(s);
       break;
     }
 
