@@ -23,6 +23,11 @@
 #include "config.h"
 #endif
 
+#include <list>
+#include <pthread.h>
+#include <string>
+#include <vector>
+
 #include "licq_events.h"
 #include "licq_icqd.h"
 #include "licq_socket.h"
@@ -42,20 +47,13 @@ const unsigned short FLAG_ALLOW_LIST   = 2;
 const unsigned short FLAG_BLOCK_LIST   = 4;
 const unsigned short FLAG_REVERSE_LIST = 8;
 
+const char MSN_DEFAULT_SERVER_ADDRESS[]         = "messenger.hotmail.com";
+const unsigned short MSN_DEFAULT_SERVER_PORT    = 1863;
+
 #ifndef HAVE_STRNDUP
 char *strndup(const char *s, size_t n);
 #endif
 
-#include <string>
-#include <list>
-#include <vector>
-#include <cctype>
-#include <pthread.h>
-
-using std::string;
-using std::list;
-using std::vector;
-using std::isalnum;
 
   const unsigned long MSN_DP_EVENT = 1;
 
@@ -65,11 +63,11 @@ class CMSNDataEvent;
 struct SBuffer
 {
   CMSNBuffer *m_pBuf;
-  string m_strUser;
+  std::string m_strUser;
   bool m_bStored;
 };
 
-typedef list<SBuffer *> BufferList;
+typedef std::list<SBuffer *> BufferList;
 
 struct SStartMessage
 {
@@ -82,7 +80,7 @@ struct SStartMessage
        m_bDataConnection;
 };
 
-typedef list<SStartMessage *> StartList;
+typedef std::list<SStartMessage*> StartList;
 
 class CMSN
 {
@@ -91,16 +89,19 @@ public:
   ~CMSN();
 
   void Run();
-  
+
   void MSNPing();
   bool Connected() { return m_nServerSocket != -1; }
   bool CanSendPing() { return m_bCanPing; }
   void MSNLogoff(bool = false);
   void MSNLogon(const char *, int);
- 
+
   bool WaitingPingReply()          { return m_bWaitingPingReply; }
   void SetWaitingPingReply(bool b) { m_bWaitingPingReply = b; }
 
+  const std::string& serverAddress() const   { return myServerAddress; }
+  unsigned short serverPort() const     { return myServerPort; }
+
   pthread_mutex_t mutex_ServerSocket; // Ugly, but whatever.
 
 private:
@@ -110,51 +111,56 @@ private:
   void ProcessNexusPacket(CMSNBuffer &);
   void ProcessSSLServerPacket(CMSNBuffer &);
   void ProcessSBPacket(char *, CMSNBuffer *, int);
-  
+
   // Network functions
   void SendPacket(CMSNPacket *);
-  void Send_SB_Packet(string &, CMSNPacket *, int = -1, bool = true);
+  void Send_SB_Packet(const std::string& user, CMSNPacket* p, int nSocket = -1,
+      bool bDelete = true);
   void MSNLogon(const char *, int, unsigned long);
   void MSNGetServer();
-  void MSNAuthenticateRedirect(string &, string &);
+  void MSNAuthenticateRedirect(const std::string& host, const std::string& param);
   void MSNAuthenticate(char *);
-  bool MSNSBConnectStart(string &, string &);
-  bool MSNSBConnectAnswer(string &, string &, string &, string &);
+  bool MSNSBConnectStart(const std::string& server, const std::string& cookie);
+  bool MSNSBConnectAnswer(const std::string& server, const std::string& sessionId,
+      const std::string& cookie, const std::string& user);
 
-  void MSNSendInvitation(char *, CMSNPacket *);
-  void MSNSendMessage(char *, char *, pthread_t, unsigned long);
-  void MSNSendTypingNotification(char *, unsigned long);
+  void MSNSendInvitation(const char* _szUser, CMSNPacket* _pPacket);
+  void MSNSendMessage(const char* _szUser, const char* _szMsg,
+      pthread_t _tPlugin, unsigned long _nCID);
+  void MSNSendTypingNotification(const char* _szUser, unsigned long _nCID);
   void MSNChangeStatus(unsigned long);
-  void MSNAddUser(char *);
-  void MSNRemoveUser(char *);
-  void MSNRenameUser(char *);
-  void MSNGrantAuth(char *);
-  void MSNUpdateUser(char *);
-  void MSNBlockUser(char *);
-  void MSNUnblockUser(char *);
-  void MSNGetDisplayPicture(const string &, const string &);
+  void MSNAddUser(const char* szUser);
+  void MSNRemoveUser(const char* szUser);
+  void MSNRenameUser(const char* szUser);
+  void MSNGrantAuth(const char* szUser);
+  void MSNUpdateUser(const char* szUser);
+  void MSNBlockUser(const char* szUser);
+  void MSNUnblockUser(const char* szUser);
+  void MSNGetDisplayPicture(const std::string& user, const std::string& msnObject);
 
   // Internal functions
   int HashValue(int n) { return n % 211; }
   void StorePacket(SBuffer *, int);
-  void RemovePacket(string, int, int = 0);
-  SBuffer *RetrievePacket(const string &, int);
+  void RemovePacket(const std::string& user, int socketId, int size = 0);
+  SBuffer *RetrievePacket(const std::string& user, int socketId);
   ICQEvent *RetrieveEvent(unsigned long);
   void HandlePacket(int, CMSNBuffer &, const char *);
   unsigned long SocketToCID(int);
-  static string Decode(const string &);
-  static string Encode(const string &);
+  static std::string Decode(const std::string& strIn);
+  static std::string Encode(const std::string& strIn);
   void WaitDataEvent(CMSNDataEvent *);
   bool RemoveDataEvent(CMSNDataEvent *);
-  CMSNDataEvent *FetchDataEvent(const string &, int);
-  CMSNDataEvent *FetchStartDataEvent(const string &);
+  CMSNDataEvent* FetchDataEvent(const std::string& user, int socketId);
+  CMSNDataEvent* FetchStartDataEvent(const std::string& user);
 
   // Interface to CICQDaemon
   void PushPluginSignal(CICQSignal *);
 
   // Config
   unsigned long m_nListVersion;
-    
+  std::string myServerAddress;
+  unsigned short myServerPort;
+
   // Variables
   CICQDaemon *m_pDaemon;
   bool m_bExit;
@@ -165,9 +171,9 @@ private:
   CMSNBuffer *m_pPacketBuf,
              *m_pNexusBuff,
              *m_pSSLPacket;
-  vector<BufferList> m_vlPacketBucket;
-  list<ICQEvent *> m_pEvents;
-  list<CMSNDataEvent *> m_lMSNEvents;
+  std::vector<BufferList> m_vlPacketBucket;
+  std::list<ICQEvent*> m_pEvents;
+  std::list<CMSNDataEvent*> m_lMSNEvents;
   StartList m_lStart;
   bool m_bWaitingPingReply,
        m_bCanPing;
@@ -176,7 +182,7 @@ private:
   unsigned long m_nStatus,
                 m_nOldStatus,
                 m_nSessionStart;
-  string m_strMSPAuth,
+  std::string m_strMSPAuth,
          m_strSID,
          m_strKV;
          
