@@ -2,25 +2,28 @@
 // author: Martin Maurer (martinmaurer@gmx.at)
 // first version 7 Apr 2003
 
+#include <cerrno>
+#include <cstdio>
+#include <cstring>
+#include <ctime>
+#include <iconv.h>
+#include <iostream>
+#include <langinfo.h>
+#include <libintl.h>
+#include <locale.h>
+#include <unistd.h>
+
 #include <licq_plugin.h>
 #include <licq_icqd.h>
 #include <licq_events.h>
 #include <licq_file.h>
 #include <licq_log.h>
-#include <time.h>
-#include <unistd.h>
-#include <iostream>
-#include <iconv.h>
-#include <errno.h>
-#include <locale.h>
-#include <langinfo.h>
-#include <cstdio>
-#include <cstring>
-#include <libintl.h>
 
 #include "my_xosd.h"
 #include "licq_osd.conf.h"
-                                 
+
+using namespace std;
+
 //#if CVSLICQ==1
 //#warning compiling for licq>=1.2.8
 //#else
@@ -76,7 +79,7 @@ void ProcessSignal(CICQSignal *s);
 void ProcessEvent(ICQEvent *e);
 #ifdef CP_TRANSLATE
     const char *get_iconv_encoding_name(const char *licq_encoding_name);
-    char *my_translate(unsigned long uin, const char *msg, char *userenc);
+char* my_translate(unsigned long uin, const char* msg, const char* userenc);
 #endif
 
 // some variables representing the internal state
@@ -142,12 +145,13 @@ void log(int mode, const char *message)
 }
 
 // issue a few warnings for wrong config values.
-void verifyconfig(string pluginfont, unsigned long timeout, unsigned long hoffset,
-		  unsigned long voffset, string vpos, string hpos, unsigned long lines,
-		  unsigned long linelen, unsigned long quiettimeout, string colour,
-		  bool wait, unsigned long showmessages, unsigned long showlogon,
-		  unsigned long shadowoffset, unsigned long outlineoffset, string shadowcolour,
-		  string outlinecolour, string localencoding)
+void verifyconfig(string pluginfont, unsigned long /* timeout */,
+    unsigned long hoffset, unsigned long voffset, string vpos, string hpos,
+    unsigned long lines, unsigned long linelen, unsigned long quiettimeout,
+    string colour, bool /* wait */, unsigned long showmessages,
+    unsigned long showlogon, unsigned long shadowoffset,
+    unsigned long outlineoffset, string shadowcolour, string outlinecolour,
+    string localencoding)
 {
     try {
 	if ((pluginfont=="") || (pluginfont.at(0)=='"') || (pluginfont.at(0)=='\''))
@@ -211,7 +215,7 @@ unsigned long parseShowInModesStr(char *ShowInModesStr)
 }
 
 // called once on Load of the plugin
-bool LP_Init(int argc, char **argv)
+bool LP_Init(int /* argc */, char** /* argv */)
 {
     char ShowInModesStr[MAX_LINE_LEN+1];
     char ShowMsgsInModesStr[MAX_LINE_LEN+1];
@@ -423,7 +427,7 @@ void ProcessSignal(CICQSignal *s)
     bool want_osd_msgs_only=false; // though we don't want OSD we want msgs
     bool secure=false;
     unsigned long status=0;
-    char *userencoding=0;
+  const char* userencoding = NULL;
     CUserEvent *e=0;
 
     switch (s->Signal()) // signaltype
@@ -451,9 +455,9 @@ void ProcessSignal(CICQSignal *s)
 
             if (want_osd)
 	    {
-		o=gUserManager.FetchOwner(LOCK_R);
-		if (o)
-		{
+        o = gUserManager.FetchOwner(LICQ_PPID, LOCK_R);
+        if (o != NULL)
+        {
 		    status=o->Status();
 		    //want_osd=true;
 
@@ -493,14 +497,14 @@ void ProcessSignal(CICQSignal *s)
 			}
 		    }
 
-		    gUserManager.DropOwner();
-		}
+          gUserManager.DropOwner(o);
+        }
 	    }
 
 	    if (want_osd)
 	    {
 //#ifdef CVSLICQ
-		char *szId = s->Id();
+		const char* szId = s->Id();
 		unsigned long nPPID = s->PPID();
 		u = gUserManager.FetchUser(szId, nPPID, LOCK_R);
 //#else
@@ -552,7 +556,7 @@ void ProcessSignal(CICQSignal *s)
 		want_osd=false;
 	    if (ignore)
 		want_osd=false;
-	    if (s->Uin() == gUserManager.OwnerUin()) // no messages for our own actions
+	    if (gUserManager.FindOwner(s->Id(), s->PPID()) != NULL) // no messages for our own actions
                 want_osd=false;
 
 	    // user checked our auto-response
@@ -784,7 +788,7 @@ const char *get_iconv_encoding_name(const char *licq_encoding_name)
 // the other user. (change it for example via the licq-qt-gui message window)
 // CICQSignal is needed to get the User for this message -
 // some day i will do this more elegant
-char *my_translate(unsigned long uin, const char *msg, char *userenc)
+char* my_translate(unsigned long /* uin */, const char* msg, const char* userenc)
 {
     // will be deleted outside of this function
     char *result = (char*)malloc(strlen(msg) + 1);
@@ -836,7 +840,7 @@ char *my_translate(unsigned long uin, const char *msg, char *userenc)
 		tosize += fromsize + 4;
 		continue;
 	    }
-	    gLog.Warn("%sError in my_translate - stopping translation, error on %i. char\n", L_OSD_STR, msgptr-msg+1);
+	    gLog.Warn("%sError in my_translate - stopping translation, error on %li. char\n", L_OSD_STR, msgptr-msg+1);
 	    strcpy(result, msg); // return original string
 	    return result;
 	}
