@@ -34,8 +34,6 @@ class AuthUserDlg : public LicqDialog
 {
    Q_OBJECT
 public:
-   AuthUserDlg (CICQDaemon *s, unsigned long nUin, bool bGrant,
-      QWidget *parent = 0);
    AuthUserDlg (CICQDaemon *s, const char *szId, unsigned long nPPID,
       bool bGrant, QWidget *parent = 0);
 
@@ -47,7 +45,6 @@ protected:
    QLineEdit *edtUin;
    MLEditWrap* mleResponse;
 
-   unsigned long m_nUin;
    char *m_szId;
    unsigned long m_nPPID;
    bool m_bGrant;
