@@ -1,18 +1,20 @@
 /*
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ * This file is part of Licq, an instant messaging client for UNIX.
+ * Copyright (C) 2004-2006 Licq developers
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ * Licq is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * Licq is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
  *
+ * You should have received a copy of the GNU General Public License
+ * along with Licq; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
 #include <qlineedit.h>
@@ -110,14 +112,14 @@ EditCategoryDlg::EditCategoryDlg(QWidget *parent, ICQUserCategory *uCat)
 
 void EditCategoryDlg::ok()
 {
-  ICQOwner* o = gUserManager.FetchOwner(LOCK_R);
+  ICQOwner* o = gUserManager.FetchOwner(LICQ_PPID, LOCK_R);
   if (o == NULL)
   {
     close();
     return;
   }
   QTextCodec *codec = UserCodec::codecForICQUser(o);
-  gUserManager.DropOwner();
+  gUserManager.DropOwner(o);
 
   ICQUserCategory *cat = new ICQUserCategory(m_uc);
   for (unsigned short i = 0; i < m_nCats; i++)
@@ -135,7 +137,7 @@ void EditCategoryDlg::ok()
 
 //------------------------------------------------------------------------------
 
-void EditCategoryDlg::checkEnabled(int index)
+void EditCategoryDlg::checkEnabled(int /* index */)
 {
   for (unsigned short i = 0; i < m_nCats; i++)
     leDescr[i]->setEnabled(cbCat[i]->currentItem() != 0);
