@@ -663,12 +663,21 @@ void CInfoField::setData(QString data)
   setText(data);
 }
 
-
-void CInfoField::setData(const unsigned long data)
+void CInfoField::setData(unsigned long data)
 {
-  char t[32];
-  sprintf(t, "%lu", data);
-  setData(t);
+  setText(QString::number(data));
+}
+
+void CInfoField::setDateTime(uint timestamp)
+{
+  if (timestamp == 0)
+    setText(tr("Unknown"));
+  else
+  {
+    QDateTime t;
+    t.setTime_t(timestamp);
+    setText(t.toString());
+  }
 }
 
 void CInfoField::keyPressEvent(QKeyEvent *e)
@@ -828,57 +837,6 @@ CMessageViewWidget::CMessageViewWidget(const char *szId, unsigned long nPPID,
 */
 }
 
-CMessageViewWidget::CMessageViewWidget(unsigned long _nUin, CMainWindow *m, QWidget *parent, const char *name, bool historyMode)
-  : MLView(parent,name)
-{
-  setTextFormat(RichText);
-  m_nUin= _nUin;
-  m_szId = NULL; // avoid desalocation error at destructor
-  if (historyMode)
-  {
-    m_useBuffer = true;
-    m_nMsgStyle = m->m_histMsgStyle;
-    m_nDateFormat = m->m_histDateFormat;
-    m_extraSpacing = m->m_histVertSpacing;
-    m_appendLineBreak = false;
-  }
-  else
-  {
-    m_useBuffer = false;
-    m_nMsgStyle = m->m_chatMsgStyle;
-    m_nDateFormat = m->m_chatDateFormat;
-    m_extraSpacing = m->m_chatVertSpacing;
-    m_appendLineBreak = m->m_chatAppendLineBreak;
-  }
-  m_showNotices = m->m_showNotices;
-  m_colorRcv = m->m_colorRcv;
-  m_colorSnt = m->m_colorSnt;
-  m_colorRcvHistory = m->m_colorRcvHistory;
-  m_colorSntHistory = m->m_colorSntHistory;
-  m_colorNotice = m->m_colorNotice;
-  setPaletteBackgroundColor(m->m_colorChatBkg);
-  mainwin = m;
-
-  clear();
-
-/*
-  // add all unread messages.
-  vector<CUserEvent*> newEventList;
-  ICQUser *u = gUserManager.FetchUser(_nUin, LOCK_W);
-  if (u != NULL && u->NewMessages() > 0)
-  {
-    for (unsigned short i = 0; i < u->NewMessages(); i++)
-    {
-      CUserEvent *e = u->EventPeek(i);
-      if (e->Direction() == D_RECEIVER && e->SubCommand() == ICQ_CMDxSUB_MSG)
-	newEventList.push_back(e);
-    }
-  }
-  gUserManager.DropUser(u);
-  for (unsigned short i = 0; i < newEventList.size(); i++)
-    addMsg(newEventList[i]); */
-}
-
 CMessageViewWidget::~CMessageViewWidget()
 {
   if (m_szId)
@@ -1093,7 +1051,7 @@ void CMessageViewWidget::addMsg(direction dir, bool fromHistory, QString eventDe
   internalAddMsg(s);
 }
 
-void CMessageViewWidget::addMsg(CUserEvent* e, const char *_szId, unsigned long _nPPID)
+void CMessageViewWidget::addMsg(const CUserEvent* e, const char* _szId, unsigned long _nPPID)
 {
   QDateTime date;
   date.setTime_t(e->Time());
@@ -1133,7 +1091,7 @@ void CMessageViewWidget::addMsg(CUserEvent* e, const char *_szId, unsigned long
       // since we're using the contact's encoding, not ours.
       QTextCodec *ownerCodec = UserCodec::codecForICQUser(o);
       contactName = ownerCodec->toUnicode(o->GetAlias());
-      gUserManager.DropOwner(m_nPPID);
+      gUserManager.DropOwner(o);
     }
   }
 
