@@ -1,6 +1,27 @@
+/*
+ * This file is part of Licq, an instant messaging client for UNIX.
+ * Copyright (C) 1999-2006 Licq developers
+ *
+ * Licq is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * Licq is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with Licq; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
 #ifndef EWIDGETS_H
 #define EWIDGETS_H
 
+#include <list>
+
 #include <qlabel.h>
 #include <qpushbutton.h>
 #include <qpopupmenu.h>
@@ -13,13 +34,12 @@
 #include <qmessagebox.h>
 #include <qsize.h>
 #include <qlistview.h>
+#include <qstringlist.h>
+#include <qspinbox.h>
+
+#include <licq_message.h>
 
 #include "mlview.h"
-#include "licq_message.h"
-
-#include <list>
-
-using std::list;
 
 class CUserEvent;
 class ICQEvent;
@@ -56,7 +76,7 @@ protected:
   bool m_bTransparent;
   QPopupMenu *mnuPopUp;
   QPixmap addPix;
-  list<QPixmap> m_lPixmaps;
+  std::list<QPixmap> m_lPixmaps;
   int addIndent,
       startingIndent;
 signals:
@@ -108,13 +128,19 @@ public:
 
   void setPreviousTab();
   void setNextTab();
-  
+
+signals:
+  void middleClick(int t);
+
 protected:
   virtual void paintLabel(QPainter *, const QRect &, QTab *, bool) const;
   virtual void wheelEvent(QWheelEvent *e);
-  
+  virtual void mousePressEvent(QMouseEvent* e);
+  virtual void mouseReleaseEvent(QMouseEvent* e);
+
 private:
   QMap<int, QColor> mTabColors;
+  int clickedTab;
 };
 
 class CETabWidget : public QTabWidget
@@ -127,8 +153,14 @@ public:
   void setPreviousPage();
   void setNextPage();
 
+signals:
+  void middleClick(QWidget *p);
+
 protected:
   virtual void wheelEvent(QWheelEvent *e);
+
+protected slots:
+  void slot_middleClick(int t);
 };
 
 /* ----------------------------------------------------------------------------- */
@@ -148,64 +180,81 @@ protected:
 
 class CInfoField : public QLineEdit
 {
+  Q_OBJECT
 public:
   CInfoField(QWidget *parent, bool readonly);
   void setData(const char *data);
   void setData(QString data);
-  void setData(const unsigned long data);
+  void setData(unsigned long data);
+  void setDateTime(uint timestamp);
   void SetReadOnly(bool);
 protected:
   virtual void keyPressEvent(QKeyEvent *e);
   QColor baseRO, baseRW;
 };
 
-
 /* ----------------------------------------------------------------------------- */
 
-class CHistoryWidget : public MLView
+/**
+ * Input field for timezones.
+ */
+class CTimeZoneField : public QSpinBox
 {
   Q_OBJECT
 public:
-  CHistoryWidget(QWidget* parent = 0, const char* name = 0);
-  virtual ~CHistoryWidget() {};
-      
+  CTimeZoneField(QWidget *parent);
+  void setData(char data);
+  char data();
 protected:
-#if QT_VERSION < 300
-  virtual void paintCell(QPainter* p, int row, int col);
-#endif
+  QString mapValueToText(int v);
+  int mapTextToValue(bool* ok);
+
+  static const int undefinedValue = -24;
 };
 
-class CMessageViewWidget : public CHistoryWidget
+/* ----------------------------------------------------------------------------- */
+
+class CMessageViewWidget : public MLView
 {
   Q_OBJECT
 private:
   char *m_szId;
   unsigned long m_nPPID;
-  unsigned long m_nUin;
   CMainWindow *mainwin;
 public:
+  static QStringList getStyleNames(bool includeHistoryStyles = false);
+
   CMessageViewWidget(const char *szId, unsigned long nPPID,
-    CMainWindow *m, QWidget *parent = 0, const char *name = 0);
-  CMessageViewWidget(unsigned long _nUin, CMainWindow *m,
-		     QWidget* parent=0, const char * name =0);
+    CMainWindow *m, QWidget *parent = 0, const char *name = 0, bool historyMode = false);
   virtual ~CMessageViewWidget();
 
   void setOwner(const char *szId);
+  void updateContent();
+  void clear();
   void addMsg(direction dir, bool fromHistory, QString eventDescription, QDateTime date, 
     bool isDirect, bool isMultiRec, bool isUrgent, bool isEncrypted, 
     QString contactName, QString messageText);
-  void addNotice(QString dateTime, QString messageText);
+  void addNotice(QDateTime dateTime, QString messageText);
 
   unsigned short m_nMsgStyle;
-  bool m_bAppendLineBreak;
+  QString m_nDateFormat;
+  bool m_extraSpacing;
+  bool m_appendLineBreak;
+  bool m_useBuffer;
+  bool m_showNotices;
   QColor m_colorRcvHistory;
   QColor m_colorSntHistory;
   QColor m_colorRcv;
   QColor m_colorSnt;
+  QColor m_colorNotice;
   
 public slots:
-  virtual void addMsg(CUserEvent *, const char * = 0, unsigned long = 0);
+  virtual void addMsg(const CUserEvent* e, const char* _szId = NULL, unsigned long _nPPID = 0);
   void addMsg(ICQEvent *);
+
+private:
+  void internalAddMsg(QString s);
+  QString m_buffer;
 };
 
 /* ----------------------------------------------------------------------------- */
