@@ -1,13 +1,41 @@
+/*
+ * This file is part of Licq, an instant messaging client for UNIX.
+ * Copyright (C) 1999-2006 Licq developers
+ *
+ * Licq is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * Licq is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with Licq; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
 #ifndef GUIDEFINES_H
 #define GUIDEFINES_H
 
+#ifdef USE_KDE
+# define PLUGIN_NAME "kde-gui"
+#else
+# define PLUGIN_NAME "qt-gui"
+#endif
+
 #define QTGUI_DIR "qt-gui/"
 #define EMOTICONS_DIR "emoticons/"
+#define QTGUI_CONFIGFILE "licq_qt-gui.conf"
 
 #define NUM_MSG_PER_HISTORY 40
 #define COLOR_SENT "blue"
 #define COLOR_RECEIVED "red"
 
+// Defined here until there is a way to get constant from daemon
+#define MSN_PPID 0x4D534E5F
 
 enum UserMenu {
   mnuUserView,
