@@ -1,20 +1,22 @@
 // -*- c-basic-offset: 2; -*-
 /*
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-
-*/
+ * This file is part of Licq, an instant messaging client for UNIX.
+ * Copyright (C) 1999-2006 Licq developers
+ *
+ * Licq is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * Licq is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with Licq; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -22,28 +24,7 @@
 
 #ifdef USE_KDE
 #include <kapp.h>
-#if KDE_VERSION < 290
-#include <kthemestyle.h>
 #endif
-#endif
-
-// In Qt 3.x and above, we should use QStyleFactory, since the styles
-// can come as plugins.
-#if QT_VERSION < 300
-#include <qwindowsstyle.h>
-#include <qmotifstyle.h>
-#include <qplatinumstyle.h>
-#include <qcdestyle.h>
-#if QT_VERSION >= 230
-#if HAVE_QINTERLACESTYLE_H
-#include <qinterlacestyle.h>
-#endif
-#endif
-#if QT_VERSION >= 220
-#include <qsgistyle.h>
-#include <qmotifplusstyle.h>
-#endif
-#endif // end of QT_VERSION < 300
 
 #include <qsessionmanager.h>
 #include <qaccel.h>
@@ -54,9 +35,7 @@
 #endif
 #include <unistd.h> // for getopt
 
-#if QT_VERSION < 300
-#include "jfcstyle.h"
-#endif
+#include "gui-defines.h"
 #include "mainwin.h"
 #include "sigman.h"
 #include "outputwin.h"
@@ -65,45 +44,39 @@
 
 #include "licq_icqd.h"
 
-#if QT_VERSION >= 300
 #include <qstylefactory.h>
-#endif
-
-CLicqGui *licqQtGui;
 
 const char *LP_Usage(void)
 {
-#ifdef USE_KDE
-  static const char usage[] =
-    "Usage:  Licq [options] -p kde-gui -- [-h] [-s skinname] [-i iconpack] [-e extendediconpack] [-g gui style]\n"
-    " -h : this help screen\n"
-    " -s : set the skin to use (must be in {base dir}/qt-gui/skin.skinname)\n"
-    " -i : set the icons to use (must be in {base dir}/qt-gui/icons.iconpack)\n"
-    " -e : set the extended icons to use (must be in [base dir]/qt-gui/extended.icons.iconpack)\n"
-    " -g : set the gui style (MOTIF / WINDOWS / MAC / CDE /"
-#if QT_VERSION < 300
-    " JFC /"
+  static const QString usage = QString(
+      "Usage:  Licq [options] -p %1 -- [-h] [-s skinname] [-i iconpack] [-e extendediconpack]"
+#ifndef USE_KDE
+      " [-g gui style]"
 #endif
-    " GTK / SGI / LCD), ignored by KDE support\n"
-    " -d : start hidden (dock icon only)\n"
-    " -D : disable dock icon for this session (does not affect dock icon settings)";
-#else
-  static const char usage[] =
-    "Usage:  Licq [options] -p qt-gui -- [-h] [-s skinname] [-i iconpack] [-e extendediconpack] [-g gui style]\n"
-    " -h : this help screen\n"
-    " -s : set the skin to use (must be in {base dir}/qt-gui/skin.skinname)\n"
-    " -i : set the icons to use (must be in {base dir}/qt-gui/icons.iconpack)\n"
-    " -e : set the extended icons to use (must be in [base dir]/qt-gui/extended.icons.iconpack)\n"
-    " -g : set the gui style (MOTIF / WINDOWS / MAC / CDE /"
-#if QT_VERSION < 300
-    " JFC /"
+      "\n"
+      " -h : this help screen\n"
+      " -s : set the skin to use (must be in %2%3skin.skinname)\n"
+      " -i : set the icons to use (must be in %4%5icons.iconpack)\n"
+      " -e : set the extended icons to use (must be in %6%7extended.icons.iconpack)\n"
+#ifndef USE_KDE
+      " -g : set the gui style (%8 or 'default' to follow global Qt settings)\n"
 #endif
-    " GTK / SGI / LCD), ignored by KDE support\n"
-    " -d : start hidden (dock icon only)\n"
-    " -D : disable dock icon for this session (does not affect dock icon settings)";
+      " -d : start hidden (dock icon only)\n"
+      " -D : disable dock icon for this session (does not affect dock icon settings)"
+      )
+      .arg(PLUGIN_NAME)
+      .arg(BASE_DIR)
+      .arg(QTGUI_DIR)
+      .arg(BASE_DIR)
+      .arg(QTGUI_DIR)
+      .arg(BASE_DIR)
+      .arg(QTGUI_DIR)
+#ifndef USE_KDE
+      .arg(QStyleFactory::keys().join(", "))
 #endif
+      ;
 
-  return usage;
+  return usage.latin1();
 }
 
 const char *LP_Name(void)
@@ -119,7 +92,7 @@ const char *LP_Name(void)
 
 const char *LP_ConfigFile(void)
 {
-  return "licq_qt-gui.conf";
+  return QTGUI_CONFIGFILE;
 }
 
 
@@ -169,10 +142,14 @@ bool LP_Init(int argc, char **argv)
 
 int LP_Main(CICQDaemon *_licqDaemon)
 {
-  licqQtGui = new CLicqGui(gui_argc, gui_argv);
+#ifdef USE_KDE
+  // Don't use the KDE crash handler (drkonqi).
+  setenv("KDE_DEBUG", "true", 0);
+#endif
 
-  int nResult = licqQtGui->Run(_licqDaemon);
-  licqQtGui->Shutdown();
+  CLicqGui licqQtGui(gui_argc, gui_argv);
+
+  int nResult = licqQtGui.Run(_licqDaemon);
 
   gui_argc = 0;
   gui_argv = NULL;
@@ -184,32 +161,6 @@ QStyle *CLicqGui::SetStyle(const char *_szStyle)
 {
   QStyle *s = NULL;
 
-#if QT_VERSION < 300
-  if (strncmp(_szStyle, "MOTIF", 3) == 0)
-    s = new QMotifStyle;
-  else if (strncmp(_szStyle, "WINDOWS", 3) == 0)
-    s = new QWindowsStyle;
-  else if (strncmp(_szStyle, "MAC", 3) == 0)
-    s = new QPlatinumStyle;
-  else if (strncmp(_szStyle, "CDE", 3) == 0)
-    s = new QCDEStyle;
-#if QT_VERSION >= 220
-  else if (strncmp( _szStyle, "SGI", 3 ) == 0 )
-    s = new QSGIStyle;
-#endif
-#if (QT_VERSION >= 230) && defined(QINTERLACESTYLE_H)
-  else if (strncmp( _szStyle, "LCD", 3 ) == 0 )
-    s = new QInterlaceStyle;
-#endif
-#if QT_VERSION < 300
-  else if (strncmp(_szStyle, "JFC", 3) == 0)
-    s = new JFCStyle;
-#endif
-#if QT_VERSION >= 220
-  else if (strncmp(_szStyle, "GTK", 3) == 0)
-    s = new QMotifPlusStyle(true);
-#endif
-#else  // QT_VERSION >= 300
   if (strncmp(_szStyle, "MOTIF", 3) == 0)
     s = QStyleFactory::create("motif");
   else if (strncmp(_szStyle, "WINDOWS", 3) == 0)
@@ -220,7 +171,6 @@ QStyle *CLicqGui::SetStyle(const char *_szStyle)
     s = QStyleFactory::create("cde");
   else if (strncmp( _szStyle, "SGI", 3 ) == 0 )
     s = QStyleFactory::create("sgi");
-#endif
 
   return s;
 }
@@ -272,7 +222,9 @@ CLicqGui::CLicqGui(int argc, char **argv)
   char skinName[32] = "";
   char iconsName[32] = "";
   char extendedIconsName[32] = "";
+#ifndef USE_KDE
   char styleName[32] = "";
+#endif
   bool bStartHidden = false;
 
   // initialize Members
@@ -282,11 +234,7 @@ CLicqGui::CLicqGui(int argc, char **argv)
   // store command line arguments for session management
   cmdLineParams += QString(argv[0]);
   cmdLineParams += QString("-p");
-#ifdef USE_KDE
-  cmdLineParams += QString("kde-gui");
-#else
-  cmdLineParams += QString("qt-gui");
-#endif
+  cmdLineParams += QString(PLUGIN_NAME);
   cmdLineParams += QString("--");
   for(int i=1;i<argc;i++)
     cmdLineParams += QString(argv[i]);
@@ -312,10 +260,12 @@ CLicqGui::CLicqGui(int argc, char **argv)
       snprintf(extendedIconsName, sizeof(extendedIconsName), "%s", optarg);
       extendedIconsName[sizeof(extendedIconsName) - 1] = '\0';
       break;
+#ifndef USE_KDE
     case 'g': // gui style
       strncpy(styleName, optarg, sizeof(styleName));
       styleName[sizeof(styleName) - 1] = '\0';
       break;
+#endif
     case 'd': // dock icon
       if (!m_bDisableDockIcon)
         bStartHidden = true;
@@ -327,9 +277,9 @@ CLicqGui::CLicqGui(int argc, char **argv)
   }
 
 #ifndef USE_KDE
-  char buf[500];
-  snprintf(buf, sizeof(buf), "%s/licq_qt-gui.style", BASE_DIR);
-  buf[sizeof(buf) - 1] = '\0';
+  char buf[MAX_FILENAME_LEN];
+  snprintf(buf, MAX_FILENAME_LEN, "%s/licq_qt-gui.style", BASE_DIR);
+  buf[MAX_FILENAME_LEN - 1] = '\0';
 
   QStyle *style = SetStyle(styleName);
 
@@ -366,35 +316,23 @@ CLicqGui::CLicqGui(int argc, char **argv)
   // Try and load a translation
   gLog.Info("%sAttempting to load %s Qt-GUI translation.\n", L_INITxSTR,
             GetLocale());
-  QString str;
-  str.sprintf("%sqt-gui/locale/%s", SHARE_DIR, GetLocale());
   QTranslator *trans = new QTranslator(this);
-  trans->load(str);
+  trans->load(QString("%1" QTGUI_DIR "locale/%2").arg(SHARE_DIR).arg(GetLocale()));
   installTranslator(trans);
 }
 
 
 CLicqGui::~CLicqGui(void)
 {
+  licqMainWindow->hide();
+  delete licqMainWindow;
   delete licqSignalManager;
   delete licqLogWindow;
+  free(m_szSkin);
+  free(m_szIcons);
+  free(m_szExtendedIcons);
 }
 
-void CLicqGui::Shutdown(void)
-{
-  gLog.Info("%sShutting down gui.\n", L_ENDxSTR);
-  gLog.ModifyService(S_PLUGIN, 0);
-  if(licqMainWindow) {
-    licqMainWindow->close();
-    delete licqMainWindow;
-    licqMainWindow = 0;
-  }
-
-  delete qApp;
-  qApp = 0;
-}
-
-
 int CLicqGui::Run(CICQDaemon *_licqDaemon)
 {
   // Register with the daemon, we want to receive all signals
@@ -410,6 +348,8 @@ int CLicqGui::Run(CICQDaemon *_licqDaemon)
   setMainWidget(licqMainWindow);
   int r = exec();
   _licqDaemon->UnregisterPlugin();
+  gLog.Info("%sShutting down gui.\n", L_ENDxSTR);
+  gLog.ModifyService(S_PLUGIN, 0);
 
   return r;
 }
