@@ -95,9 +95,9 @@ QStringList LicqKIMIface::allContacts()
 
 QStringList LicqKIMIface::reachableContacts()
 {
-    ICQOwner* owner = gUserManager.FetchOwner(LOCK_R);
+    ICQOwner* owner = gUserManager.FetchOwner(LICQ_PPID, LOCK_R);
     bool offline = owner->StatusOffline();
-    gUserManager.DropOwner();
+    gUserManager.DropOwner(owner);
 
     if (offline)
         return QStringList();
@@ -296,7 +296,7 @@ QString LicqKIMIface::locate(const QString& contactId, const QString& protocol)
     QString kabcID;
 
     FOR_EACH_PROTO_USER_START(PPID, LOCK_R)
-    QString uin = pUser->UinString();
+    QString uin = pUser->IdString();
     if (!uin.isEmpty() && uin == contactId)
     {
         QString id = pUser->IdString();
