@@ -1,3 +1,22 @@
+/*
+ * This file is part of Licq, an instant messaging client for UNIX.
+ * Copyright (C) 1999-2006 Licq developers
+ *
+ * Licq is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * Licq is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with Licq; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
 #ifndef MAINWIN_H
 #define MAINWIN_H
 
@@ -5,30 +24,27 @@
 #include "config.h"
 #endif
 
+#include <vector>
+
 #ifdef USE_KDE
 #include <kmenubar.h>
 #else
 #include <qmenubar.h>
 #endif
 
-#if QT_VERSION < 300
-  #include <qlist.h>
-#else
-  #include <qptrlist.h>
-#endif
+#include <qptrlist.h>
 #include <qwidget.h>
 #include <qtimer.h>
 #include <qbitmap.h>
 #include <qdialog.h>
-#include <vector>
+#include <qvaluelist.h>
+
+#include <licq_user.h>
 
-#include "userbox.h"
-#include "licq_user.h"
-#include "licqdialog.h"
 #include "gui-defines.h"
+#include "licqdialog.h"
 #include "support.h"
-
-#define MSN_PPID 0x4D534E5F
+#include "userbox.h"
 
 class QStyle;
 
@@ -55,25 +71,15 @@ class IconManager;
 
 class CICQSignal;
 class UserInfoDlg;
-#if QT_VERSION >= 300
 class UserEventTabDlg;
-#endif
 
 #ifdef USE_KDE
 class LicqKIMIface;
 #endif
 
-#if QT_VERSION < 300
-  typedef QList<UserViewEvent> UserViewEventList;
-  typedef QList<UserInfoDlg> UserInfoList;
-  typedef QList<UserSendCommon> UserSendEventList;
-#else
-  typedef QPtrList<UserViewEvent> UserViewEventList;
-  typedef QPtrList<UserInfoDlg> UserInfoList;
-  typedef QPtrList<UserSendCommon> UserSendEventList;
-#endif
-
-using std::vector;
+typedef QPtrList<UserViewEvent> UserViewEventList;
+typedef QPtrList<UserInfoDlg> UserInfoList;
+typedef QPtrList<UserSendCommon> UserSendEventList;
 
 //=====CMainWindow==============================================================
 class CMainWindow : public QWidget
@@ -88,8 +94,6 @@ public:
               const char *extendedIconsName, bool bDisableDockIcon,
               QWidget *parent = 0);
   virtual ~CMainWindow();
-  bool RemoveUserFromList(unsigned long, QWidget *);
-  bool RemoveUserFromGroup(GroupType gtype, unsigned long group, unsigned long, QWidget *);
   UserEventCommon *callFunction(int fcn, const char *, unsigned long, int = -1);
   bool RemoveUserFromList(const char *, unsigned long, QWidget *);
   bool RemoveUserFromGroup(GroupType, unsigned long, const char *,
@@ -100,13 +104,11 @@ public:
   void ApplyExtendedIcons(const char *, bool = false);
   CUserView *UserView()  { return userView; }
   QPopupMenu *UserMenu() { return mnuUser; }
-  void SetUserMenuUin(unsigned long n) { m_nUserMenuUin = n; } // deprecated, use SetUserMenuUser instead!
   void SetUserMenuUser(const char *_szId, unsigned long _nPPID)
   {
     if (m_szUserMenuId)  free(m_szUserMenuId);
     m_szUserMenuId = strdup(_szId);
     m_nUserMenuPPID = _nPPID;
-    m_nUserMenuUin = strtoul(_szId, (char **)NULL, 10); // deprecated, use m_nUserMenuId instead!
   }
   static QPixmap &iconForStatus(unsigned long FullStatus, const char *szId = "0",
     unsigned long nPPID = LICQ_PPID);
@@ -137,6 +139,7 @@ public:
        m_bAutoPosReplyWin,
        m_bAutoSendThroughServer,
        m_bEnableMainwinMouseMovement,
+       m_bPopPicture,
        m_bPopAlias,
        m_bPopName,
        m_bPopEmail,
@@ -147,13 +150,17 @@ public:
        m_bPopLastOnline,
        m_bPopOnlineSince,
        m_bPopIdleTime,
+       m_bPopLocalTime,
        m_bPopID,
        m_bShowAllEncodings,
        m_bTabbedChatting,
        m_bShowHistory,
+       m_showNotices,
        m_bDisableDockIcon,
        m_bSortColumnAscending,
-       m_bAppendLineBreak,
+       m_chatVertSpacing,
+       m_chatAppendLineBreak,
+       m_histVertSpacing,
        m_bFlashTaskbar,
        m_bMainWinSticky,
        m_bMsgWinSticky,
@@ -171,12 +178,17 @@ public:
   unsigned long m_nCurrentGroup, m_nGroupStates;
   unsigned short m_nSortByStatus,
                  m_nSortColumn,
-                 m_nMsgStyle;
-                 
+                 m_chatMsgStyle,
+                 m_histMsgStyle;
+
+  QString m_chatDateFormat,
+          m_histDateFormat;
+
   QColor m_colorRcvHistory,
          m_colorSntHistory,
          m_colorRcv,
          m_colorSnt,
+         m_colorNotice,
          m_colorTab,
          m_colorTabTyping,
          m_colorChatBkg;
@@ -202,9 +214,7 @@ public:
   OptionsDlg *optionsDlg;
   OwnerManagerDlg *ownerManagerDlg;
   PluginDlg *pluginDlg;
-#if QT_VERSION >= 300
   UserEventTabDlg *userEventTabDlg;
-#endif
 
   // Widgets
   CUserView *userView;
@@ -251,13 +261,12 @@ public:
           pmPhoneFollowMeBusy, pmSharedFiles, pmMSNOnline, pmMSNOffline,
           pmMSNOccupied, pmMSNPrivate, pmMSNAway, pmAIMOnline, pmAIMAway,
           pmAIMOffline, pmGPGKey, pmGPGKeyEnabled, pmGPGKeyDisabled;
-  unsigned long m_nUserMenuUin;
-  unsigned int positionChanges;
+  bool positionChanges;
   int m_nProtoNum;
   char *m_szUserMenuId;
   unsigned long m_nUserMenuPPID;
-  vector<unsigned long> m_lnProtMenu;
-  
+  std::vector<unsigned long> m_lnProtMenu;
+
   // AutoAway
   QTimer autoAwayTimer;
   unsigned short autoAwayTime,
@@ -277,8 +286,6 @@ public:
   
   // Functions
   void CreateUserView();
-  void CreateUserFloaty(unsigned long nUin, unsigned short x = 0,
-     unsigned short y = 0, unsigned short w = 0);
   void CreateUserFloaty(const char *szId, unsigned long nPPID,
     unsigned short x = 0, unsigned short y = 0, unsigned short w = 0);
   void initMenu();
@@ -328,8 +335,6 @@ protected slots:
   void callDefaultFunction(QListViewItem *);
   void callOwnerFunction(int, unsigned long = LICQ_PPID);
   void callMsgFunction();
-  void callFileFunction (const char *);
-  void callUrlFunction (const char *);
   void callUserFunction(int);
   void slot_socket(const char *, unsigned long, unsigned long);
   void slot_convoJoin(const char *, unsigned long, unsigned long);
@@ -434,6 +439,7 @@ private:
         MNU_SYS_GPG = 12
   };
 
+  QValueList<unsigned short> myGroupIds;
 
 };
 
