@@ -1,3 +1,22 @@
+/*
+ * This file is part of Licq, an instant messaging client for UNIX.
+ * Copyright (C) 2000-2006 Licq developers
+ *
+ * Licq is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * Licq is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with Licq; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
 #ifndef MMSENDDLG_H
 #define MMSENDDLG_H
 
@@ -9,7 +28,7 @@
 #include <qstring.h>
 
 #include "licqdialog.h"
-#include "licq_user.h"
+#include "licq_icqd.h"
 
 class QPushButton;
 class QVGroupBox;
@@ -31,22 +50,18 @@ public:
 
   int go_message(QString);
   int go_url(QString, QString);
-  //TODO String List please
-  int go_contact(UinList &_uins);
+  int go_contact(StringList& users);
+
 protected:
   QString s1, s2;
-  //TODO String list please
-  UinList *uins;
+  StringList* myUsers;
 
   unsigned long m_nEventType;
   QVGroupBox *grpSending;
   QPushButton *btnCancel;
   QProgressBar *barSend;
-  unsigned long m_nUin;
-#ifdef QT_PROTOCOL_PLUGIN
   char *m_szId;
   unsigned long m_nPPID;
-#endif
   CMMUserView *mmv;
   CMMUserViewItem *mmvi;
   CICQDaemon *server;
