@@ -1,20 +1,22 @@
 // -*- c-basic-offset: 2 -*-
 /*
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-
-*/
+ * This file is part of Licq, an instant messaging client for UNIX.
+ * Copyright (C) 1999-2006 Licq developers
+ *
+ * Licq is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * Licq is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with Licq; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -45,6 +47,7 @@
 #include <qfontdialog.h>
 #endif
 
+#include "ewidgets.h"
 #include "optionsdlg.h"
 #include "licq_log.h"
 #include "licq_constants.h"
@@ -67,7 +70,7 @@ OptionsDlg::OptionsDlg(CMainWindow *_mainwin, tabs settab, QWidget *parent)
 
   QBoxLayout* top_lay = new QVBoxLayout(this, 4);
 
-  tabw = new QTabWidget(this);
+  tabw = new CETabWidget(this);
   top_lay->addWidget(tabw);
 
   QBoxLayout* lay = new QHBoxLayout(top_lay);
@@ -209,31 +212,34 @@ void OptionsDlg::SetupOptions()
   chkSysBack->setChecked(mainwin->m_bSystemBackground);
   chkSendFromClipboard->setChecked(mainwin->m_bSendFromClipboard);
   chkMsgChatView->setChecked(mainwin->m_bMsgChatView);
-  
-  chkLineBreak->setChecked(mainwin->m_bAppendLineBreak);
-  cmbStyle->setCurrentItem(mainwin->m_nMsgStyle);
+  cmbChatDateFormat->lineEdit()->setText(mainwin->m_chatDateFormat);
+  chkChatVertSpacing->setChecked(mainwin->m_chatVertSpacing);
+  chkChatLineBreak->setChecked(mainwin->m_chatAppendLineBreak);
+  cmbChatStyle->setCurrentItem(mainwin->m_chatMsgStyle);
+  cmbHistStyle->setCurrentItem(mainwin->m_histMsgStyle);
+  chkHistVertSpacing->setChecked(mainwin->m_histVertSpacing);
+  cmbHistDateFormat->lineEdit()->setText(mainwin->m_histDateFormat);
   btnColorRcv->setPaletteBackgroundColor(mainwin->m_colorRcv);
   btnColorSnt->setPaletteBackgroundColor(mainwin->m_colorSnt);
   btnColorRcvHistory->setPaletteBackgroundColor(mainwin->m_colorRcvHistory);
   btnColorSntHistory->setPaletteBackgroundColor(mainwin->m_colorSntHistory);
+  btnColorNotice->setPaletteBackgroundColor(mainwin->m_colorNotice);
   /*btnColorTabLabel->setPaletteBackgroundColor(mainwin->m_colorTab);*/
   btnColorTypingLabel->setPaletteBackgroundColor(mainwin->m_colorTabTyping);
   btnColorChatBkg->setPaletteBackgroundColor(mainwin->m_colorChatBkg);
   slot_refresh_msgViewer();
-  
+
   if (mainwin->m_bMsgChatView)
   {
-#if QT_VERSION >= 300
     chkTabbedChatting->setChecked(mainwin->m_bTabbedChatting);
-#endif
     chkShowHistory->setChecked(mainwin->m_bShowHistory);
+    chkShowNotices->setChecked(mainwin->m_showNotices);
   }
   else
   {
-#if QT_VERSION >= 300
     chkTabbedChatting->setEnabled(false);
-#endif
     chkShowHistory->setEnabled(false);
+    chkShowNotices->setChecked(false);
   }
   chkSendTN->setChecked(mainwin->licqDaemon->SendTypingNotification());
   chkAutoPosReplyWin->setChecked(mainwin->m_bAutoPosReplyWin);
@@ -243,6 +249,8 @@ void OptionsDlg::SetupOptions()
   chkMainWinSticky->setChecked(mainwin->m_bMainWinSticky);
   chkMsgWinSticky->setChecked(mainwin->m_bMsgWinSticky);
   chkSingleLineChatMode->setChecked(mainwin->m_bSingleLineChatMode);
+  chkUseDoubleReturn->setChecked(MLEditWrap::useDoubleReturn);
+  popPicture->setChecked(mainwin->m_bPopPicture);
   popAlias->setChecked(mainwin->m_bPopAlias);
   popName->setChecked(mainwin->m_bPopName);
   popEmail->setChecked(mainwin->m_bPopEmail);
@@ -253,6 +261,7 @@ void OptionsDlg::SetupOptions()
   popLastOnline->setChecked(mainwin->m_bPopLastOnline);
   popOnlineSince->setChecked(mainwin->m_bPopOnlineSince);
   popIdleTime->setChecked(mainwin->m_bPopIdleTime);
+  popLocalTime->setChecked(mainwin->m_bPopLocalTime);
   popID->setChecked(mainwin->m_bPopID);
 
   switch(mainwin->m_nDockMode)
@@ -315,10 +324,9 @@ void OptionsDlg::SetupOptions()
 #endif
   }
   if (mainwin->m_bDisableDockIcon)
-    boxDocking->setEnabled(FALSE);
+    boxDocking->setEnabled(false);
 
   edtICQServer->setText(QString(mainwin->licqDaemon->ICQServer()));
-  spnICQServerPort->setValue(mainwin->licqDaemon->ICQServerPort());
   chkFirewall->setChecked(mainwin->licqDaemon->Firewall());
   chkTCPEnabled->setChecked(mainwin->licqDaemon->TCPEnabled());
   spnPortLow->setValue(mainwin->licqDaemon->TCPPortsLow());
@@ -346,7 +354,11 @@ void OptionsDlg::SetupOptions()
   chkProxyAuthEnabled->setChecked(mainwin->licqDaemon->ProxyAuthEnabled());
   edtProxyLogin->setText(QString(mainwin->licqDaemon->ProxyLogin()));
   edtProxyPasswd->setText(QString(mainwin->licqDaemon->ProxyPasswd()));
-  
+
+  // Set server port after chkProxyEnabled as it will trigger slot_useProxy
+  // which overwrites server port field.
+  spnICQServerPort->setValue(mainwin->licqDaemon->ICQServerPort());
+
   chkReconnectAfterUinClash->setChecked(mainwin->licqDaemon->ReconnectAfterUinClash());
 
   if (!mainwin->licqDaemon->ProxyEnabled())
@@ -382,13 +394,8 @@ void OptionsDlg::SetupOptions()
                                                 ->AutoUpdateStatusPlugins());
 
   // plugins tab
-#if QT_VERSION > 300
   cmbUrlViewer->setCurrentText(mainwin->licqDaemon->getUrlViewer() == NULL ?
              DEFAULT_URL_VIEWER : QString(mainwin->licqDaemon->getUrlViewer()));
-#else
-  cmbUrlViewer->changeItem(mainwin->licqDaemon->getUrlViewer() == NULL ?
-             DEFAULT_URL_VIEWER : QString(mainwin->licqDaemon->getUrlViewer()), cmbUrlViewer->currentItem());
-#endif
   edtTerminal->setText(mainwin->licqDaemon->Terminal() == NULL ?
                        tr("none") : QString(mainwin->licqDaemon->Terminal()));
 
@@ -443,24 +450,25 @@ void OptionsDlg::SetupOptions()
    COnEventManager *oem = mainwin->licqDaemon->OnEventManager();
    chkOnEvents->setChecked(oem->CommandType() != ON_EVENT_IGNORE);
    oem->Lock();
-   edtSndPlayer->setURL(oem->Command());
-   edtSndMsg->setURL(oem->Parameter(ON_EVENT_MSG));
-   edtSndUrl->setURL(oem->Parameter(ON_EVENT_URL));
-   edtSndChat->setURL(oem->Parameter(ON_EVENT_CHAT));
-   edtSndFile->setURL(oem->Parameter(ON_EVENT_FILE));
-   edtSndNotify->setURL(oem->Parameter(ON_EVENT_NOTIFY));
-   edtSndSysMsg->setURL(oem->Parameter(ON_EVENT_SYSMSG));
-   edtSndMsgSent->setURL(oem->Parameter(ON_EVENT_MSGSENT));
+   edtSndPlayer->setURL(oem->command());
+   edtSndMsg->setURL(oem->parameter(ON_EVENT_MSG));
+   edtSndUrl->setURL(oem->parameter(ON_EVENT_URL));
+   edtSndChat->setURL(oem->parameter(ON_EVENT_CHAT));
+   edtSndFile->setURL(oem->parameter(ON_EVENT_FILE));
+   edtSndNotify->setURL(oem->parameter(ON_EVENT_NOTIFY));
+   edtSndSysMsg->setURL(oem->parameter(ON_EVENT_SYSMSG));
+   edtSndMsgSent->setURL(oem->parameter(ON_EVENT_MSGSENT));
    oem->Unlock();
+
    //TODO make general for all plugins
-   ICQOwner *o = gUserManager.FetchOwner(LOCK_R);
+   ICQOwner* o = gUserManager.FetchOwner(LICQ_PPID, LOCK_R);
    if (o)
    {
      chkOEAway->setChecked(o->AcceptInAway());
      chkOENA->setChecked(o->AcceptInNA());
      chkOEOccupied->setChecked(o->AcceptInOccupied());
      chkOEDND->setChecked(o->AcceptInDND());
-     gUserManager.DropOwner();
+     gUserManager.DropOwner(o);
    }
    chkAlwaysOnlineNotify->setChecked(mainwin->licqDaemon->AlwaysOnlineNotify());
 }
@@ -480,21 +488,14 @@ void OptionsDlg::ApplyOptions()
 {
   QFont f(mainwin->defaultFont);
   if(edtEditFont->text().find(tr("default"), 0, false) != 0)
-#if QT_VERSION >= 300
     f.fromString(edtEditFont->text());
-#else
-    f.setRawName(edtEditFont->text());
-#endif
+
   delete MLEditWrap::editFont;
   MLEditWrap::editFont = new QFont(f);
 
   f = mainwin->defaultFont;
   if(edtFont->text().find(tr("default"), 0, false) != 0)
-#if QT_VERSION >= 300
     f.fromString(edtFont->text());
-#else
-    f.setRawName(edtFont->text());
-#endif
   qApp->setFont(f, true);
 
   mainwin->m_bGridLines = chkGridLines->isChecked();
@@ -521,20 +522,25 @@ void OptionsDlg::ApplyOptions()
   mainwin->m_bSystemBackground = chkSysBack->isChecked();
   mainwin->m_bSendFromClipboard = chkSendFromClipboard->isChecked();
   mainwin->m_bMsgChatView = chkMsgChatView->isChecked();
-  mainwin->m_bAppendLineBreak = chkLineBreak->isChecked();
-  mainwin->m_nMsgStyle = cmbStyle->currentItem();
+  mainwin->m_chatVertSpacing = chkChatVertSpacing->isChecked();
+  mainwin->m_chatAppendLineBreak = chkChatLineBreak->isChecked();
+  mainwin->m_chatMsgStyle = cmbChatStyle->currentItem();
+  mainwin->m_chatDateFormat = cmbChatDateFormat->currentText();
+  mainwin->m_histMsgStyle = cmbHistStyle->currentItem();
+  mainwin->m_histVertSpacing = chkHistVertSpacing->isChecked();
+  mainwin->m_histDateFormat = cmbHistDateFormat->currentText();
   mainwin->m_colorRcv = btnColorRcv->paletteBackgroundColor();
   mainwin->m_colorSnt = btnColorSnt->paletteBackgroundColor();
   mainwin->m_colorRcvHistory = btnColorRcvHistory->paletteBackgroundColor();
   mainwin->m_colorSntHistory = btnColorSntHistory->paletteBackgroundColor();
+  mainwin->m_colorNotice = btnColorNotice->paletteBackgroundColor();
   /*mainwin->m_colorTab = btnColorTabLabel->paletteBackgroundColor();*/
   mainwin->m_colorTabTyping = btnColorTypingLabel->paletteBackgroundColor();
   mainwin->m_colorChatBkg = btnColorChatBkg->paletteBackgroundColor();
 
-#if QT_VERSION >= 300
   mainwin->m_bTabbedChatting = chkTabbedChatting->isChecked();
-#endif
   mainwin->m_bShowHistory = chkShowHistory->isChecked();
+  mainwin->m_showNotices = chkShowNotices->isChecked();
   mainwin->m_bAutoPosReplyWin = chkAutoPosReplyWin->isChecked();
   mainwin->m_bFlashTaskbar = chkFlashTaskbar->isChecked();
   mainwin->m_bAutoSendThroughServer = chkAutoSendThroughServer->isChecked();
@@ -544,7 +550,9 @@ void OptionsDlg::ApplyOptions()
   mainwin->changeMainWinSticky(chkMainWinSticky->isChecked());
   mainwin->m_bMsgWinSticky = chkMsgWinSticky->isChecked();
   mainwin->m_bSingleLineChatMode = chkSingleLineChatMode->isChecked();
+  MLEditWrap::useDoubleReturn = chkUseDoubleReturn->isChecked();
 
+  mainwin->m_bPopPicture = popPicture->isChecked();
   mainwin->m_bPopAlias = popAlias->isChecked();
   mainwin->m_bPopName = popName->isChecked();
   mainwin->m_bPopEmail = popEmail->isChecked();
@@ -555,6 +563,7 @@ void OptionsDlg::ApplyOptions()
   mainwin->m_bPopLastOnline = popLastOnline->isChecked();
   mainwin->m_bPopOnlineSince = popOnlineSince->isChecked();
   mainwin->m_bPopIdleTime = popIdleTime->isChecked();
+  mainwin->m_bPopLocalTime = popLocalTime->isChecked();
   mainwin->m_bPopID = popID->isChecked();
 
 
@@ -700,23 +709,17 @@ void OptionsDlg::ApplyOptions()
   COnEventManager *oem = mainwin->licqDaemon->OnEventManager();
   oem->SetCommandType(chkOnEvents->isChecked() ? ON_EVENT_RUN : ON_EVENT_IGNORE);
 
-  QString txtSndPlayer = edtSndPlayer->url();
-  QString txtSndMsg = edtSndMsg->url();
-  QString txtSndUrl = edtSndUrl->url();
-  QString txtSndChat = edtSndChat->url();
-  QString txtSndFile = edtSndFile->url();
-  QString txtSndNotify = edtSndNotify->url();
-  QString txtSndSysMsg = edtSndSysMsg->url();
-  QString txtSndMsgSent = edtSndMsgSent->url();
+  oem->setCommand(edtSndPlayer->url());
+  oem->setParameter(ON_EVENT_MSG, edtSndMsg->url());
+  oem->setParameter(ON_EVENT_URL, edtSndUrl->url());
+  oem->setParameter(ON_EVENT_CHAT, edtSndChat->url());
+  oem->setParameter(ON_EVENT_FILE, edtSndFile->url());
+  oem->setParameter(ON_EVENT_NOTIFY, edtSndNotify->url());
+  oem->setParameter(ON_EVENT_SYSMSG, edtSndSysMsg->url());
+  oem->setParameter(ON_EVENT_MSGSENT, edtSndMsgSent->url());
 
-  const char *oemparams[8] = { txtSndMsg.latin1(), txtSndUrl.latin1(),
-   txtSndChat.latin1(), txtSndFile.latin1(),
-   txtSndNotify.latin1(), txtSndSysMsg.latin1(),
-   txtSndMsgSent.latin1(), 0 };
-
-  oem->SetParameters(txtSndPlayer.latin1(), oemparams);
   //TODO Make general for all plugins
-  ICQOwner *o = gUserManager.FetchOwner(LOCK_W);
+  ICQOwner* o = gUserManager.FetchOwner(LICQ_PPID, LOCK_W);
   if (o)
   {
     o->SetEnableSave(false);
@@ -726,7 +729,7 @@ void OptionsDlg::ApplyOptions()
     o->SetAcceptInDND(chkOEDND->isChecked());
     o->SetEnableSave(true);
     o->SaveLicqInfo();
-    gUserManager.DropOwner();
+    gUserManager.DropOwner(o);
   }
   mainwin->licqDaemon->SetAlwaysOnlineNotify(chkAlwaysOnlineNotify->isChecked());
 }
@@ -735,17 +738,11 @@ void OptionsDlg::ApplyOptions()
 void OptionsDlg::setupFontName(QLineEdit* le, const QFont& font)
 {
   QString s;
-#if QT_VERSION >= 300
   if (font == mainwin->defaultFont)
     s = tr("default (%1)").arg(font.toString());
   else
     s = font.toString();
-#else
-  if (font == mainwin->defaultFont)
-    s = tr("default (%1)").arg(font.rawName());
-  else
-    s = font.rawName();
-#endif
+
   le->setFont(font);
   le->setText(s);
   le->setCursorPosition(0);
@@ -834,19 +831,25 @@ QWidget* OptionsDlg::new_appearance_options()
   QWhatsThis::add(chkSingleLineChatMode, tr("In single line chat mode you send messages with Enter "
     "and insert new lines with Ctrl+Enter, opposite of the normal mode"));
 
+  chkUseDoubleReturn = new QCheckBox(tr("Use double return"), boxMainWin);
+  QWhatsThis::add(chkUseDoubleReturn, tr("Hitting Return twice will be used instead of Ctrl+Return "
+    "to send messages and close input dialogs. Multiple new lines can be inserted with Ctrl+Return."));
+
   chkMsgChatView = new QCheckBox(tr("Chatmode Messageview"), boxMainWin);
   QWhatsThis::add(chkMsgChatView, tr("Show the current chat history in Send Window"));
 
-#if QT_VERSION >= 300
   chkTabbedChatting = new QCheckBox(tr("Tabbed Chatting"), boxMainWin);
   QWhatsThis::add(chkTabbedChatting, tr("Use tabs in Send Window"));
   //connect(chkMsgChatView, SIGNAL(toggled(bool)), this, SLOT(slot_useMsgChatView(bool)));
-#endif
 
   chkShowHistory = new QCheckBox(tr("Show recent messages"), boxMainWin);
   QWhatsThis::add(chkShowHistory, tr("Show the last 5 messages when a Send Window is opened"));
   connect(chkMsgChatView, SIGNAL(toggled(bool)), this, SLOT(slot_useMsgChatView(bool)));
 
+  chkShowNotices = new QCheckBox(tr("Show Join/Left Notices"), boxMainWin);
+  QWhatsThis::add(chkShowNotices, tr("Show a notice in the chat window when a user joins or leaves the conversation."));
+  connect(chkShowNotices, SIGNAL(toggled(bool)), this, SLOT(slot_refresh_msgViewer()));
+
   chkSendTN = new QCheckBox(tr("Send typing notifications"), boxMainWin);
   QWhatsThis::add(chkSendTN, tr("Send a notification to the user so they can see when you are typing a message to them"));
 
@@ -874,10 +877,6 @@ QWidget* OptionsDlg::new_appearance_options()
   chkShowAllEncodings = new QCheckBox(tr("Show all encodings"), boxLocale);
   QWhatsThis::add(chkShowAllEncodings, tr("Show all available encodings in the User Encoding selection menu. Normally, this menu shows only commonly used encodings."));
 
-#if QT_VERSION < 210
-  QWidget* dummy_w = new QWidget(boxLocale);
-  if (dummy_w);
-#endif
   l->addWidget(boxLocale);
 
   boxDocking = new QGroupBox(1, Horizontal, tr("Docking"), w);
@@ -929,6 +928,8 @@ QWidget* OptionsDlg::new_appearance_options()
   connect(btnEditFont, SIGNAL(clicked()), this, SLOT(slot_selecteditfont()));
   l->addWidget(boxFont);
 
+  l->addStretch(1);
+
   return w;
 }
 
@@ -1026,7 +1027,7 @@ QWidget* OptionsDlg::new_sounds_options()
   edtSndMsgSent = new KURLRequester(boxSndEvents);
 
   QGroupBox *boxAcceptEvents = new QGroupBox(4, Vertical, tr("Accept Modes"), w);
-  lay->addWidget(boxAcceptEvents, 1);
+  lay->addWidget(boxAcceptEvents);
 
   chkOEAway = new QCheckBox(tr("OnEvent in Away"), boxAcceptEvents);
   QWhatsThis::add(chkOEAway, tr("Perform OnEvent command in away mode"));
@@ -1042,6 +1043,8 @@ QWidget* OptionsDlg::new_sounds_options()
 
   slot_chkOnEventsToggled(chkOnEvents->isChecked());
 
+  lay->addStretch(1);
+
   return w;
 }
 
@@ -1169,10 +1172,12 @@ void OptionsDlg::slot_useMsgChatView(bool b)
   {
     chkTabbedChatting->setChecked(false);
     chkShowHistory->setChecked(false);
+    chkShowNotices->setEnabled(false);
   }
 
   chkTabbedChatting->setEnabled(b);
   chkShowHistory->setEnabled(b);
+  chkShowNotices->setEnabled(b);
 }
 
 void OptionsDlg::slot_useProxy(bool b)
@@ -1357,7 +1362,7 @@ QWidget* OptionsDlg::new_column_options()
   lblColFormat = new QLabel (tr("Format"), grp);
   QWhatsThis::add(lblColFormat, QStyleSheet::convertFromPlainText(tr("The format string used to define what will "
                                    "appear in each column.\n"
-                                   "The following parameters can be used:\n")) + gMainWindow->usprintfHelp);
+                                   "The following parameters can be used:")) + gMainWindow->usprintfHelp);
   lblColWidth = new QLabel (tr("Width"), grp);
   QWhatsThis::add(lblColWidth, tr("The width of the column"));
   lblColAlign = new QLabel(tr("Alignment"), grp);
@@ -1460,6 +1465,7 @@ QWidget* OptionsDlg::new_column_options()
 
   boxPopWin = new QGroupBox(1, Horizontal, tr("Popup info"), w);
 
+  popPicture= new QCheckBox(tr("Picture"), boxPopWin);
   popAlias  = new QCheckBox(tr("Alias"), boxPopWin);
   popName   = new QCheckBox(tr("Name"),  boxPopWin);
   popEmail  = new QCheckBox(tr("Email"), boxPopWin);
@@ -1470,6 +1476,7 @@ QWidget* OptionsDlg::new_column_options()
   popLastOnline = new QCheckBox(tr("Last online"), boxPopWin);
   popOnlineSince = new QCheckBox(tr("Online Time"), boxPopWin);
   popIdleTime = new QCheckBox(tr("Idle Time"), boxPopWin);
+  popLocalTime = new QCheckBox(tr("Local time"), boxPopWin);
   popID = new QCheckBox(tr("Protocol ID"), boxPopWin);
 
 
@@ -1531,11 +1538,6 @@ QWidget* OptionsDlg::new_misc_options()
   chkIgnoreEmailPager = new QCheckBox(tr("Ignore Email Pager"), boxParanoia);
   QWhatsThis::add(chkIgnoreEmailPager, tr("Determines if email pager messages are ignored or not."));
 
-#if QT_VERSION < 210
-  QWidget* dummy_w= new QWidget(boxParanoia);
-  dummy_w->setMinimumHeight(10);
-#endif
-
   QWidget* boxAutoStatus = new QGroupBox(2, Horizontal, tr("Auto Away Messages"), w);
   lay->addWidget(boxAutoStatus);
   new QLabel(tr("Away:"), boxAutoStatus);
@@ -1570,30 +1572,100 @@ QWidget* OptionsDlg::new_misc_options()
 QWidget* OptionsDlg::new_chat_options()
 {
   QWidget* w = new QWidget(this);
-  QBoxLayout* lay = new QHBoxLayout(w, 8, 4);
+  QVBoxLayout* l = new QVBoxLayout(w, 8, 4);
+  QBoxLayout* lay = new QHBoxLayout();
 
   QVBox* boxRight = new QVBox(w);
   lay->addWidget(boxRight);
-  
-  QGroupBox* boxOptions = new QGroupBox(1, Horizontal, tr("Options"), boxRight);
 
-  new QLabel(tr("Style:"), boxOptions);
-  cmbStyle = new QComboBox(false, boxOptions);
-  cmbStyle->insertItem("Default");
-  cmbStyle->insertItem("Compact");
-  cmbStyle->insertItem("Tiny");
-  cmbStyle->insertItem("Table");
-  cmbStyle->insertItem("History");
-  connect(cmbStyle, SIGNAL(activated(int)), this, SLOT(slot_refresh_msgViewer()));
-  
-  chkLineBreak = new QCheckBox(tr("Insert Horizontal Line"), boxOptions);
-  connect(chkLineBreak, SIGNAL(toggled(bool)), this, SLOT(slot_refresh_msgViewer()));
-  
+  QGroupBox* boxChatOptions = new QGroupBox(1, Horizontal, tr("Chat Options"), boxRight);
+
+  static const int dateFormatsLength = 7;
+  static const char *const dateFormats[dateFormatsLength] = {
+      "hh:mm:ss",
+      "yyyy-MM-dd hh:mm:ss",
+      "yyyy-MM-dd",
+      "yyyy/MM/dd hh:mm:ss",
+      "yyyy/MM/dd",
+      "dd.MM.yyyy hh:mm:ss",
+      "dd.MM.yyyy"
+  };
+
+  new QLabel(tr("Style:"), boxChatOptions);
+  cmbChatStyle = new QComboBox(false, boxChatOptions);
+  cmbChatStyle->insertStringList(CMessageViewWidget::getStyleNames(false));
+  connect(cmbChatStyle, SIGNAL(activated(int)), this, SLOT(slot_refresh_msgViewer()));
+
+  chkChatVertSpacing = new QCheckBox(tr("Insert Vertical Spacing"), boxChatOptions);
+  connect(chkChatVertSpacing, SIGNAL(toggled(bool)), this, SLOT(slot_refresh_msgViewer()));
+  QWhatsThis::add(chkChatVertSpacing, tr("Insert extra space between messages."));
+
+  chkChatLineBreak = new QCheckBox(tr("Insert Horizontal Line"), boxChatOptions);
+  connect(chkChatLineBreak, SIGNAL(toggled(bool)), this, SLOT(slot_refresh_msgViewer()));
+  QWhatsThis::add(chkChatLineBreak, tr("Insert a line between each message."));
+
+  QLabel* lblChatDateFormat = new QLabel(tr("Date Format:"), boxChatOptions);
+  cmbChatDateFormat = new QComboBox(true, boxChatOptions);
+  for(int i = 0; i < dateFormatsLength; ++i)
+    cmbChatDateFormat->insertItem(dateFormats[i]);
+  connect(cmbChatDateFormat, SIGNAL(activated(int)), this, SLOT(slot_refresh_msgViewer()));
+  connect(cmbChatDateFormat, SIGNAL(textChanged(const QString&)), this, SLOT(slot_refresh_msgViewer()));
+
+  QString helpDateFormat = tr(
+      "<p>Available custom date format variables.</p>\n"
+      "<table>\n"
+      "<tr><th>Expression</th><th>Output</th></tr>\n"
+      "<tr><td>d</td><td>the day as number without a leading zero (1-31)</td></tr>\n"
+      "<tr><td>dd</td><td>the day as number with a leading zero (01-31)</td></tr>\n"
+      "<tr><td>ddd</td><td>the abbreviated localized day name (e.g. 'Mon'..'Sun')</td></tr>\n"
+      "<tr><td>dddd</td><td>the long localized day name (e.g. 'Monday'..'Sunday')</td></tr>\n"
+      "<tr><td>M</td><td>the month as number without a leading zero (1-12)</td></tr>\n"
+      "<tr><td>MM</td><td>the month as number with a leading zero (01-12)</td></tr>\n"
+      "<tr><td>MMM</td><td>the abbreviated localized month name (e.g. 'Jan'..'Dec')</td></tr>\n"
+      "<tr><td>MMMM</td><td>the long localized month name (e.g. 'January'..'December')</td></tr>\n"
+      "<tr><td>yy</td><td>the year as two digit number (00-99)</td></tr>\n"
+      "<tr><td>yyyy</td><td>the year as four digit number (1752-8000)</td></tr>\n"
+      "<tr><td colspan=2></td></tr>\n"
+      "<tr><td>h</td><td>the hour without a leading zero (0..23 or 1..12 if AM/PM display)</td></tr>\n"
+      "<tr><td>hh</td><td>the hour with a leading zero (00..23 or 01..12 if AM/PM display)</td></tr>\n"
+      "<tr><td>m</td><td>the minute without a leading zero (0..59)</td></tr>\n"
+      "<tr><td>mm</td><td>the minute with a leading zero (00..59)</td></tr>\n"
+      "<tr><td>s</td><td>the second without a leading zero (0..59)</td></tr>\n"
+      "<tr><td>ss</td><td>the second with a leading zero (00..59)</td></tr>\n"
+      "<tr><td>z</td><td>the millisecond without leading zero (0..999)</td></tr>\n"
+      "<tr><td>zzz</td><td>the millisecond with leading zero (000..999)</td></tr>\n"
+      "<tr><td>AP</td><td>use AM/PM display. AP will be replaced by either 'AM' or 'PM'</td></tr>\n"
+      "<tr><td>ap</td><td>use am/pm display. ap will be replaced by either 'am' or 'pm'</td></tr>\n"
+      "</table>");
+
+  QWhatsThis::add(lblChatDateFormat, helpDateFormat);
+  QWhatsThis::add(cmbChatDateFormat, helpDateFormat);
+
+  QGroupBox* boxHistOptions = new QGroupBox(1, Horizontal, tr("History Options"), boxRight);
+  new QLabel(tr("Style:"), boxHistOptions);
+  cmbHistStyle = new QComboBox(false, boxHistOptions);
+  cmbHistStyle->insertStringList(CMessageViewWidget::getStyleNames(true));
+  connect(cmbHistStyle, SIGNAL(activated(int)), this, SLOT(slot_refresh_msgViewer()));
+
+  chkHistVertSpacing = new QCheckBox(tr("Insert Vertical Spacing"), boxHistOptions);
+  connect(chkHistVertSpacing, SIGNAL(toggled(bool)), this, SLOT(slot_refresh_msgViewer()));
+  QWhatsThis::add(chkHistVertSpacing, tr("Insert extra space between messages."));
+
+  QLabel* lblHistDateFormat = new QLabel(tr("Date Format:"), boxHistOptions);
+  cmbHistDateFormat = new QComboBox(true, boxHistOptions);
+  for(int i = 0; i < dateFormatsLength; ++i)
+    cmbHistDateFormat->insertItem(dateFormats[i]);
+  connect(cmbHistDateFormat, SIGNAL(activated(int)), this, SLOT(slot_refresh_msgViewer()));
+  connect(cmbHistDateFormat, SIGNAL(textChanged(const QString&)), this, SLOT(slot_refresh_msgViewer()));
+  QWhatsThis::add(lblHistDateFormat, helpDateFormat);
+  QWhatsThis::add(cmbHistDateFormat, helpDateFormat);
+
+
   QGroupBox *boxColors = new QGroupBox(2, Horizontal, tr("Colors"), boxRight);
 
   new QLabel(tr("Message Received:"), boxColors);
   btnColorRcv = new CColorOption(boxColors);
-  
+
   new QLabel(tr("Message Sent:"), boxColors);
   btnColorSnt = new CColorOption(boxColors);
 
@@ -1603,6 +1675,9 @@ QWidget* OptionsDlg::new_chat_options()
   new QLabel(tr("History Sent:"), boxColors);
   btnColorSntHistory = new CColorOption(boxColors);
 
+  new QLabel(tr("Notice:"), boxColors);
+  btnColorNotice = new CColorOption(boxColors);
+
   /*new QLabel(tr("Tab Label Color:"), boxColors);
   btnColorTabLabel = new CColorOption(boxColors);*/
 
@@ -1616,53 +1691,92 @@ QWidget* OptionsDlg::new_chat_options()
   connect(btnColorRcv, SIGNAL(changed()), this, SLOT(slot_refresh_msgViewer()));
   connect(btnColorSntHistory, SIGNAL(changed()), this, SLOT(slot_refresh_msgViewer()));
   connect(btnColorRcvHistory, SIGNAL(changed()), this, SLOT(slot_refresh_msgViewer()));
+  connect(btnColorNotice, SIGNAL(changed()), this, SLOT(slot_refresh_msgViewer()));
   /*connect(btnColorTabLabel, SIGNAL(changed()), this, SLOT(slot_refresh_msgViewer()));*/
   connect(btnColorTypingLabel, SIGNAL(changed()), this, SLOT(slot_refresh_msgViewer()));
   connect(btnColorChatBkg, SIGNAL(changed()), this, SLOT(slot_refresh_msgViewer()));
-  
-  tabViewer = new QTabWidget(w);
+
+  tabViewer = new CETabWidget(w);
   lay->addWidget(tabViewer);
-  
-  msgViewer = new CMessageViewWidget(0, gMainWindow, tabViewer);
-  tabViewer->insertTab(msgViewer, "Marge");
-  
+
+  msgChatViewer = new CMessageViewWidget(NULL, 0, gMainWindow, tabViewer);
+  tabViewer->insertTab(msgChatViewer, "Marge");
+
+  msgHistViewer = new CMessageViewWidget(NULL, 0, gMainWindow, tabViewer, 0, true);
+  tabViewer->insertTab(msgHistViewer, tr("History"));
+
   lay->activate();
 
+  l->addLayout(lay);
+  l->addStretch(1);
+
   return w;
 }
 
 void OptionsDlg::slot_refresh_msgViewer()
 {
-  QDateTime date;
-  const char *names[2] = {"Marge", "Homer"};
-  const char *msgs[7] = {
-      "This is received message",
-      "This is a sent message",
-      "Have you gone to the Licq IRC Channel?",
-      "No, where is it?",
-      "#Licq on irc.freenode.net",
-      "Cool, I'll see you there :)",
-      "We'll be waiting!"};
-  
-  msgViewer->m_nMsgStyle = cmbStyle->currentItem();
-  msgViewer->m_bAppendLineBreak = chkLineBreak->isChecked();
-  msgViewer->m_colorSnt = btnColorSnt->paletteBackgroundColor();
-  msgViewer->m_colorRcv = btnColorRcv->paletteBackgroundColor();
-  msgViewer->m_colorSntHistory = btnColorSntHistory->paletteBackgroundColor();
-  msgViewer->m_colorRcvHistory = btnColorRcvHistory->paletteBackgroundColor();
-  tabViewer->setPaletteForegroundColor(btnColorTypingLabel->paletteBackgroundColor());
-  msgViewer->setPaletteBackgroundColor(btnColorChatBkg->paletteBackgroundColor());
-  
-  msgViewer->clear();
+  // Don't update the time at every refresh
+  static QDateTime date = QDateTime::currentDateTime();
+
+  const char* const names[2] = {"Marge", "Homer"};
+  const char* const msgs[8] = {
+      QT_TR_NOOP("This is a received message"),
+      QT_TR_NOOP("This is a sent message"),
+      QT_TR_NOOP("Have you gone to the Licq IRC Channel?"),
+      QT_TR_NOOP("No, where is it?"),
+      QT_TR_NOOP("#Licq on irc.freenode.net"),
+      QT_TR_NOOP("Cool, I'll see you there :)"),
+      QT_TR_NOOP("We'll be waiting!"),
+      QT_TR_NOOP("Marge has left the conversation.")
+  };
+
+  msgChatViewer->m_nMsgStyle = cmbChatStyle->currentItem();
+  msgChatViewer->m_extraSpacing = chkChatVertSpacing->isChecked();
+  msgChatViewer->m_appendLineBreak = chkChatLineBreak->isChecked();
+  msgChatViewer->m_showNotices = chkShowNotices->isChecked();
+  msgChatViewer->m_colorSnt = btnColorSnt->paletteBackgroundColor();
+  msgChatViewer->m_colorRcv = btnColorRcv->paletteBackgroundColor();
+  msgChatViewer->m_colorSntHistory = btnColorSntHistory->paletteBackgroundColor();
+  msgChatViewer->m_colorRcvHistory = btnColorRcvHistory->paletteBackgroundColor();
+  msgChatViewer->m_colorNotice = btnColorNotice->paletteBackgroundColor();
+  tabViewer->setTabColor(msgChatViewer, btnColorTypingLabel->paletteBackgroundColor());
+  msgChatViewer->setPaletteBackgroundColor(btnColorChatBkg->paletteBackgroundColor());
+
+  msgChatViewer->m_nDateFormat = cmbChatDateFormat->currentText();
+
+  msgHistViewer->m_nMsgStyle = cmbHistStyle->currentItem();
+  msgHistViewer->m_extraSpacing = chkHistVertSpacing->isChecked();
+  msgHistViewer->m_colorSnt = btnColorSnt->paletteBackgroundColor();
+  msgHistViewer->m_colorRcv = btnColorRcv->paletteBackgroundColor();
+  msgHistViewer->setPaletteBackgroundColor(btnColorChatBkg->paletteBackgroundColor());
+  msgHistViewer->m_nDateFormat = cmbHistDateFormat->currentText();
+
+  msgChatViewer->clear();
+  msgHistViewer->clear();
+
+  QDateTime msgDate = date;
   for (unsigned int i = 0; i<7; i++)
   {
-    msgViewer->addMsg(i%2 == 0 ? D_RECEIVER : D_SENDER, (i<2),
+    msgChatViewer->addMsg(i%2 == 0 ? D_RECEIVER : D_SENDER, (i<2),
           QString(""),
-          date,
+          msgDate,
           true, false, false, false, 
           names[i % 2],
-          MLView::toRichText(msgs[i], true, true));
+          MLView::toRichText(tr(msgs[i]), true, true));
+
+    msgHistViewer->addMsg(i%2 == 0 ? D_RECEIVER : D_SENDER, false,
+          QString(""),
+          msgDate,
+          true, false, false, false,
+          names[i % 2],
+          MLView::toRichText(tr(msgs[i]), true, true));
+
+    msgDate = msgDate.addSecs(i + 12);
   }
+  msgDate = msgDate.addSecs(12);
+  msgChatViewer->addNotice(msgDate, MLView::toRichText(tr(msgs[7]), true, true));
+
+  msgHistViewer->updateContent();
 }
 
 CColorOption::CColorOption (QWidget* parent)
