@@ -1,20 +1,22 @@
 // -*- c-basic-offset: 2 -*-
 /*
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or   
-    (at your option) any later version.                              
-                                       
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of 
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
-    GNU General Public License for more details.                 
-                                                
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software      
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  
-
-*/
+ * This file is part of Licq, an instant messaging client for UNIX.
+ * Copyright (C) 1999-2006 Licq developers
+ *
+ * Licq is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * Licq is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with Licq; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -48,9 +50,9 @@ CQtLogWindow::CQtLogWindow(QWidget *parent)
   QBoxLayout* top_lay = new QVBoxLayout(this, 8);
 
   outputBox = new CLogWidget(this);
-  outputBox->setMinimumHeight(outputBox->frameWidth()*2
-                              + 16*outputBox->fontMetrics().lineSpacing());
-  outputBox->setMinimumWidth(outputBox->minimumHeight()*2);
+  outputBox->setMinimumHeight(outputBox->frameWidth() * 2
+      + 16 * outputBox->fontMetrics().lineSpacing());
+  outputBox->setMinimumWidth(outputBox->minimumHeight() * 2);
   top_lay->addWidget(outputBox);
 
   QBoxLayout* lay = new QHBoxLayout(top_lay, 8);
@@ -75,6 +77,8 @@ CQtLogWindow::CQtLogWindow(QWidget *parent)
   lay->addWidget(btnClear);
   lay->addWidget(btnHide);
 
+  adjustSize();
+
   sn = new QSocketNotifier(Pipe(), QSocketNotifier::Read, this);
   connect (sn, SIGNAL(activated(int)), this, SLOT(slot_log(int)));
 }
@@ -94,7 +98,7 @@ void CQtLogWindow::slot_log(int s)
   char buf[4];
   read(s, buf, 1);
 
-  QString str = QString::fromLocal8Bit(NextLogMsg());
+  QString str = QString::fromUtf8(NextLogMsg());
 
   outputBox->appendNoNewLine(str);
   outputBox->GotoEnd();
@@ -158,36 +162,4 @@ CLogWidget::CLogWidget(QWidget* parent, const char* name)
 //  setTextFormat(LogText);
 }
 
-
-// -----------------------------------------------------------------------------
-
-void CLogWidget::paintCell(QPainter* p, int row, int col)
-{
-#if QT_VERSION < 300
-  QColorGroup& cg = const_cast<QColorGroup&>(colorGroup());
-  QColor cgback = cg.text();
-
-  QString s;
-  int i = row;
-  while ( i >= 0 && (s = stringShown(i).mid(11, 3)) == "   ") i--;
-
-  if (s == "WRN")
-    cg.setColor(QColorGroup::Text, Qt::darkYellow);
-  else if (s == "ERR")
-    cg.setColor(QColorGroup::Text, Qt::darkRed);
-  else if (s == "PKT")
-    cg.setColor(QColorGroup::Text, Qt::darkBlue);
-  else if (s == "???")
-    cg.setColor(QColorGroup::Text, Qt::magenta);
-
-  MLEditWrap::paintCell(p, row, col);
-
-  cg.setColor(QColorGroup::Text, cgback);
-#endif
-}
-
-
-
-// -----------------------------------------------------------------------------
-
 #include "outputwin.moc"
