@@ -1,20 +1,22 @@
 // -*- c-basic-offset: 2 -*-
 /*
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-
-*/
+ * This file is part of Licq, an instant messaging client for UNIX.
+ * Copyright (C) 1999-2006 Licq developers
+ *
+ * Licq is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * Licq is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with Licq; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
 
 // Search user code base written by Alan Penner (apenner@andrew.cmu.edu)
 // modified by Graham Roff
@@ -31,13 +33,13 @@
 #include <qpainter.h>
 #include <qpushbutton.h>
 #include <qlineedit.h>
-#include <qtabwidget.h>
 #include <qvbuttongroup.h>
 #include <qvalidator.h>
 #include <qcombobox.h>
 #include <qspinbox.h>
 #include <qtextcodec.h>
 
+#include "ewidgets.h"
 #include "searchuserdlg.h"
 #include "sigman.h"
 
@@ -64,27 +66,26 @@ SearchUserView::SearchUserView(QWidget *parent)
   setAllColumnsShowFocus (true);
   setMinimumHeight(150);
   setMinimumWidth(600);
-#if QT_VERSION >= 210
   setSelectionMode(QListView::Extended);
   setShowSortIndicator(true);
-#endif
 }
 
 
-SearchItem::SearchItem(CSearchAck *s, const QString &encoding, QListView *parent)
+SearchItem::SearchItem(const CSearchAck* s, const QString& encoding, QListView* parent)
   : QListViewItem(parent)
 {
   QString qsStatus;
   QString qsGender;
   QString qsAge;
   QString qsAuth;
-  
-  uinVal = s->Uin();
+
+  myId = s->Id();
+  myPpid = s->PPID();
   QTextCodec *codec = QTextCodec::codecForName(encoding);
   if (codec == 0)
     codec = QTextCodec::codecForLocale();
   setText(0, codec->toUnicode(s->Alias()));
-  setText(1, QString::number(s->Uin()));
+  setText(1, s->Id());
   setText(2, codec->toUnicode(s->FirstName()) + QString(" ") + codec->toUnicode(s->LastName()));
   setText(3, s->Email());
 
@@ -120,13 +121,6 @@ SearchItem::SearchItem(CSearchAck *s, const QString &encoding, QListView *parent
   setText(6, qsAuth);
 }
 
-
-unsigned long SearchItem::uin()
-{
-  return (uinVal);
-}
-
-
 // -----------------------------------------------------------------------------
 
 SearchUserDlg::SearchUserDlg(CMainWindow *m, CICQDaemon *s, CSignalManager *theSigMan, const QString &encoding)
@@ -143,7 +137,7 @@ SearchUserDlg::SearchUserDlg(CMainWindow *m, CICQDaemon *s, CSignalManager *theS
   QBoxLayout* lay = new QHBoxLayout(top_lay, 10);
 
   // pre-search widgets
-  search_tab = new QTabWidget(this);
+  search_tab = new CETabWidget(this);
 
   //-- first tab: search by Alias/name
   whitepages_tab = new QWidget(this);
@@ -452,13 +446,12 @@ void SearchUserDlg::searchResult(ICQEvent *e)
     searchFailed();
 }
 
-void SearchUserDlg::searchFound(CSearchAck *s)
+void SearchUserDlg::searchFound(const CSearchAck* s)
 {
   (void) new SearchItem(s, m_Encoding, foundView);
 }
 
-
-void SearchUserDlg::searchDone(CSearchAck *sa)
+void SearchUserDlg::searchDone(const CSearchAck* sa)
 {
   if (sa == NULL || sa->More() == 0)
     lblSearch->setText(tr("Search complete."));
@@ -513,16 +506,13 @@ void SearchUserDlg::viewInfo()
   {
     if (current->isSelected())
     {
-      //XXX Convert this to a string
-      char szId[64];
-      snprintf(szId, 64, "%lu", current->uin());
-      ICQUser *u = gUserManager.FetchUser(szId, LICQ_PPID, LOCK_R);
+      ICQUser* u = gUserManager.FetchUser(current->id().latin1(), current->ppid(), LOCK_R);
       if (!u)
-        server->AddUserToList(szId, LICQ_PPID, false, true);
+        server->AddUserToList(current->id().latin1(), current->ppid(), false, true);
       else
         gUserManager.DropUser(u);
-      
-      mainwin->callInfoTab(mnuUserGeneral, szId, LICQ_PPID, false, true);
+
+      mainwin->callInfoTab(mnuUserGeneral, current->id().latin1(), current->ppid(), false, true);
       break;
     }
     current = static_cast<SearchItem*>(current->nextSibling());
@@ -533,21 +523,27 @@ void SearchUserDlg::addUser()
 {
   SearchItem* current = static_cast<SearchItem*>(foundView->firstChild());
 
-  while(current) {
-    if(current->isSelected()) {
-      ICQUser* user = gUserManager.FetchUser(current->uin(), LOCK_N);
+  while (current)
+  {
+    if (current->isSelected())
+    {
+      ICQUser* user = gUserManager.FetchUser(current->id().latin1(), current->ppid(), LOCK_R);
 
-      if(user)
+      if (user)
+      {
+        bool tempUser = user->NotInList();
         gUserManager.DropUser(user);
-      else {
-        server->AddUserToList(current->uin());
-        if (qcbAlertUser->isChecked()) // alert the user they were added
-            server->icqAlertUser(current->uin());
+        if (tempUser)
+          gUserManager.RemoveUser(current->id().latin1(), current->ppid());
       }
-//      current->setSelected(false);
+
+      if (server->AddUserToList(current->id().latin1(), current->ppid()) &&
+          qcbAlertUser->isChecked()) // alert the user they were added
+        server->icqAlertUser(current->id().latin1(), current->ppid());
     }
     current = static_cast<SearchItem*>(current->nextSibling());
   }
+
   foundView->triggerUpdate();
   selectionChanged();
 }
