@@ -1,20 +1,22 @@
 // -*- c-basic-offset: 2 -*-
 /*
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-
-*/
+ * This file is part of Licq, an instant messaging client for UNIX.
+ * Copyright (C) 1999-2006 Licq developers
+ *
+ * Licq is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * Licq is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with Licq; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
 
 // written by Graham Roff <graham@licq.org>
 // enhanced by Dirk A. Mueller <dmuell@gmx.net>
@@ -31,6 +33,7 @@
 #include "showawaymsgdlg.h"
 #include "sigman.h"
 #include "licq_user.h"
+#include "gui-strings.h"
 #include "usercodec.h"
 #include <ctype.h>
 
@@ -67,7 +70,9 @@ ShowAwayMsgDlg::ShowAwayMsgDlg(CICQDaemon *_server, CSignalManager* _sigman,
   QTextCodec * codec = UserCodec::codecForICQUser(u);
 //  chkShowAgain->setChecked(u->ShowAwayMsg());
 
-  setCaption(QString(tr("%1 Response for %2")).arg(u->StatusStr()).arg(QString::fromUtf8(u->GetAlias())));
+  setCaption(QString(tr("%1 Response for %2"))
+             .arg(Strings::getStatus(u, false))
+             .arg(QString::fromUtf8(u->GetAlias())));
 
   btnOk = new QPushButton(tr("&Ok"), this);
   btnOk->setMinimumWidth(75);
@@ -96,18 +101,6 @@ ShowAwayMsgDlg::ShowAwayMsgDlg(CICQDaemon *_server, CSignalManager* _sigman,
   show();
 }
 
-ShowAwayMsgDlg::ShowAwayMsgDlg(CICQDaemon *_server, CSignalManager* _sigman, unsigned long _nUin, QWidget *parent)
-{
-  char szUin[13];
-  snprintf(szUin, 12, "%lu", _nUin);
-  szUin[12] = 0;
-  
-  ShowAwayMsgDlg(_server, _sigman, szUin, LICQ_PPID, parent);
-}
-
-
-// -----------------------------------------------------------------------------
-
 ShowAwayMsgDlg::~ShowAwayMsgDlg()
 {
   if (m_szId) free(m_szId);
