@@ -49,7 +49,6 @@
 
 #include "licq_icqd.h"
 #include "licq_user.h"
-#include "licq_socket.h"
 
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
@@ -133,13 +132,12 @@ CUserViewItem::CUserViewItem (ICQUser *_cUser, CUserViewItem* item)
   setGraphics(_cUser);
 }
 
-CUserViewItem::CUserViewItem(unsigned short Id, const char* name, QListView* lv)
+CUserViewItem::CUserViewItem(unsigned short Id, const char* name, unsigned short sortKey, QListView* lv)
   : QListViewItem(lv),
     m_nGroupId(Id),
     m_sGroupName(name)
 {
   m_szId = 0;
-  m_szAlias = 0;
   m_nPPID =0;
   m_pIcon = NULL;
   m_cBack = s_cGroupBack;
@@ -157,20 +155,15 @@ CUserViewItem::CUserViewItem(unsigned short Id, const char* name, QListView* lv)
   m_bGPGKey = false;
   m_bNotInList = true;
   m_pUserIcon = 0;
-  // Other users group is sorted at the end
-  if (m_nGroupId)
-  {
-    QString strTemp = QString("%1").arg((int)m_nGroupId);
-    while (strTemp.length() < 10)
-      strTemp = "0" + strTemp;
-    m_sSortKey = strTemp;
-  }
-  else
-    m_sSortKey = QString("9999999999");
+
+  QString strTemp = QString("%1").arg((int)sortKey);
+  while (strTemp.length() < 10)
+    strTemp = "0" + strTemp;
+  m_sSortKey = strTemp;
+
   m_sPrefix = "1";
   setPixmap(0, gMainWindow->pmCollapsed);
   setText(1, QString::fromLocal8Bit(name));
-
 }
 
 CUserViewItem::CUserViewItem(BarType barType, QListView *parent)
@@ -593,9 +586,9 @@ void CUserViewItem::paintCell( QPainter *p, const QColorGroup & cgdefault, int c
         qDrawShadeLine( p, 0, height() >> 1, width - 5, (height() >> 1) + 2, cg, true, 1, 0);
       }
     } // isGroupItem
-    else if (m_szAlias                                        // this row contains a user 
+    else if (!m_szAlias.isNull()                          // this row contains a user
              && text(column).ascii()                          // ...+ it is not empty 
-             && strcmp(m_szAlias, text(column).ascii()) == 0  // ...+ this col contains the users alias 
+             && m_szAlias == text(column).ascii()       // ...+ this col contains the users alias
              && gMainWindow->m_bShowExtendedIcons)            // ...+ extended icons are enabled
     {
       // pmPhone
@@ -1367,7 +1360,7 @@ void CUserView::viewportDropEvent(QDropEvent* e)
 
       if (szId)
       {
-        gUserManager.AddUserToGroup(szId, nPPID, it->GroupId());
+        gUserManager.SetUserInGroup(szId, nPPID, GROUPS_USER, it->GroupId(), true, true);
         gMainWindow->updateUserWin();
       }
 
@@ -1769,7 +1762,7 @@ void CUserView::maybeTip(const QPoint& c)
 
     if ((!u->StatusOffline()) && gMainWindow->m_bPopOnlineSince)
     {
-      time_t nLoggedIn = time(0) - u->OnlineSince();
+      time_t nLoggedIn = (time(0) > u->OnlineSince() ? time(0) - u->OnlineSince() : 0);
       unsigned long nWeek, nDay, nHour, nMinute;
       nWeek = nLoggedIn / 604800;
       nDay = (nLoggedIn % 604800) / 86400;
