@@ -1,20 +1,22 @@
 // -*- c-basic-offset: 2 -*-
 /*
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-
-*/
+ * This file is part of Licq, an instant messaging client for UNIX.
+ * Copyright (C) 2000-2006 Licq developers
+ *
+ * Licq is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * Licq is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with Licq; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
 
 #ifndef USEREVENTDLG_H
 #define USEREVENTDLG_H
@@ -27,15 +29,16 @@
 #include <string>
 #include <utility>
 
-using std::binary_function;
-using std::list;
-using std::pair;
-using std::string;
-
 #include "licq_color.h"
 #include "licq_filetransfer.h"
 #include "licq_message.h"
 
+#ifdef USE_KDE
+#include <kdeversion.h>
+#else
+#define KDE_IS_VERSION(a, b, c) 0
+#endif
+
 class QTabWidget;
 class QBoxLayout;
 class QGroupBox;
@@ -66,17 +69,6 @@ class CETabWidget;
 class CMessageViewWidget;
 class MLView;
 
-/* ----------------------------------------------------------------------------- */
-typedef pair<CUserEvent *, char *> MessageIter;
-
-struct OrderMessages : public binary_function<const MessageIter &, const MessageIter &, bool>
-{
-  bool operator()(const MessageIter& m1, const MessageIter& m2)
-  {
-    return (m1.first->Time() < m2.first->Time());
-  }
-};
-
 /* ----------------------------------------------------------------------------- */
 class UserEventTabDlg : public QWidget
 {
@@ -86,7 +78,6 @@ public:
   ~UserEventTabDlg();
 
   void addTab(UserEventCommon *tab, int index = -1);
-  void removeTab(QWidget *tab);
   void selectTab(QWidget *tab);
   void replaceTab(QWidget *oldTab, UserEventCommon *newTab);
   bool tabIsSelected(QWidget *tab);
@@ -96,11 +87,9 @@ public:
   void gotTyping(ICQUser *u, int);
   void changeMsgWinSticky(bool _bStick);
 
-#ifdef USE_KDE
-#if KDE_VERSION >= 320
+#if defined(USE_KDE) && KDE_IS_VERSION(3, 2, 0)
   virtual void setIcon(const QPixmap &icon);
-#endif   // KDE_VERSION
-#endif // USE_KDE
+#endif
   
 private:
   CETabWidget *tabw;
@@ -113,6 +102,7 @@ public slots:
   void slot_currentChanged(QWidget *tab);
   void moveLeft();
   void moveRight();
+  void removeTab(QWidget* tab);
 
 protected slots:
   void setMsgWinSticky();
@@ -134,10 +124,10 @@ public:
   const char *Id()  { return m_lUsers.front().c_str(); }
   unsigned long PPID()  { return m_nPPID; }
   unsigned long ConvoId() { return m_nConvoId; }
-  list<string>& ConvoUsers() { return m_lUsers; }
+  std::list<std::string>& ConvoUsers() { return m_lUsers; }
   void SetConvoId(unsigned long n)  { m_nConvoId = n; }
   
-  bool FindUserInConvo(char *);
+  bool FindUserInConvo(const char* id);
   void AddEventTag(unsigned long n)  { if (n) m_lnEventTag.push_back(n); }
   void gotTyping(unsigned short);
   void changeMsgWinSticky(bool _bStick);
@@ -154,7 +144,7 @@ protected:
   QTextCodec *codec;
   bool m_bOwner;
   char *m_szId;
-  list<string> m_lUsers;
+  std::list<std::string> m_lUsers;
   unsigned long m_nPPID;
   QBoxLayout* top_lay, *top_hlay;
   CICQDaemon *server;
@@ -177,7 +167,8 @@ protected:
   // which events we already processed in the ctor.
   int m_highestEventId;
 
-  virtual void UserUpdated(CICQSignal *, char * = 0, unsigned long = 0) = 0;
+  virtual void UserUpdated(CICQSignal* signal, const char* id = NULL,
+      unsigned long ppid = 0) = 0;
   void SetGeneralInfo(ICQUser *);
   void FlashTaskbar(bool _bFlash);
 
@@ -228,7 +219,7 @@ protected:
   void generateReply();
   void sendMsg(QString txt);
   void updateNextButton();
-  virtual void UserUpdated(CICQSignal *, char * = 0, unsigned long = 0);
+  virtual void UserUpdated(CICQSignal* sig, const char* szId = NULL, unsigned long nPPID = 0);
 
 protected slots:
   void slot_close();
@@ -262,10 +253,8 @@ public:
   void setText(const QString& txt);
   void convoJoin(const char *, unsigned long _nConvoId);
   void convoLeave(const char *, unsigned long _nConvoId);
-  
-#if QT_VERSION >= 300
+
   virtual void windowActivationChange(bool oldActive);
-#endif
   int clearDelay;
   
 signals:
@@ -288,7 +277,7 @@ protected:
   QTimer *tmrSendTyping;
 
   void RetrySend(ICQEvent *e, bool bOnline, unsigned short nLevel);
-  virtual void UserUpdated(CICQSignal *, char * = 0, unsigned long = 0);
+  virtual void UserUpdated(CICQSignal* sig, const char* szId = NULL, unsigned long nPPID = 0);
   virtual bool sendDone(ICQEvent *) = 0;
   bool checkSecure();
 
@@ -311,6 +300,7 @@ protected slots:
   void slot_ClearNewEvents();
   void slot_textChanged();
   void slot_textChanged_timeout();
+  void slot_sendServerToggled();
 
 public slots:
   void changeEventType(int);
@@ -330,8 +320,6 @@ class UserSendMsgEvent : public UserSendCommon
   Q_OBJECT
 public:
 
-  UserSendMsgEvent(CICQDaemon *s, CSignalManager *theSigMan, CMainWindow *m,
-                  unsigned long _nUin, QWidget* parent = 0);
   UserSendMsgEvent(CICQDaemon *s, CSignalManager *theSigMan, CMainWindow *m,
                   const char *_szId, unsigned long _nPPID, QWidget *parent = 0);
   virtual ~UserSendMsgEvent();
@@ -438,8 +426,6 @@ class UserSendContactEvent : public UserSendCommon
   Q_OBJECT
 public:
 
-  UserSendContactEvent(CICQDaemon *s, CSignalManager *theSigMan, CMainWindow *m,
-                       unsigned long _nUin, QWidget* parent = 0);
   UserSendContactEvent(CICQDaemon *s, CSignalManager *theSigMan, CMainWindow *m,
                   const char *_szId, unsigned long _nPPID, QWidget *parent = 0);
   virtual ~UserSendContactEvent();
