@@ -69,7 +69,6 @@
 #include "licq_providers.h"
 #include "licq_user.h"
 #include "licq_icqd.h"
-#include "licq_socket.h"
 #include "licq_log.h"
 #include "usercodec.h"
 
@@ -743,7 +742,7 @@ void UserInfoDlg::SetMoreInfo(ICQUser *u)
   if (u->GetICQHomepagePresent())
   {
     QString url;
-    url.sprintf("(http://%lu.homepage.icq.com/)", u->Uin());
+    url.sprintf("(http://%s.homepage.icq.com/)", u->IdString());
     lblICQHomepage->setText(tr("User has an ICQ Homepage ") + url);
   }
   else
@@ -1499,7 +1498,7 @@ void UserInfoDlg::SavePicture()
   o->SetPicture(m_sFilename.latin1());
   o->SetEnableSave(true);
   o->SavePictureInfo();
-  gUserManager.DropOwner();
+  gUserManager.DropOwner(o);
 
   server->icqUpdatePictureTimestamp();
 }
@@ -1537,6 +1536,10 @@ void UserInfoDlg::CreateLastCountersInfo()
   nfoOnlineSince = new CInfoField(p, true);
   lay->addWidget(nfoOnlineSince, CR, 1);
 
+  lay->addWidget(new QLabel(tr("Registration Date:"), p), ++CR, 0);
+  nfoRegDate = new CInfoField(p, true);
+  lay->addWidget(nfoRegDate, CR, 1);
+
   lay->setRowStretch(++CR, 5);
 }
 
@@ -1548,68 +1551,28 @@ void UserInfoDlg::SetLastCountersInfo(ICQUser *u)
   if (u == NULL)
   {
     u = gUserManager.FetchUser(m_szId, m_nPPID, LOCK_R);
-    if (u == NULL) return;
+    if (u == NULL)
+      return;
     bDropUser = true;
   }
 
-  QDateTime t;
-  QString ds;
-
   if (!u->StatusOffline())
     nfoLastOnline->setData(tr("Now"));
-  else if (u->LastOnline() == 0)
-    nfoLastOnline->setData(tr("Unknown"));
   else
-  {
-    t.setTime_t(u->LastOnline());
-    ds = t.toString();
-    ds.truncate(ds.length() - 8);
-    nfoLastOnline->setData(ds);
-  }
+    nfoLastOnline->setDateTime(u->LastOnline());
 
-  if (u->LastSentEvent() == 0)
-    nfoLastSent->setData(tr("Unknown"));
-  else
-  {
-    t.setTime_t(u->LastSentEvent());
-    ds = t.toString();
-    ds.truncate(ds.length() - 8);
-    nfoLastSent->setData(ds);
-  }
-
-  if (u->LastReceivedEvent() == 0)
-    nfoLastRecv->setData(tr("Unknown"));
-  else
-  {
-    t.setTime_t(u->LastReceivedEvent());
-    ds = t.toString();
-    ds.truncate(ds.length() - 8);
-    nfoLastRecv->setData(ds);
-  }
-
-  if (u->LastCheckedAutoResponse() == 0)
-    nfoLastCheckedAR->setData(tr("Unknown"));
-  else
-  {
-    t.setTime_t(u->LastCheckedAutoResponse());
-    ds = t.toString();
-    ds.truncate(ds.length() - 8);
-    nfoLastCheckedAR->setData(ds);
-  }
+  nfoLastSent->setDateTime(u->LastSentEvent());
+  nfoLastRecv->setDateTime(u->LastReceivedEvent());
+  nfoLastCheckedAR->setDateTime(u->LastCheckedAutoResponse());
+  nfoRegDate->setDateTime(u->RegisteredTime());
 
   if (u->StatusOffline())
     nfoOnlineSince->setData(tr("Offline"));
-  else if (u->OnlineSince() == 0)
-    nfoOnlineSince->setData(tr("Unknown"));
   else
-  {
-    t.setTime_t(u->OnlineSince());
-    ds = t.toString();
-    ds.truncate(ds.length() - 8);
-    nfoOnlineSince->setData(ds);
-  }
+    nfoOnlineSince->setDateTime(u->OnlineSince());
 
-  if (bDropUser) gUserManager.DropUser(u);
+  if (bDropUser)
+    gUserManager.DropUser(u);
 }
 
 //-----KDE AddressBook info--------------------------------------------------------
@@ -2035,7 +1998,7 @@ void UserInfoDlg::ShowHistory()
     // since we're using the contact's encoding, not ours.
     QTextCodec *ownerCodec = UserCodec::codecForICQUser(o);
     ownerName = ownerCodec->toUnicode(o->GetAlias());
-    gUserManager.DropOwner(m_nPPID);
+    gUserManager.DropOwner(o);
   }
 
   mlvHistory->clear();
@@ -2320,8 +2283,8 @@ void UserInfoDlg::slotRetrieve()
   if(o == NULL)  return;
   unsigned short status = o->Status();
   QTextCodec *codec = UserCodec::codecForICQUser(o);
-  gUserManager.DropOwner(m_nPPID);
-  
+  gUserManager.DropOwner(o);
+
   if (m_bOwner && currentTab == PhoneInfo)
   {
     QListViewItem *selected = lsvPhoneBook->currentItem();
@@ -2394,11 +2357,7 @@ void UserInfoDlg::slotRetrieve()
     }
     case PictureInfo:
     {
-      ICQUser *u = gUserManager.FetchUser(m_szId, m_nPPID, LOCK_R);
-      if (u == NULL) return;
-      bool bSendServer = (u->SocketDesc(ICQ_CHNxINFO) < 0);
-      gUserManager.DropUser(u);
-      icqEventTag = server->icqRequestPicture(m_szId, bSendServer);
+      icqEventTag = server->ProtoRequestPicture(m_szId, m_nPPID);
       break;
     }
   }
@@ -2426,7 +2385,7 @@ void UserInfoDlg::slotUpdate()
     if(o == NULL)  return;
     unsigned short status = o->Status();
     codec = UserCodec::codecForICQUser(o);
-    gUserManager.DropOwner(m_nPPID);
+    gUserManager.DropOwner(o);
 
     if(status == ICQ_STATUS_OFFLINE) {
       InformUser(this, tr("You need to be connected to the\n"
