@@ -1,19 +1,22 @@
 // -*- c-basic-offset: 2 -*-
 /*
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-*/
+ * This file is part of Licq, an instant messaging client for UNIX.
+ * Copyright (C) 1999-2006 Licq developers
+ *
+ * Licq is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * Licq is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with Licq; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
 
 // written by Graham Roff <graham@licq.org>
 // KDE support by Dirk Mueller <dirk@licq.org>
@@ -49,6 +52,7 @@
 #include "mainwin.h"
 #include "licq_user.h"
 #include "ewidgets.h"
+#include "gui-strings.h"
 
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
@@ -67,7 +71,7 @@
 /*
   Constructs a WharfIcon widget.
 */
-IconManager::IconManager(CMainWindow *_mainwin, QPopupMenu *_menu, QWidget *parent)
+IconManager::IconManager(CMainWindow *_mainwin, QPopupMenu *_menu, QWidget* /* parent */)
   : QWidget(NULL, "LicqWharf", WType_TopLevel)
 {
   setCaption("LicqWharf");
@@ -89,8 +93,8 @@ void IconManager::X11Init()
   XWMHints *hints;  // hints
   XClassHint classhint;  // class hints
   XSizeHints sizeHints;
-  classhint.res_name = "licq";  // res_name
-  classhint.res_class = "Wharf";  // res_class
+  classhint.res_name = const_cast<char*>("licq");  // res_name
+  classhint.res_class = const_cast<char*>("Wharf");  // res_class
   XSetClassHint(dsp, win, &classhint); // set the class hints
   sizeHints.flags= USPosition;
   sizeHints.x = 0;
@@ -214,7 +218,7 @@ IconManager_Default::~IconManager_Default()
 void IconManager_Default::SetDockIconStatus()
 {
   QPixmap m;
-  ICQOwner *o = gUserManager.FetchOwner(LOCK_R);
+  ICQOwner* o = gUserManager.FetchOwner(LICQ_PPID, LOCK_R);
   if (o == 0) return;
   switch (o->Status())
   {
@@ -227,7 +231,7 @@ void IconManager_Default::SetDockIconStatus()
     case ICQ_STATUS_OFFLINE: m = QPixmap((const char **)iconOffline_xpm); break;
   }
   if (o->StatusInvisible()) m = QPixmap((const char **)iconInvisible_xpm);
-  gUserManager.DropOwner();
+  gUserManager.DropOwner(o);
   QPainter painter(wharfIcon->vis);
   if (!m_bFortyEight)
   {
@@ -259,11 +263,11 @@ void IconManager_Default::SetDockIconStatus()
 //-----IconManager_Default::GetDockIconStatusIcon----------------------------
 QPixmap *IconManager_Default::GetDockIconStatusIcon()
 {
-  ICQOwner *o = gUserManager.FetchOwner(LOCK_R);
+  ICQOwner* o = gUserManager.FetchOwner(LICQ_PPID, LOCK_R);
   if (o == 0) return NULL;
-  
+
   unsigned long s = o->Status();
-  gUserManager.DropOwner();
+  gUserManager.DropOwner(o);
   switch (s)
   {
     case ICQ_STATUS_ONLINE: return &mainwin->pmOnline;
@@ -333,7 +337,9 @@ void IconManager_Default::SetDockIconMsg(unsigned short nNewMsg, unsigned short
   {
     // Draw the little icon now
     QPixmap *m = NULL;
-    if (nSysMsg > 0 || nNewMsg > 0)
+    if (nSysMsg > 0)
+      m = &mainwin->pmReqAuthorize;
+    else if (nNewMsg > 0)
       m = &mainwin->pmMessage;
     else
       m = GetDockIconStatusIcon();
@@ -541,7 +547,7 @@ IconManager_Themed::~IconManager_Themed()
 void IconManager_Themed::SetDockIconStatus()
 {
   QPixmap *p = NULL;
-  ICQOwner *o = gUserManager.FetchOwner(LOCK_R);
+  ICQOwner* o = gUserManager.FetchOwner(LICQ_PPID, LOCK_R);
   if (o == 0) return;
   switch (o->Status())
   {
@@ -554,7 +560,7 @@ void IconManager_Themed::SetDockIconStatus()
     case ICQ_STATUS_OFFLINE: p = pixOffline; break;
   }
   if (o->StatusInvisible()) p = pixInvisible;
-  gUserManager.DropOwner();
+  gUserManager.DropOwner(o);
   if (p != NULL)
   {
     QPainter painter(wharfIcon->vis);
@@ -660,12 +666,12 @@ IconManager_KDEStyle::~IconManager_KDEStyle()
 void IconManager_KDEStyle::SetDockIconStatus()
 {
   //TODO make general for all plugins
-  ICQOwner *o = gUserManager.FetchOwner(LOCK_R);
+  ICQOwner* o = gUserManager.FetchOwner(LICQ_PPID, LOCK_R);
   if (o == 0) return;
   m_statusIcon = CMainWindow::iconForStatus(o->StatusFull());
   m_ownerStatus = o->Status();
   m_bStatusInvisible = o->StatusInvisible();
-  gUserManager.DropOwner();
+  gUserManager.DropOwner(o);
 
   updateTooltip();
 
@@ -678,7 +684,12 @@ void IconManager_KDEStyle::SetDockIconMsg(unsigned short nNewMsg, unsigned short
   m_nSysMsg = nSysMsg;
 
   killTimers();
-  if(m_nNewMsg + m_nSysMsg)
+  if (m_nSysMsg > 0)
+  {
+    m_eventIcon = mainwin->pmReqAuthorize;
+    startTimer(500);
+  }
+  else if (m_nNewMsg > 0)
   {
     m_eventIcon = mainwin->pmMessage;
     startTimer(500);
@@ -691,13 +702,13 @@ void IconManager_KDEStyle::SetDockIconMsg(unsigned short nNewMsg, unsigned short
   repaint();
 }
 
-void IconManager_KDEStyle::timerEvent(QTimerEvent* e)
+void IconManager_KDEStyle::timerEvent(QTimerEvent* /* e */)
 {
   m_timerToggle = !m_timerToggle;
   repaint();
 }
 
-void IconManager_KDEStyle::paintEvent( QPaintEvent *e)
+void IconManager_KDEStyle::paintEvent( QPaintEvent* /* e */)
 {
   QPainter p(this);
 
@@ -747,8 +758,8 @@ void IconManager_KDEStyle::mousePressEvent( QMouseEvent *e )
 void IconManager_KDEStyle::updateTooltip()
 {
   QToolTip::remove(this);
-  QString s = QString("<nobr>") + QString(ICQUser::StatusToStatusStr(m_ownerStatus, m_bStatusInvisible))
-      + QString("</nobr>");
+  QString s = QString("<nobr>%1</nobr>")
+      .arg(Strings::getStatus(m_ownerStatus, m_bStatusInvisible));
 
   if(m_nSysMsg)
     s += tr("<br><b>%1 system messages</b>").arg(m_nSysMsg);
