@@ -2,25 +2,30 @@
 #include "config.h"
 #endif
 
+#include "rms.h"
+
+#include <cctype>
+#include <climits>
+#include <cstdio>
+#include <cstring>
 #include <sys/select.h>
 #include <sys/time.h>
 #include <sys/types.h>
 #include <unistd.h>
-#include <string.h>
-#include <ctype.h>
-#include <stdio.h>
 #ifdef HAVE_ERRNO_H
-#include <errno.h>
+#include <cerrno>
 #else
 extern int errno;
 #endif
 
-#include "rms.h"
-#include "licq_log.h"
-#include "licq_icqd.h"
-#include "licq_file.h"
-#include "licq_user.h"
-#include "licq_constants.h"
+#include <licq_constants.h>
+#include <licq_icqd.h>
+#include <licq_file.h>
+#include <licq_log.h>
+#include <licq_socket.h>
+#include <licq_user.h>
+
+using namespace std;
 
 extern "C" { const char *LP_Version(); }
 
@@ -96,9 +101,9 @@ const unsigned short STATE_ENTERxSMSxNUMBER = 9;
 
 struct Command
 {
-  char *name;
+  const char *name;
   int (CRMSClient::*fcn)();
-  char *help;
+  const char *help;
 };
 
 static struct Command commands[] =
@@ -196,7 +201,7 @@ int CLicqRMS::Run(CICQDaemon *_licqDaemon)
     conf.CloseFile();
   }
 
-  server = new TCPSocket(0);
+  server = new TCPSocket();
 
   if (licqDaemon->TCPPortsLow() != 0 && nPort == 0)
   {
@@ -459,7 +464,7 @@ char CRMSClient::buf[128];
 /*---------------------------------------------------------------------------
  * CRMSClient::constructor
  *-------------------------------------------------------------------------*/
-CRMSClient::CRMSClient(TCPSocket *sin) : sock(0)
+CRMSClient::CRMSClient(TCPSocket* sin)
 {
   sin->RecvConnection(sock);
   sockman.AddSocket(&sock);
@@ -584,7 +589,7 @@ bool CRMSClient::ProcessEvent(ICQEvent *e)
   tags.erase(iter);
 
   unsigned short nCode = 0;
-  char *szr = NULL;
+  const char *szr = NULL;
   switch(e->Result())
   {
     case EVENT_ACKED:
@@ -675,7 +680,7 @@ int CRMSClient::StateMachine()
     }
     case STATE_PASSWORD:
     {
-      ICQOwner *o = gUserManager.FetchOwner(LOCK_R);
+      ICQOwner* o = gUserManager.FetchOwner(LICQ_PPID, LOCK_R);
       if (o == NULL)
         return -1;
 
@@ -685,7 +690,7 @@ int CRMSClient::StateMachine()
       m_szCheckId = 0;
       if (!ok)
       {
-        gUserManager.DropOwner();
+        gUserManager.DropOwner(o);
         gLog.Info("%sClient failed validation from %s.\n", L_RMSxSTR,
            sock.RemoteIpStr(buf));
         fprintf(fs, "%d Invalid ID/Password.\n", CODE_INVALID);
@@ -697,7 +702,7 @@ int CRMSClient::StateMachine()
       fprintf(fs, "%d Hello %s.  Type HELP for assistance.\n", CODE_HELLO,
          o->GetAlias());
       fflush(fs);
-      gUserManager.DropOwner();
+      gUserManager.DropOwner(o);
       m_nState = STATE_COMMAND;
       break;
     }
@@ -792,7 +797,7 @@ int CRMSClient::Process_INFO()
 
   //XXX Handle the case when we have the owner
   if (szId == 0)
-    m_nUin = gUserManager.OwnerUin();
+    m_nUin = strtoul(gUserManager.OwnerId(LICQ_PPID).c_str(), (char**)NULL, 10);
 
   // Print the user info
   ICQUser *u = gUserManager.FetchUser(szId, nPPID, LOCK_R);
@@ -847,7 +852,7 @@ int CRMSClient::Process_STATUS()
       if (o)
       {
         fprintf(fs, "%d %s %s %s\n", CODE_STATUS, o->IdString(), (*it)->Name(), o->StatusStr());
-        gUserManager.DropOwner((*it)->PPID());
+        gUserManager.DropOwner(o);
       }
     }
     fprintf(fs, "%d\n", CODE_STATUSxDONE);
@@ -901,7 +906,7 @@ int CRMSClient::ChangeStatus(unsigned long nPPID, unsigned long nStatus, const c
   {
     ICQOwner *o = gUserManager.FetchOwner(nPPID, LOCK_R);
     bool b = o->StatusOffline();
-    gUserManager.DropOwner(nPPID);
+    gUserManager.DropOwner(o);
     unsigned long tag = 0;
     if (b)
     {
@@ -970,12 +975,13 @@ int CRMSClient::Process_HELP()
 int CRMSClient::Process_GROUPS()
 {
   fprintf(fs, "%d 000 All Users\n", CODE_LISTxGROUP);
-  GroupList *g = gUserManager.LockGroupList(LOCK_R);
-  for (unsigned short i = 0; i < g->size(); i++)
+  int i = 1;
+  FOR_EACH_GROUP_START_SORTED(LOCK_R)
   {
-    fprintf(fs, "%d %03d %s\n", CODE_LISTxGROUP, i + 1, (*g)[i]);
+    fprintf(fs, "%d %03d %s\n", CODE_LISTxGROUP, i, pGroup->name().c_str());
+    ++i;
   }
-  gUserManager.UnlockGroupList();
+  FOR_EACH_GROUP_END
   fprintf(fs, "%d\n", CODE_LISTxDONE);
 
   return fflush(fs);
@@ -1235,7 +1241,9 @@ int CRMSClient::Process_SMS_number()
 
 int CRMSClient::Process_SMS_message()
 {
-  unsigned long tag = licqDaemon->icqSendSms(m_szLine,m_szText,m_nUin);
+  char id[16];
+  snprintf(id, 16, "%lu", m_nUin);
+  unsigned long tag = licqDaemon->icqSendSms(id, LICQ_PPID, m_szLine, m_szText);
 
   fprintf(fs, "%d [%lu] Sending SMS to %lu (%s).\n", CODE_COMMANDxSTART,
      tag, m_nUin, m_szLine);
@@ -1285,7 +1293,7 @@ int CRMSClient::Process_AR_text()
   {
     ICQOwner *o = gUserManager.FetchOwner(m_nPPID, LOCK_W);
     o->SetAutoResponse(m_szText);
-    gUserManager.DropOwner();
+    gUserManager.DropOwner(o);
   }
   else
   {
@@ -1482,7 +1490,8 @@ int CRMSClient::Process_ADDUSER()
   {
     fprintf(fs, "%d User not added\n", CODE_ADDUSERxERROR);
   }
-  
+
+  free(szId);
   return fflush(fs);
 }
 
@@ -1501,7 +1510,7 @@ int CRMSClient::Process_REMUSER()
 
   if (nUin >= 10000)
   {
-    licqDaemon->RemoveUserFromList(nUin);
+    licqDaemon->RemoveUserFromList(data_arg, LICQ_PPID);
     fprintf(fs, "%d User removed\n", CODE_REMUSERxDONE);
   }
   else
@@ -1530,19 +1539,16 @@ int CRMSClient::Process_SECURE()
     fprintf(fs, "%d Licq secure channel not compiled. Please recompile with OpenSSL.\n", CODE_SECURExNOTCOMPILED);
     return fflush(fs);
   }
-  
 
-  if (isdigit(*data_arg))
-  {
-    nUin = strtoul(data_arg, (char**)NULL, 10);
-    while (*data_arg != '\0' && *data_arg != ' ') data_arg++;
-    NEXT_WORD(data_arg);
-  }
-   else
+  if (!isdigit(*data_arg))
   {
     fprintf(fs, "%d Invalid UIN.\n", CODE_INVALIDxUSER);
     return fflush(fs);
   }
+  char* id = strdup(data_arg);
+  nUin = strtoul(data_arg, (char**)NULL, 10);
+  while (*data_arg != '\0' && *data_arg != ' ') data_arg++;
+  NEXT_WORD(data_arg);
 
   if (nUin < 10000)
   {
@@ -1553,17 +1559,17 @@ int CRMSClient::Process_SECURE()
   if (strncasecmp(data_arg, "open", 4) == 0)
   {
     fprintf(fs, "%d Opening secure connection.\n", CODE_SECURExOPEN);
-    licqDaemon->icqOpenSecureChannel(nUin);
+    licqDaemon->icqOpenSecureChannel(id);
   }
   else
   if (strncasecmp(data_arg, "close", 5) == 0)
   {
     fprintf(fs, "%d Closing secure connection.\n", CODE_SECURExCLOSE);
-    licqDaemon->icqCloseSecureChannel(nUin);
+    licqDaemon->icqCloseSecureChannel(id);
   }
   else
   {
-   ICQUser *u = gUserManager.FetchUser(nUin,LOCK_R);
+    ICQUser* u = gUserManager.FetchUser(id, LICQ_PPID, LOCK_R);
    if (u->Secure() == 0)
    {
     fprintf(fs, "%d Status: secure connection is closed.\n", CODE_SECURExSTAT);
@@ -1574,6 +1580,7 @@ int CRMSClient::Process_SECURE()
    }
    gUserManager.DropUser(u);
   }
-  
+
+  free(id);
   return fflush(fs);
 }
