@@ -22,14 +22,17 @@
 #include "config.h"
 #endif
 
-#include <string.h>
+#include <climits>
+#include <cstdio>
+#include <cstdlib>
+#include <cstring>
+#include <cctype>
+#include <string>
 #include <sys/types.h>
 #include <unistd.h>
-#include <stdio.h>
-#include <stdlib.h>
 #include <sys/stat.h>
 #include <fcntl.h>
-#include <ctype.h>
+
 #include "time-fix.h"
 
 #ifdef HAVE_ERRNO_H
@@ -44,7 +47,6 @@ extern int errno;
 #include "licq_icq.h"
 #include "licq_user.h"
 #include "licq_constants.h"
-#include "licq_file.h"
 #include "licq_log.h"
 #include "licq_translate.h"
 #include "licq_packets.h"
@@ -54,6 +56,7 @@ extern int errno;
 
 #include "licq_icqd.h"
 
+using std::string;
 
 #define ReportMissingParams(cmdname) \
   (gLog.Info("%s `%s': missing arguments. try `help %s'\n",  \
@@ -148,10 +151,9 @@ struct command_t
 
 static int process_tok(const command_t *table,const char *tok);
 
-unsigned long
-StringToStatus(char *_szStatus)
+unsigned long StringToStatus(const char* _szStatus)
 {
-  ICQOwner *o = gUserManager.FetchOwner(LOCK_R);
+  const ICQOwner* o = gUserManager.FetchOwner(LICQ_PPID, LOCK_R);
   unsigned long nStatus = o->AddStatusFlags(0);
   int i =0;
   static struct 
@@ -169,7 +171,7 @@ StringToStatus(char *_szStatus)
     { "offline",  ICQ_STATUS_OFFLINE     },
     { NULL,       0                      }
   };
-  gUserManager.DropOwner();
+  gUserManager.DropOwner(o);
   if (_szStatus[0] == '*')
   {
     _szStatus++;
@@ -302,12 +304,8 @@ static bool atoid( const char *buff, bool bOnList,
     {
         if( strcasecmp(s, pUser->GetAlias()) == 0)
         {
-          _szId = (char *) malloc(26);
-          if( _szId )
-          { 
-            sprintf(_szId, "%ld", pUser->Uin() );
-            ret = true;
-          }
+        _szId = strdup(pUser->IdString());
+        ret = true;
           FOR_EACH_PROTO_USER_BREAK
         }
     }
@@ -336,7 +334,6 @@ static bool atoid( const char *buff, bool bOnList,
 // status 
 static int fifo_status( int argc, const char *const *argv, void *data)
 {
-  ICQOwner *o;
   CICQDaemon *d= (CICQDaemon *) data;
   const char *szStatus = argv[1];
   bool bOffline;
@@ -349,9 +346,9 @@ static int fifo_status( int argc, const char *const *argv, void *data)
   }
 
   // Determine the status to go to
-  o = gUserManager.FetchOwner(LOCK_R);
+  const ICQOwner* o = gUserManager.FetchOwner(LICQ_PPID, LOCK_R);
   bOffline = o->StatusOffline();
-  gUserManager.DropOwner();
+  gUserManager.DropOwner(o);
   nStatus = StringToStatus(const_cast<char *>(szStatus));
 
   if (nStatus == INT_MAX)
@@ -375,9 +372,9 @@ static int fifo_status( int argc, const char *const *argv, void *data)
   // Now set the auto response
   if( argc > 2 )
   {
-    ICQOwner *o = gUserManager.FetchOwner(LOCK_W);
+    ICQOwner* o = gUserManager.FetchOwner(LICQ_PPID, LOCK_W);
     o->SetAutoResponse(argv[2]);
-    gUserManager.DropOwner();
+    gUserManager.DropOwner(o);
   }
 
   return 0;
@@ -387,17 +384,15 @@ static int fifo_status( int argc, const char *const *argv, void *data)
 // auto_response <auto response>
 static int fifo_auto_response( int argc, const char *const *argv, void* /* data */)
 {
-  ICQOwner *o; 
-
   if( argc == 1 )
   {
     ReportMissingParams(argv[0]);
     return -1;
   }
 
-  o = gUserManager.FetchOwner(LOCK_W);
+  ICQOwner* o = gUserManager.FetchOwner(LICQ_PPID, LOCK_W);
   o->SetAutoResponse(argv[1]);
-  gUserManager.DropOwner();
+  gUserManager.DropOwner(o);
 
   return 0;
 }
@@ -470,17 +465,16 @@ static int fifo_sms(int argc, const char *const *argv, void *data)
   { 
     if( nPPID == LICQ_PPID )
     {
-      ICQUser *u = gUserManager.FetchUser(szId, nPPID, LOCK_R);
+      const ICQUser* u = gUserManager.FetchUser(szId, nPPID, LOCK_R);
       if (u != NULL)
       {
-        const char *szNumber = u->GetCellularNumber();
-        if (strlen(szNumber))
-          d->icqSendSms(szNumber, argv[2], u->Uin() );
+        string number = u->GetCellularNumber();
+        gUserManager.DropUser(u);
+        if (!number.empty())
+          d->icqSendSms(szId, nPPID, number.c_str(), argv[2]);
         else
           gLog.Error("%sUnable to send SMS to %s, no SMS number found.\n",
                      L_ERRORxSTR, szId);
-
-        gUserManager.DropUser(u);
       }
     }
     else
@@ -505,9 +499,8 @@ static int fifo_sms_number(int argc, const char *const *argv, void *data)
     return -1;
   }
 
-  ICQUser *owner = gUserManager.FetchOwner(LOCK_R);
-  d->icqSendSms(argv[1], argv[2], owner->Uin());
-  gUserManager.DropOwner();
+  string id = gUserManager.OwnerId(LICQ_PPID);
+  d->icqSendSms(id.c_str(), LICQ_PPID, argv[1], argv[2]);
   return 0;
 }
 
@@ -573,7 +566,6 @@ static int fifo_userinfo ( int argc, const char *const *argv, void *data)
   CICQDaemon *d = (CICQDaemon *) data;
   unsigned long nPPID;
   char *szId = 0; 
-  ICQUser *u;
   int ret = -1; 
 
   if ( argc == 1 )
@@ -584,15 +576,14 @@ static int fifo_userinfo ( int argc, const char *const *argv, void *data)
      gLog.Info(tr("%s `%s': bad protol. ICQ only alowed\n"), L_FIFOxSTR, argv[0]);
   else
   {
-    u = gUserManager.FetchUser(szId, nPPID, LOCK_R);
+    const ICQUser* u = gUserManager.FetchUser(szId, nPPID, LOCK_R);
     if (u == NULL)
       gLog.Warn(tr("%s %s: user %s not on contact list, not retrieving "
                 "info.\n"), L_WARNxSTR, argv[0], szId);
     else
     {
-      unsigned long nUin = u->Uin();
       gUserManager.DropUser(u);
-      d->icqRequestMetaInfo(nUin);
+      d->icqRequestMetaInfo(szId);
       ret = 0;
     }
   }
@@ -878,7 +869,8 @@ static bool line2argv( char *p, char **argv, int *argc, int size )
   /* trim */
   for( ; isspace(*p) ; p ++ )
     ;
-  for( q=p+strlen(p)-1; isspace(*q) ; q-- );
+  for( q=p+strlen(p)-1; isspace(*q) ; q-- )
+    ;
   q[1] = '\0'; 
 
   /* split: the ugly thing  */
@@ -924,7 +916,7 @@ static int process_tok(const command_t *table,const char *tok)
   return  bFound ? i -1 : CL_UNKNOWN;
 }
 
-void CICQDaemon::ProcessFifo(char *_szBuf)
+void CICQDaemon::ProcessFifo(const char* _szBuf)
 {
 #ifdef USE_FIFO
   int argc, index;
