@@ -10,9 +10,11 @@
 #include "config.h"
 #endif
 
-#include <ctype.h>
-
 #include "licq_chat.h"
+
+#include <cctype>
+#include <unistd.h>
+
 #include "licq_log.h"
 #include "licq_constants.h"
 #include "licq_icqd.h"
@@ -23,6 +25,7 @@
 // Localization
 #include "gettext.h"
 
+#define MAX_CONNECTS  256
 #define DEBUG_THREADS(x)
 
 
@@ -47,10 +50,8 @@ CPChat_Color::CPChat_Color(const char *_sLocalName, unsigned short _nLocalPort,
 {
   m_szName = NULL;
   m_nPort = _nLocalPort;
-  m_nUin = gUserManager.OwnerUin();
-  char szUin[24];
-  sprintf(szUin, "%lu", m_nUin);
-  m_szId = strdup(szUin);
+  m_szId = strdup(gUserManager.OwnerId(LICQ_PPID).c_str());
+  unsigned long m_nUin = atol(m_szId);
   m_nPPID = LICQ_PPID;
   m_nColorForeRed = nColorForeRed;
   m_nColorForeGreen = nColorForeGreen;
@@ -64,7 +65,7 @@ CPChat_Color::CPChat_Color(const char *_sLocalName, unsigned short _nLocalPort,
 
   buffer->PackUnsignedLong(0x65);
   buffer->PackUnsignedLong(-ICQ_VERSION_TCP);
-  buffer->PackUnsignedLong(gUserManager.OwnerUin());
+  buffer->PackUnsignedLong(m_nUin);
   buffer->PackString(_sLocalName);
   buffer->PackUnsignedShort(ReversePort(_nLocalPort));
   buffer->PackChar(nColorForeRed);
@@ -85,7 +86,7 @@ CPChat_Color::CPChat_Color(CBuffer &b)
 
   b.UnpackUnsignedLong();
   b.UnpackUnsignedLong();
-  m_nUin = b.UnpackUnsignedLong();
+  unsigned long m_nUin = b.UnpackUnsignedLong();
   char szUin[24];
   sprintf(szUin, "%lu", m_nUin);
   m_szId = strdup(szUin);
@@ -103,20 +104,25 @@ CPChat_Color::CPChat_Color(CBuffer &b)
   b.UnpackChar();
 }
 
+CPChat_Color::~CPChat_Color()
+{
+  free(m_szName);
+  free(m_szId);
+}
+
 
 //-----ChatColorFont----------------------------------------------------------------
 CChatClient::CChatClient()
 {
-  m_nVersion = m_nUin = m_nPPID = m_nIp = m_nIntIp = m_nPort = m_nMode
+  m_nVersion = m_nPPID = m_nIp = m_nIntIp = m_nPort = m_nMode
      = m_nSession = m_nHandshake = 0;
   m_szId = NULL;
 }
 
 
-CChatClient::CChatClient(ICQUser *u)
+CChatClient::CChatClient(const ICQUser* u)
 {
   m_nVersion = u->Version();
-  m_nUin = u->Uin();
   m_szId = strdup(u->IdString());
   m_nPPID = u->PPID();
   m_nIp = u->Ip();
@@ -142,13 +148,17 @@ CChatClient::CChatClient(const CChatClient &p)
   *this = p;
 }
 
+CChatClient::~CChatClient()
+{
+  free(m_szId);
+}
+
 CChatClient& CChatClient::operator=(const CChatClient &p)
 {
   if (this != &p)
   {
     m_nVersion = p.m_nVersion;
     m_nPort = p.m_nPort;
-    m_nUin = p.m_nUin;
     m_nPPID = p.m_nPPID;
     m_nIp = p.m_nIp;
     m_nIntIp = p.m_nIntIp;
@@ -167,7 +177,7 @@ bool CChatClient::LoadFromBuffer(CBuffer &b)
   m_nVersion = b.UnpackUnsignedLong();
   m_nPort = b.UnpackUnsignedShort();
   b.UnpackUnsignedShort();
-  m_nUin = b.UnpackUnsignedLong();
+  unsigned long m_nUin = b.UnpackUnsignedLong();
   char szUin[24];
   sprintf(szUin, "%lu", m_nUin);
   m_szId = strdup(szUin);
@@ -191,7 +201,7 @@ bool CChatClient::LoadFromHandshake_v2(CBuffer &b)
 
   m_nVersion = b.UnpackUnsignedLong();
   b.UnpackUnsignedLong();
-  m_nUin = b.UnpackUnsignedLong();
+  unsigned long m_nUin = b.UnpackUnsignedLong();
   char szUin[24];
   sprintf(szUin, "%lu", m_nUin);
   m_szId = strdup(szUin);
@@ -217,7 +227,7 @@ bool CChatClient::LoadFromHandshake_v4(CBuffer &b)
 
   m_nVersion = b.UnpackUnsignedLong();
   b.UnpackUnsignedLong();
-  m_nUin = b.UnpackUnsignedLong();
+  unsigned long m_nUin = b.UnpackUnsignedLong();
   char szUin[24];
   sprintf(szUin, "%lu", m_nUin);
   m_szId = strdup(szUin);
@@ -240,7 +250,7 @@ bool CChatClient::LoadFromHandshake_v6(CBuffer &b)
   CPacketTcp_Handshake_v6 hand(&b);
 
   m_nVersion = hand.VersionMajor();
-  m_nUin = hand.SourceUin();
+  unsigned long m_nUin = hand.SourceUin();
   char szUin[24];
   sprintf(szUin, "%lu", m_nUin);
   m_szId = strdup(szUin);
@@ -263,7 +273,7 @@ bool CChatClient::LoadFromHandshake_v7(CBuffer &b)
   CPacketTcp_Handshake_v7 hand(&b);
 
   m_nVersion = hand.VersionMajor();
-  m_nUin = hand.SourceUin();
+  unsigned long m_nUin = hand.SourceUin();
   char szUin[24];
   sprintf(szUin, "%lu", m_nUin);
   m_szId = strdup(szUin);
@@ -292,10 +302,8 @@ CPChat_ColorFont::CPChat_ColorFont(const char *szLocalName, unsigned short nLoca
 {
   m_szName = NULL;
   m_nPort = nLocalPort;
-  m_nUin = gUserManager.OwnerUin();
-  char szUin[24];
-  sprintf(szUin, "%lu", m_nUin);
-  m_szId = strdup(szUin);
+  m_szId = strdup(gUserManager.OwnerId(LICQ_PPID).c_str());
+  unsigned long m_nUin = atol(m_szId);
   m_nPPID = LICQ_PPID;
   m_nColorForeRed = nColorForeRed;
   m_nColorForeGreen = nColorForeGreen;
@@ -319,7 +327,7 @@ CPChat_ColorFont::CPChat_ColorFont(const char *szLocalName, unsigned short nLoca
   InitBuffer();
 
   buffer->PackUnsignedLong(0x65);
-  buffer->PackUnsignedLong(gUserManager.OwnerUin());
+  buffer->PackUnsignedLong(m_nUin);
   buffer->PackString(szLocalName);
   buffer->PackChar(nColorForeRed);
   buffer->PackChar(nColorForeGreen);
@@ -347,7 +355,7 @@ CPChat_ColorFont::CPChat_ColorFont(const char *szLocalName, unsigned short nLoca
   {
     buffer->PackUnsignedLong((*iter)->m_nVersion);
     buffer->PackUnsignedLong((*iter)->m_nPort);
-    buffer->PackUnsignedLong((*iter)->m_nUin);
+    buffer->PackUnsignedLong(strtoul((*iter)->m_szId, NULL, 10));
     buffer->PackUnsignedLong((*iter)->m_nIp);
     buffer->PackUnsignedLong((*iter)->m_nIntIp);
     buffer->PackChar((*iter)->m_nMode);
@@ -363,7 +371,7 @@ CPChat_ColorFont::CPChat_ColorFont(CBuffer &b)
   char buf[128];
 
   b.UnpackUnsignedLong();
-  m_nUin = b.UnpackUnsignedLong();
+  unsigned long m_nUin = b.UnpackUnsignedLong();
   char szUin[24];
   sprintf(szUin, "%lu", m_nUin);
   m_szId = strdup(szUin);
@@ -399,7 +407,12 @@ CPChat_ColorFont::CPChat_ColorFont(CBuffer &b)
 
 }
 
-
+CPChat_ColorFont::~CPChat_ColorFont()
+{
+  free(m_szName);
+  free(m_szFontFamily);
+  free(m_szId);
+}
 
 //-----ChatFont---------------------------------------------------------------------
 CPChat_Font::CPChat_Font(unsigned short nLocalPort, unsigned short nSession,
@@ -453,6 +466,11 @@ CPChat_Font::CPChat_Font(CBuffer &b)
   m_nFontStyle = b.UnpackChar();
 }
 
+CPChat_Font::~CPChat_Font()
+{
+  free(m_szFontFamily);
+}
+
 /*
 //-----CPChat_ChangeFontFamily-----------------------------------------------
 CPChat_ChangeFontFamily::CPChat_ChangeFontFamily(const char *szFamily)
@@ -597,7 +615,6 @@ CPChat_Beep::CPChat_Beep()
 //=====ChatUser==============================================================
 CChatUser::CChatUser()
 {
-  uin = 0;
   szId = 0;
   nPPID = 0;
   nToKick = 0;
@@ -618,6 +635,10 @@ CChatUser::CChatUser()
   pthread_mutex_init(&mutex, NULL);
 }
 
+CChatUser::~CChatUser()
+{
+  free(szId);
+}
 
 CChatEvent::CChatEvent(unsigned char nCommand, CChatUser *u, char *szData)
 {
@@ -630,8 +651,9 @@ CChatEvent::CChatEvent(unsigned char nCommand, CChatUser *u, char *szData)
 
 CChatEvent::~CChatEvent()
 {
-  if (m_szData != NULL) free(m_szData);
-  if (m_bLocked) pthread_mutex_unlock(&m_pUser->mutex);
+  free(m_szData);
+  if (m_bLocked)
+    pthread_mutex_unlock(&m_pUser->mutex);
 }
 
 
@@ -651,20 +673,19 @@ CChatManager::CChatManager(CICQDaemon *d, unsigned long nUin,
   pipe(pipe_thread);
   pipe(pipe_events);
 
-  m_nUin = nUin;
   char szUin[24];
-  sprintf(szUin, "%lu", m_nUin);
+  sprintf(szUin, "%lu", nUin);
   m_szId = strdup(szUin);
   m_nPPID = LICQ_PPID;
 
 //  m_nSession = rand();
   licqDaemon = d;
 
-  ICQOwner *o = gUserManager.FetchOwner(LOCK_R);
+  const ICQOwner* o = gUserManager.FetchOwner(LICQ_PPID, LOCK_R);
   strncpy(m_szName, o->GetAlias(), 32);
   m_szName[31] = '\0';
   m_nSession = o->Port();
-  gUserManager.DropOwner();
+  gUserManager.DropOwner(o);
 
   m_nFontFace = FONT_PLAIN;
   if (fontBold) m_nFontFace |= FONT_BOLD;
@@ -739,7 +760,7 @@ void CChatManager::StartAsClient(unsigned short nPort)
 {
   if (!StartChatServer()) return;
 
-  ICQUser *u = gUserManager.FetchUser(m_szId, LICQ_PPID, LOCK_R);
+  const ICQUser* u = gUserManager.FetchUser(m_szId, LICQ_PPID, LOCK_R);
   if (u == NULL) return;
   m_pChatClient = new CChatClient(u);
   m_pChatClient->m_nPort = nPort;
@@ -760,14 +781,13 @@ bool CChatManager::ConnectToChat(CChatClient *c)
   CChatUser *u = new CChatUser;
   u->m_pClient = c;
   u->m_pClient->m_nSession = m_nSession;
-  u->uin = c->m_nUin;
   u->szId = strdup(c->m_szId);
   u->nPPID = c->m_nPPID;
 
   bool bSendIntIp = false;
   bool bTryDirect = true;
   bool bResult = false;
-  ICQUser *temp_user = gUserManager.FetchUser(u->uin, LOCK_R);
+  const ICQUser* temp_user = gUserManager.FetchUser(u->szId, u->nPPID, LOCK_R);
   if (temp_user != NULL)
   {
     bSendIntIp = temp_user->SendIntIp();
@@ -785,12 +805,12 @@ bool CChatManager::ConnectToChat(CChatClient *c)
 
   if (!bSuccess)
   {
-    ICQOwner *o = gUserManager.FetchOwner(LOCK_R);
+    const ICQOwner* o = gUserManager.FetchOwner(LICQ_PPID, LOCK_R);
     unsigned long nIp = bSendIntIp ? o->IntIp() : o->Ip();
-    gUserManager.DropOwner();
-  
+    gUserManager.DropOwner(o);
+
     // try reverse connect
-    int nId = licqDaemon->RequestReverseConnection(c->m_nUin, c->m_nSession,
+    int nId = licqDaemon->RequestReverseConnection(c->m_szId, c->m_nSession,
                                                  nIp, LocalPort(), c->m_nPort);
     if (nId != -1)
     {
@@ -828,7 +848,7 @@ bool CChatManager::SendChatHandshake(CChatUser *u)
   gLog.Info(tr("%sChat: Shaking hands [v%d].\n"), L_TCPxSTR, VersionToUse(c->m_nVersion));
 
   // Send handshake packet:
-  if (!CICQDaemon::Handshake_Send(&u->sock, c->m_nUin, LocalPort(),
+  if (!CICQDaemon::Handshake_Send(&u->sock, c->m_szId, LocalPort(),
      VersionToUse(c->m_nVersion), false))
     return false;
 
@@ -857,8 +877,7 @@ void CChatManager::AcceptReverseConnection(TCPSocket *s)
 
   u->m_pClient = new CChatClient();
   u->m_pClient->m_nVersion = s->Version();
-  u->m_pClient->m_nUin = s->Owner();
-  u->m_pClient->m_szId = s->OwnerId();
+  u->m_pClient->m_szId = strdup(s->OwnerId());
   u->m_pClient->m_nPPID = s->OwnerPPID();
   u->m_pClient->m_nIp = s->RemoteIp();
   u->m_pClient->m_nIntIp = s->RemoteIp();
@@ -869,7 +888,6 @@ void CChatManager::AcceptReverseConnection(TCPSocket *s)
   u->m_pClient->m_nPort = 0;
   u->m_pClient->m_nSession = 0;
 
-  u->uin = u->m_pClient->m_nUin;
   u->szId = strdup(u->m_pClient->m_szId);
   u->nPPID = u->m_pClient->m_nPPID;
   u->state = CHAT_STATE_WAITxFORxCOLOR;
@@ -947,7 +965,6 @@ bool CChatManager::ProcessPacket(CChatUser *u)
       }
       gLog.Info(tr("%sChat: Received handshake from %s [v%ld].\n"), L_TCPxSTR,
          u->m_pClient->m_szId, u->sock.Version());
-      u->uin = u->m_pClient->m_nUin;
       if (u->szId)  free(u->szId);
       u->szId = strdup(u->m_pClient->m_szId);
       u->nPPID = u->m_pClient->m_nPPID;
@@ -959,7 +976,7 @@ bool CChatManager::ProcessPacket(CChatUser *u)
         for (iter = licqDaemon->m_lReverseConnect.begin();
                          iter != licqDaemon->m_lReverseConnect.end();  ++iter)
         {
-          if ((*iter)->nUin == u->uin)
+          if ((*iter)->myIdString == u->Id())
           {
             bFound = true;
             (*iter)->bSuccess = true;
@@ -1064,7 +1081,6 @@ bool CChatManager::ProcessPacket(CChatUser *u)
       gLog.Info(tr("%sChat: Received color/font packet.\n"), L_TCPxSTR);
 
       CPChat_ColorFont pin(u->sock.RecvBuffer());
-      u->uin = pin.Uin();
       u->szId = strdup(pin.Id());
       u->nPPID = pin.PPID();
 //      m_nSession = pin.Session();
@@ -1436,6 +1452,8 @@ bool CChatManager::ProcessRaw_v2(CChatUser *u)
         if (u->chatQueue.size() < 6)  return true;
         unsigned long nUin = u->chatQueue[0] | (u->chatQueue[1] << 8) |
           (u->chatQueue[2] << 16) | (u->chatQueue[3] << 24);
+        char id[16];
+        snprintf(id, 16, "%lu", nUin);
 
         // Deque all the characters
         for (unsigned short i = 0; i < 6; i++)
@@ -1445,14 +1463,14 @@ bool CChatManager::ProcessRaw_v2(CChatUser *u)
         ChatUserList::iterator iter;
         for (iter = chatUsers.begin(); iter != chatUsers.end(); ++iter)
         {
-          if((*iter)->Uin() == nUin)
+          if (strcmp((*iter)->Id(), id) == 0)
             break;
         }
 
         if (iter == chatUsers.end())   return true;
 
         CBuffer bye(4);
-        SendBuffer(&bye, CHAT_DISCONNECTIONxKICKED, nUin, true);
+        SendBuffer(&bye, CHAT_DISCONNECTIONxKICKED, id, true);
 
         CloseClient(*iter);
         break;
@@ -1723,19 +1741,21 @@ bool CChatManager::ProcessRaw_v6(CChatUser *u)
           if (u->chatQueue.size() < 6)  return true;
           unsigned long nUin = u->chatQueue[0] | (u->chatQueue[1] << 8) |
             (u->chatQueue[2] << 16) | (u->chatQueue[3] << 24);
+          char id[16];
+          snprintf(id, 16, "%lu", nUin);
 
           // Find the user and say bye-bye to him
           ChatUserList::iterator iter;
           for (iter = chatUsers.begin(); iter != chatUsers.end(); ++iter)
           {
-            if((*iter)->Uin() == nUin)
+            if (strcmp((*iter)->Id(), id) == 0)
              break;
           }
 
           if (iter == chatUsers.end())   return true;
 
           CBuffer bye(4);
-          SendBuffer(&bye, CHAT_DISCONNECTIONxKICKED, nUin, true);
+          SendBuffer(&bye, CHAT_DISCONNECTIONxKICKED, id, true);
 
           CloseClient(*iter);
           break;
@@ -1838,18 +1858,17 @@ void CChatManager::SendPacket(CPacket *p)
 
 //-----CChatManager::SendBuffer----------------------------------------------
 void CChatManager::SendBuffer(CBuffer *b, unsigned char cmd,
-                              unsigned long _nUin,
-                              bool bNotIter)
+    const char* id, bool bNotIter)
 {
   ChatUserList::iterator iter;
   ChatUserList::iterator u_iter;
   bool ok = false;
 
-  if (_nUin != 0)
+  if (id != NULL)
   {
     for (u_iter = chatUsers.begin(); u_iter != chatUsers.end(); ++u_iter)
     {
-      if ((*u_iter)->Uin() == _nUin)
+      if (strcmp((*u_iter)->Id(), id) == 0)
         break;
     }
 
@@ -1862,7 +1881,7 @@ void CChatManager::SendBuffer(CBuffer *b, unsigned char cmd,
     ok = true;
 
     // Send it to every user
-    if (_nUin == 0)
+    if (id == NULL)
     {
       for (iter = chatUsers.begin(); iter != chatUsers.end(); ++iter)
         ok = SendBufferToClient(b, cmd, *iter);
@@ -1996,8 +2015,10 @@ void CChatManager::SendCharacter(char c)
 }
 
 
-void CChatManager::SendKick(unsigned long _nUin)
+void CChatManager::SendKick(const char* id)
 {
+  unsigned long _nUin = strtoul(id, NULL, 10);
+
   // Take care of the vote stuff now
   // The user we are kicking automatically is a no vote
   // And we are an automatic yes vote
@@ -2012,37 +2033,39 @@ void CChatManager::SendKick(unsigned long _nUin)
   // requesting to kick
   CBuffer buf(4);
   buf.PackUnsignedLong(_nUin);
-  SendBuffer(&buf, CHAT_KICK, _nUin, true);
+  SendBuffer(&buf, CHAT_KICK, id, true);
 }
 
 
-void CChatManager::SendKickNoVote(unsigned long _nUin)
+void CChatManager::SendKickNoVote(const char *id)
 {
+  unsigned long _nUin = strtoul(id, NULL, 10);
+
   // Tell everyone that this user has been kicked
   CBuffer buf_TellAll(6);
   buf_TellAll.PackUnsignedLong(_nUin);
   buf_TellAll.PackChar(0x02);
   buf_TellAll.PackChar(0x01);
-  SendBuffer(&buf_TellAll, CHAT_KICKxPASS, _nUin, true);
+  SendBuffer(&buf_TellAll, CHAT_KICKxPASS, id, true);
 
   // They don't know if there was a vote or not, they just see they've been kicked
   CBuffer buf(2);
   buf.PackChar(0x02);
   buf.PackChar(0x01);
-  SendBuffer(&buf, CHAT_KICKxYOU, _nUin, false);
+  SendBuffer(&buf, CHAT_KICKxYOU, id, false);
 
   // And close the connection to the kicked user
   ChatUserList::iterator iter;
   for (iter = chatUsers.begin(); iter != chatUsers.end(); ++iter)
   {
-    if((*iter)->Uin() == _nUin)
+    if(strcmp((*iter)->Id(), id) == 0)
       break;
   }
 
   if (iter == chatUsers.end())   return;
 
   CBuffer bye(4);
-  SendBuffer(&bye, CHAT_DISCONNECTIONxKICKED, _nUin, false);
+  SendBuffer(&bye, CHAT_DISCONNECTIONxKICKED, id, false);
 
   CloseClient(*iter);
 }
@@ -2221,11 +2244,14 @@ void CChatManager::CloseChat()
 //----CChatManager::FinishKickVote-------------------------------------------
 void CChatManager::FinishKickVote(VoteInfoList::iterator iter, bool bPassed)
 {
+  char voteId[16];
+  snprintf(voteId, 16, "%lu", (*iter)->nUin);
+
   // Find the person we are kicking in the ChatUserList
   ChatUserList::iterator userIter;
   for (userIter = chatUsers.begin(); userIter != chatUsers.end(); ++userIter)
   {
-    if ((*userIter)->Uin() == (*iter)->nUin)
+    if (strcmp((*userIter)->Id(), voteId) == 0)
       break;
   }
 
@@ -2245,14 +2271,14 @@ void CChatManager::FinishKickVote(VoteInfoList::iterator iter, bool bPassed)
   buf.PackChar((*iter)->nNo);
 
   if (bPassed)
-    SendBuffer(&buf, CHAT_KICKxPASS, (*iter)->nUin, true);
+    SendBuffer(&buf, CHAT_KICKxPASS, voteId, true);
   else
-    SendBuffer(&buf, CHAT_KICKxFAIL, (*iter)->nUin, true);
+    SendBuffer(&buf, CHAT_KICKxFAIL, voteId, true);
 
   // Send the person a notice if they were kicked
   if (bPassed)
   {
-    SendBuffer(&buf, CHAT_KICKxYOU, (*iter)->nUin, false);
+    SendBuffer(&buf, CHAT_KICKxYOU, voteId, false);
     CloseClient(*userIter);
   }
 
@@ -2295,7 +2321,7 @@ char *CChatManager::ClientsStr()
   {
     if (sz[0] != '\0') nPos += sprintf(&sz[nPos], ", ");
     if ((*iter)->Name()[0] == '\0')
-      nPos += sprintf(&sz[nPos], "%lu", (*iter)->Uin());
+      nPos += sprintf(&sz[nPos], "%s", (*iter)->Id());
     else
       nPos += sprintf(&sz[nPos], "%s", (*iter)->Name());
   }
@@ -2357,16 +2383,30 @@ void *ChatManager_tep(void *arg)
         // Connection on the server port ---------------------------------------
         else if (nCurrentSocket == chatman->chatServer.Descriptor())
         {
-          CChatUser *u = new CChatUser;
-          u->m_pClient = new CChatClient;
+          if (chatman->sockman.Num() >= MAX_CONNECTS)
+          {
+            // Too many sockets, drop this one
+            gLog.Warn(tr("%sToo many connected clients, rejecting new connection.\n"), L_WARNxSTR);
+          }
+          else
+          {
+            CChatUser *u = new CChatUser;
+            u->m_pClient = new CChatClient;
 
-          chatman->chatServer.RecvConnection(u->sock);
-          chatman->sockman.AddSocket(&u->sock);
-          chatman->sockman.DropSocket(&u->sock);
+            if (chatman->chatServer.RecvConnection(u->sock))
+            {
+              chatman->sockman.AddSocket(&u->sock);
+              chatman->sockman.DropSocket(&u->sock);
 
-          u->state = CHAT_STATE_HANDSHAKE;
-          chatman->chatUsers.push_back(u);
-          gLog.Info(tr("%sChat: Received connection.\n"), L_TCPxSTR);
+              u->state = CHAT_STATE_HANDSHAKE;
+              chatman->chatUsers.push_back(u);
+              gLog.Info(tr("%sChat: Received connection.\n"), L_TCPxSTR);
+            }
+            else
+            {
+              gLog.Error(tr("%sChat: Unable to receive new connection.\n"), L_ERRORxSTR);
+            }
+          }
         }
 
         // Message from connected socket----------------------------------------
@@ -2424,7 +2464,7 @@ void *ChatWaitForSignal_tep(void *arg)
 
   pthread_setcancelstate(PTHREAD_CANCEL_DISABLE, NULL);
   gLog.Info("%sChat: Waiting for reverse connection.\n", L_TCPxSTR);
-  bool bConnected = d->WaitForReverseConnection(rc->nId, rc->u->Uin());
+  bool bConnected = d->WaitForReverseConnection(rc->nId, rc->u->Id());
   pthread_setcancelstate(PTHREAD_CANCEL_ENABLE, NULL);
 
   pthread_mutex_lock(cancel_mutex);
@@ -2461,7 +2501,7 @@ void *ChatWaitForSignal_tep(void *arg)
   pthread_mutex_unlock(cancel_mutex);
 
   bool bSendIntIp = false;
-  ICQUser *temp_user = gUserManager.FetchUser(rc->u->Uin(), LOCK_R);
+  const ICQUser* temp_user = gUserManager.FetchUser(rc->u->Id(), rc->u->PPID(), LOCK_R);
   if (temp_user != NULL)
   {
     bSendIntIp = temp_user->SendIntIp();
@@ -2617,7 +2657,7 @@ CChatManager::~CChatManager()
   }
   if (iter != cmList.end()) cmList.erase(iter);
   pthread_mutex_unlock(&cmList_mutex);
-  
-  if (m_szId) free(m_szId);
+
+  free(m_szId);
 }
 
