@@ -16,7 +16,9 @@
 #include <errno.h>
 #include <sys/types.h>
 #include <fcntl.h>
+#include <string>
 #include <string.h>
+#include <sys/time.h>
 
 // Localization
 #include "gettext.h"
@@ -26,6 +28,7 @@
 #include "licq_constants.h"
 #include "licq_icqd.h"
 #include "licq_translate.h"
+#include "licq_user.h"
 #include "support.h"
 
 #define DEBUG_THREADS(x)
@@ -40,6 +43,7 @@ const unsigned short FT_STATE_RECEIVINGxFILE = 6;
 const unsigned short FT_STATE_SENDINGxFILE = 7;
 const unsigned short FT_STATE_CONFIRMINGxFILE = 8;
 
+using std::string;
 
 
 //=====FILE==================================================================
@@ -198,13 +202,14 @@ CFileTransferManager::CFileTransferManager(CICQDaemon *d, unsigned long nUin)
   pipe(pipe_events);
 
   m_nUin = nUin;
+  sprintf(myId, "%lu", m_nUin);
   m_nSession = rand();
   licqDaemon = d;
 
-  ICQOwner *o = gUserManager.FetchOwner(LOCK_R);
+  const ICQOwner* o = gUserManager.FetchOwner(LICQ_PPID, LOCK_R);
   strncpy(m_szLocalName, o->GetAlias(), sizeof(m_szLocalName) - 1);
   m_szLocalName[sizeof(m_szLocalName) - 1] = '\0';
-  gUserManager.DropOwner();
+  gUserManager.DropOwner(o);
 
   m_nCurrentFile = m_nBatchFiles = 0;
   m_nFileSize = m_nBatchSize = m_nFilePos = m_nBatchPos = 0;
@@ -336,7 +341,7 @@ void CFileTransferManager::SendFiles(ConstFileList lPathNames, unsigned short nP
 //-----CFileTransferManager::ConnectToFileServer-----------------------------
 bool CFileTransferManager::ConnectToFileServer(unsigned short nPort)
 {
-  ICQUser *u = gUserManager.FetchUser(m_nUin, LOCK_R);
+  const ICQUser* u = gUserManager.FetchUser(myId, LICQ_PPID, LOCK_R);
   if (u == NULL)
     return false;
 
@@ -348,18 +353,18 @@ bool CFileTransferManager::ConnectToFileServer(unsigned short nPort)
   if (bTryDirect)
   {
     gLog.Info("%sFile Transfer: Connecting to server.\n", L_TCPxSTR);
-    bSuccess = licqDaemon->OpenConnectionToUser(m_nUin, &ftSock, nPort);
+    bSuccess = licqDaemon->OpenConnectionToUser(myId, &ftSock, nPort);
    }
- 
+
   bool bResult = false;
   if (!bSuccess)
   {
-    ICQOwner *o = gUserManager.FetchOwner(LOCK_R);
+    const ICQOwner* o = gUserManager.FetchOwner(LICQ_PPID, LOCK_R);
     unsigned long nIp = bSendIntIp ? o->IntIp() : o->Ip();
-    gUserManager.DropOwner();
+    gUserManager.DropOwner(o);
 
     // try reverse connect
-    int nId = licqDaemon->RequestReverseConnection(m_nUin, 0, nIp, LocalPort(),
+    int nId = licqDaemon->RequestReverseConnection(myId, 0, nIp, LocalPort(),
                                                                         nPort);
 
     if (nId != -1)
@@ -386,10 +391,10 @@ bool CFileTransferManager::SendFileHandshake()
   gLog.Info(tr("%sFile Transfer: Shaking hands.\n"), L_TCPxSTR);
 
   // Send handshake packet:
-  ICQUser *u = gUserManager.FetchUser(m_nUin, LOCK_R);
+  const ICQUser* u = gUserManager.FetchUser(myId, LICQ_PPID, LOCK_R);
   unsigned short nVersion = u->ConnectionVersion();
   gUserManager.DropUser(u);
-  if (!CICQDaemon::Handshake_Send(&ftSock, m_nUin, LocalPort(), nVersion, false))
+  if (!CICQDaemon::Handshake_Send(&ftSock, myId, LocalPort(), nVersion, false))
     return false;
 
   // Send init packet:
@@ -477,7 +482,7 @@ bool CFileTransferManager::ProcessPacket()
         for (iter = licqDaemon->m_lReverseConnect.begin();
                           iter != licqDaemon->m_lReverseConnect.end();  ++iter)
         {
-          if ((*iter)->nId == nId && (*iter)->nUin == m_nUin)
+          if ((*iter)->nId == nId && (*iter)->myIdString == myId)
           {
             bFound = true;
             (*iter)->bSuccess = true;
@@ -592,7 +597,8 @@ bool CFileTransferManager::ProcessPacket()
       char *pTmp, *pNoPath;
       for (pTmp = m_szFileName + strlen(m_szFileName);
            pTmp >= m_szFileName && *pTmp != '/';
-           pTmp--);
+           pTmp--)
+        ;
       if (pTmp >= m_szFileName && *pTmp == '/')
       {
         pNoPath = strdup(pTmp + 1);
@@ -1188,12 +1194,18 @@ void *FileTransferManager_tep(void *arg)
           }
           else
           {
-            ftman->ftServer.RecvConnection(ftman->ftSock);
-            ftman->sockman.AddSocket(&ftman->ftSock);
-            ftman->sockman.DropSocket(&ftman->ftSock);
+            if (ftman->ftServer.RecvConnection(ftman->ftSock))
+            {
+              ftman->sockman.AddSocket(&ftman->ftSock);
+              ftman->sockman.DropSocket(&ftman->ftSock);
 
-            ftman->m_nState = FT_STATE_HANDSHAKE;
-            gLog.Info(tr("%sFile Transfer: Received connection.\n"), L_TCPxSTR);
+              ftman->m_nState = FT_STATE_HANDSHAKE;
+              gLog.Info(tr("%sFile Transfer: Received connection.\n"), L_TCPxSTR);
+            }
+            else
+            {
+              gLog.Error(tr("%sFile Transfer: Unable to receive new connection.\n"), L_ERRORxSTR);
+            }
           }
         }
 
@@ -1244,7 +1256,6 @@ void *FileWaitForSignal_tep(void *arg)
   pthread_detach(pthread_self());
 
   CICQDaemon *d;
-  unsigned long nUin;
   unsigned short nPort;
   struct SFileReverseConnectInfo *rc = (struct SFileReverseConnectInfo *)arg;
   pthread_mutex_t *cancel_mutex = &CFileTransferManager::thread_cancel_mutex;
@@ -1254,13 +1265,13 @@ void *FileWaitForSignal_tep(void *arg)
     pthread_testcancel();
   pthread_cleanup_pop(0);
   d = rc->m->licqDaemon;
-  nUin = rc->m->Uin();
+  string id = rc->m->Id();
   nPort = rc->m->m_nPort;
   pthread_mutex_unlock(cancel_mutex);
 
   pthread_setcancelstate(PTHREAD_CANCEL_DISABLE, NULL);
   gLog.Info("%sFile Transfer: Waiting for reverse connection.\n", L_TCPxSTR);
-  bool bConnected = d->WaitForReverseConnection(rc->nId, nUin);
+  bool bConnected = d->WaitForReverseConnection(rc->nId, id.c_str());
   pthread_setcancelstate(PTHREAD_CANCEL_ENABLE, NULL);
 
   pthread_mutex_lock(cancel_mutex);
@@ -1286,7 +1297,7 @@ void *FileWaitForSignal_tep(void *arg)
   gLog.Info("%sFile Transfer: Reverse connection failed, trying direct.\n",
                                                                     L_TCPxSTR);
   TCPSocket s;
-  bConnected = d->OpenConnectionToUser(nUin, &s, nPort);
+  bConnected = d->OpenConnectionToUser(id.c_str(), &s, nPort);
   pthread_setcancelstate(PTHREAD_CANCEL_ENABLE, NULL);
 
   pthread_mutex_lock(cancel_mutex);
