@@ -9,20 +9,24 @@
 #ifndef LICQTHREADS_H
 #define LICQTHREADS_H
 
-#include <errno.h>
+#include "licq_icqd.h"
+
+#include <cerrno>
+#include <unistd.h>
 
 #include "pthread_rdwr.h"
 #include "time-fix.h"
 
-#include "licq_icqd.h"
 #include "licq_log.h"
 #include "licq_packets.h"
+#include "licq_oscarservice.h"
 #include "licq_plugind.h"
 #include "licq.h"
 
 // Localization
 #include "gettext.h"
 
+#define MAX_CONNECTS  256
 #define DEBUG_THREADS(x)
 //#define DEBUG_THREADS(x) gLog.Info(x)
 
@@ -379,11 +383,12 @@ void *ProcessRunningEvent_Client_tep(void *p)
   // Check if the socket is connected
   if (e->m_nSocketDesc == -1)
   {
-    unsigned long nDestinationUin = e->m_nDestinationUin;
+    string id = e->Id();
+    unsigned long ppid = e->PPID();
     unsigned char nChannel = e->Channel();
     pthread_setcancelstate(PTHREAD_CANCEL_DISABLE, NULL);
-   
-    ICQUser *u = gUserManager.FetchUser(nDestinationUin, LOCK_R);
+
+    const ICQUser* u = gUserManager.FetchUser(id.c_str(), ppid, LOCK_R);
     if (u == NULL)
     {
       if (d->DoneEvent(e, EVENT_ERROR) != NULL)
@@ -403,20 +408,19 @@ void *ProcessRunningEvent_Client_tep(void *p)
     bool bSendIntIp = u->SendIntIp();
     gUserManager.DropUser(u);
 
-    ICQOwner *o = gUserManager.FetchOwner(LOCK_R);
+    const ICQOwner* o = gUserManager.FetchOwner(LICQ_PPID, LOCK_R);
     unsigned long nIP = bSendIntIp ? o->IntIp() : o->Ip();
     unsigned short nLocalPort = o->Port();
-    gUserManager.DropOwner();
+    gUserManager.DropOwner(o);
 
     int socket = -1;
     if (!bSendIntIp && nVersion > 6 && nMode != MODE_DIRECT)
     {
-      int nId = d->RequestReverseConnection(nDestinationUin, nChannel, nIP,
-                                            nLocalPort, nRemotePort);
+      int nId = d->RequestReverseConnection(id.c_str(), nChannel, nIP, nLocalPort, nRemotePort);
       if (nId != -1)
       {
-        d->WaitForReverseConnection(nId, nDestinationUin);
-        u = gUserManager.FetchUser(nDestinationUin, LOCK_R);
+        d->WaitForReverseConnection(nId, id.c_str());
+        u = gUserManager.FetchUser(id.c_str(), LICQ_PPID, LOCK_R);
         if (u == NULL)
         {
           if (d->DoneEvent(e, EVENT_ERROR) != NULL)
@@ -440,12 +444,12 @@ void *ProcessRunningEvent_Client_tep(void *p)
         pthread_testcancel();
         pthread_setcancelstate(PTHREAD_CANCEL_DISABLE, NULL);
 
-        socket = d->ConnectToUser(nDestinationUin, nChannel);
+        socket = d->ConnectToUser(id.c_str(), nChannel);
       }
     }
     else
     {
-      socket = d->ConnectToUser(nDestinationUin, nChannel);
+      socket = d->ConnectToUser(id.c_str(), nChannel);
 
       // if we failed, try through server
       if (socket == -1)
@@ -454,12 +458,12 @@ void *ProcessRunningEvent_Client_tep(void *p)
         pthread_testcancel();
         pthread_setcancelstate(PTHREAD_CANCEL_DISABLE, NULL);
 
-        int nId = d->RequestReverseConnection(nDestinationUin, nChannel, nIP,
+        int nId = d->RequestReverseConnection(id.c_str(), nChannel, nIP,
                                               nLocalPort, nRemotePort);
         if (nId != -1)
         {
-          d->WaitForReverseConnection(nId, nDestinationUin);
-          u = gUserManager.FetchUser(nDestinationUin, LOCK_R);
+          d->WaitForReverseConnection(nId, id.c_str());
+          u = gUserManager.FetchUser(id.c_str(), LICQ_PPID, LOCK_R);
           if (u == NULL)
           {
             if (d->DoneEvent(e, EVENT_ERROR) != NULL)
@@ -599,8 +603,8 @@ void *ReverseConnectToUser_tep(void *v)
 
   CReverseConnectToUserData *p = (CReverseConnectToUserData *)v;
 
-  gLicqDaemon->ReverseConnectToUser(p->nUin, p->nIp, p->nPort, p->nVersion,
-    p->nFailedPort, p->nId, p->nMsgID1, p->nMsgID2);
+  gLicqDaemon->ReverseConnectToUser(p->myIdString.c_str(), p->nIp, p->nPort,
+      p->nVersion, p->nFailedPort, p->nId, p->nMsgID1, p->nMsgID2);
 
   delete p;
 
@@ -665,7 +669,7 @@ void *MonitorSockets_tep(void *p)
   CICQDaemon *d = (CICQDaemon *)p;
 
   fd_set f;
-  int nSocketsAvailable, nCurrentSocket, l;
+  int nSocketsAvailable, nCurrentSocket, nServiceSocket, l;
   char buf[1024];
 
   while (true)
@@ -691,6 +695,14 @@ void *MonitorSockets_tep(void *p)
 
     /*pthread_testcancel();
     pthread_setcancelstate(PTHREAD_CANCEL_DISABLE, NULL);*/
+    if (d->m_xBARTService)
+    {
+      COscarService *svc = d->m_xBARTService;
+      nServiceSocket = svc->GetSocketDesc();
+    }
+    else
+      nServiceSocket = -1;
+                                                  
     nCurrentSocket = 0;
     while (nSocketsAvailable > 0 && nCurrentSocket < l)
     {
@@ -722,7 +734,8 @@ void *MonitorSockets_tep(void *p)
         else
         {
           INetSocket *s = gSocketManager.FetchSocket(nCurrentSocket);
-          if (s != NULL && s->Owner() == gUserManager.OwnerUin() &&
+          if (s != NULL && s->OwnerId() != NULL &&
+              s->OwnerId() == gUserManager.OwnerId(LICQ_PPID) &&
               d->m_nTCPSrvSocketDesc == -1)
           {
             /* This is the server socket and it is about to be destoryed
@@ -764,12 +777,43 @@ void *MonitorSockets_tep(void *p)
             }
           }
 
+          // Message from the service sockets -----------------------------------
+          else if (nCurrentSocket == nServiceSocket)
+          {
+            DEBUG_THREADS("[MonitorSockets_tep] Data on BART service socket.\n");
+            COscarService *svc = d->m_xBARTService;
+            SrvSocket *sock_svc = static_cast<SrvSocket*>(s);
+            if (sock_svc->Recv())
+            {
+              CBuffer packet(sock_svc->RecvBuffer());
+              sock_svc->ClearRecvBuffer();
+              gSocketManager.DropSocket(sock_svc);
+              if (!svc->ProcessPacket(packet))
+              {
+                gLog.Warn(tr("%sCan't process packet for service 0x%02X.\n"),
+                          L_WARNxSTR, svc->GetFam());
+                svc->ResetSocket();
+                svc->ChangeStatus(STATUS_UNINITIALIZED);
+                gSocketManager.CloseSocket(nCurrentSocket);
+              }
+            }
+            else
+            {
+              gLog.Warn(tr("%sCan't receive packet for service 0x%02X.\n"),
+                        L_WARNxSTR, svc->GetFam());
+              svc->ResetSocket();
+              svc->ChangeStatus(STATUS_UNINITIALIZED);
+              gSocketManager.DropSocket(sock_svc);
+              gSocketManager.CloseSocket(nCurrentSocket);
+            }
+          }
+
           // Connection on the server port -------------------------------------
           else if (nCurrentSocket == d->m_nTCPSocketDesc)
           {
             DEBUG_THREADS("[MonitorSockets_tep] Data on listening TCP socket."
                           "\n");
-            TCPSocket *tcp = static_cast<TCPSocket *>(s);
+            TCPSocket* tcp = static_cast<TCPSocket *>(s);
             if (tcp == NULL)
             {
               gLog.Warn(tr("%sInvalid server TCP socket in set.\n"), L_WARNxSTR);
@@ -777,11 +821,24 @@ void *MonitorSockets_tep(void *p)
             }
             else
             {
-              TCPSocket *newSocket = new TCPSocket(0);
-              tcp->RecvConnection(*newSocket);
+              TCPSocket *newSocket = new TCPSocket();
+              bool ok = tcp->RecvConnection(*newSocket);
               gSocketManager.DropSocket(tcp);
-              gSocketManager.AddSocket(newSocket);
-              gSocketManager.DropSocket(newSocket);
+
+              // Make sure we can handle another socket before accepting it
+              if (!ok || gSocketManager.Num() > MAX_CONNECTS)
+              {
+                // Too many sockets, drop this one
+                char remoteIp[32];
+                gLog.Warn(tr("%sToo many connected sockets, rejecting connection from %s.\n"),
+                    L_WARNxSTR, newSocket->RemoteIpStr(remoteIp));
+                delete newSocket;
+              }
+              else
+              {
+                gSocketManager.AddSocket(newSocket);
+                gSocketManager.DropSocket(newSocket);
+              }
             }
           }
 
@@ -803,13 +860,13 @@ void *MonitorSockets_tep(void *p)
             {
               int err = tcp->Error();
               if (err == 0)
-                gLog.Info(tr("%sConnection to %lu was closed.\n"), L_TCPxSTR
-                                                             , tcp->Owner());
+                gLog.Info(tr("%sConnection to %s was closed.\n"), L_TCPxSTR,
+                    tcp->OwnerId());
               else
               {
                 char buf[128];
-                gLog.Info(tr("%sConnection to %lu lost:\n%s%s.\n"), L_TCPxSTR,
-                          tcp->Owner(), L_BLANKxSTR, tcp->ErrorStr(buf, 128));
+                gLog.Info(tr("%sConnection to %s lost:\n%s%s.\n"), L_TCPxSTR,
+                    tcp->OwnerId(), L_BLANKxSTR, tcp->ErrorStr(buf, 128));
               }
               ICQUser *u = gUserManager.FetchUser(tcp->OwnerId(),
                 tcp->OwnerPPID(), LOCK_W);
@@ -835,7 +892,7 @@ void *MonitorSockets_tep(void *p)
             // Process the packet if the buffer is full
             if (tcp->RecvBufferFull())
             {
-              if (tcp->Owner() == 0)
+              if (tcp->OwnerPPID() != LICQ_PPID)
                 r = d->ProcessTcpHandshake(tcp);
               else
                 r = d->ProcessTcpPacket(tcp);
@@ -845,8 +902,8 @@ void *MonitorSockets_tep(void *p)
             // Kill the socket if there was a problem
             if (!r)
             {
-              gLog.Info(tr("%sClosing connection to %lu.\n"), L_TCPxSTR,
-                                                          tcp->Owner());
+              gLog.Info(tr("%sClosing connection to %s.\n"), L_TCPxSTR,
+                  tcp->OwnerId());
               gSocketManager.DropSocket(tcp);
               gSocketManager.CloseSocket(nCurrentSocket);
               d->FailEvents(nCurrentSocket, 0);
@@ -901,7 +958,11 @@ void *Shutdown_tep(void *p)
 
   // Cancel the update users thread
   pthread_cancel(d->thread_updateusers);
-  
+
+  // Cancel the BART service thread
+  if (d->m_xBARTService)
+    pthread_cancel(d->thread_ssbiservice);
+
   // Join our threads
   pthread_join(d->thread_monitorsockets, NULL);
 
@@ -937,15 +998,25 @@ void *UpdateUsers_tep(void *p)
       FOR_EACH_PROTO_USER_START(LICQ_PPID, LOCK_R)
       {
         bool bSent = false;
+        bool bBART = false;
 
         if (d->AutoUpdateInfo() && !pUser->UserUpdated() &&
             pUser->ClientTimestamp() != pUser->OurClientTimestamp()
             && pUser->ClientTimestamp() != 0)
         {
-          d->icqRequestMetaInfo(pUser->Uin());
+          d->icqRequestMetaInfo(pUser->IdString());
           bSent = true;
         }
 
+        if (d->UseServerSideBuddyIcons() && d->AutoUpdateInfo() &&
+            strlen(pUser->BuddyIconHash()) > 0 &&
+            strcmp(pUser->BuddyIconHash(), pUser->OurBuddyIconHash()) != 0)
+        {
+          d->m_xBARTService->SendEvent(pUser->IdString(), ICQ_SNACxBART_DOWNLOADxREQUEST, true);
+          bSent = true;
+          bBART = true;
+        }
+        
         if (!pUser->StatusOffline() && !pUser->UserUpdated() &&
             //Don't bother clients that we know don't support plugins
             pUser->Version() >= 7 &&
@@ -974,7 +1045,8 @@ void *UpdateUsers_tep(void *p)
             gLog.Info("Updating %s's info plugins.\n", pUser->GetAlias());
             d->icqRequestInfoPlugin(pUser, true, PLUGIN_QUERYxINFO);
             d->icqRequestInfoPlugin(pUser, true, PLUGIN_PHONExBOOK);
-            d->icqRequestInfoPlugin(pUser, true, PLUGIN_PICTURE);
+            if (!bBART) // Send only if we didn't request BART already
+              d->icqRequestInfoPlugin(pUser, true, PLUGIN_PICTURE);
             bSent = true;
           }
 
