@@ -10,15 +10,16 @@
 #include "config.h"
 #endif
 
-#include <string.h>
+#include <cassert>
+#include <cctype>
+#include <cstdio>
+#include <cstdlib>
+#include <cstring>
 #include <sys/types.h>
 #include <unistd.h>
-#include <stdio.h>
-#include <stdlib.h>
 #include <sys/stat.h>
 #include <fcntl.h>
-#include <ctype.h>
-#include <assert.h>
+
 #include "time-fix.h"
 
 // Localization
@@ -32,12 +33,14 @@ extern int errno;
 
 #include "licq_icq.h"
 #include "licq_user.h"
+#include "licq_oscarservice.h"
 #include "licq_constants.h"
 #include "licq_file.h"
 #include "licq_log.h"
 #include "licq_translate.h"
 #include "licq_packets.h"
 #include "licq_plugind.h"
+#include "licq_proxy.h"
 #include "licq_gpg.h"    // ##
 #include "licq.h"
 #include "support.h"
@@ -158,11 +161,14 @@ CICQDaemon::CICQDaemon(CLicq *_licq)
   m_bLoggingOn = false;
   m_bOnlineNotifies = true;
   m_bVerify = false;
+  m_bNeedSalt = true;
   m_szRegisterPasswd = 0;
   m_nRegisterThreadId = 0;
 
   fifo_fs = NULL;
 
+  receivedUserList.clear();
+
   // Begin parsing the config file
   snprintf(m_szConfigFile, MAX_FILENAME_LEN, "%s/%s", BASE_DIR, "licq.conf");
   m_szConfigFile[MAX_FILENAME_LEN - 1] = '\0';
@@ -270,23 +276,23 @@ CICQDaemon::CICQDaemon(CLicq *_licq)
   m_szProxyPasswd = (char *)malloc(strlen(t_str) + 1);
   strcpy(m_szProxyPasswd, t_str);
 
+  // Services
+  m_xBARTService = NULL;
+
   // Misc
   licqConf.ReadBool("UseSS", m_bUseSS, true); // server side list
+  licqConf.ReadBool("UseBART", m_bUseBART, true); // server side buddy icons
   licqConf.ReadBool("SendTypingNotification", m_bSendTN, true);
   licqConf.ReadBool("ReconnectAfterUinClash", m_bReconnectAfterUinClash, false);
 
   // -----OnEvent configuration-----
-  char szOnEventCommand[MAX_FILENAME_LEN], *szOnParams[MAX_ON_EVENT];
+  string onEventCommand, onEventParams[MAX_ON_EVENT];
   unsigned short nOnEventCmdType;
 
   licqConf.SetSection("onevent");
   licqConf.ReadNum("Enable", nOnEventCmdType, 0);
   licqConf.ReadBool("AlwaysOnlineNotify", m_bAlwaysOnlineNotify, false);
   m_xOnEventManager.SetCommandType(nOnEventCmdType);
-  for (int i = 0; i < MAX_ON_EVENT; i++) {
-    szOnParams[i] = new char[MAX_FILENAME_LEN];
-    szOnParams[i][0] = '\0';
-  }
 
   // Prepare default values for onEvent
   char DEF_MESSAGE[MAX_FILENAME_LEN];
@@ -313,17 +319,15 @@ CICQDaemon::CICQDaemon(CLicq *_licq)
   strncat(DEF_SYSMSG,  "sounds/icq/System.wav", MAX_APPEND);
   strncat(DEF_MSGSENT, "sounds/icq/Message.wav", MAX_APPEND);
 
-  licqConf.ReadStr("Command", szOnEventCommand, "play");
-  licqConf.ReadStr("Message", szOnParams[ON_EVENT_MSG], DEF_MESSAGE);
-  licqConf.ReadStr("Url", szOnParams[ON_EVENT_URL], DEF_URL);
-  licqConf.ReadStr("Chat", szOnParams[ON_EVENT_CHAT], DEF_CHAT);
-  licqConf.ReadStr("File", szOnParams[ON_EVENT_FILE], DEF_FILE);
-  licqConf.ReadStr("OnlineNotify", szOnParams[ON_EVENT_NOTIFY], DEF_NOTIFY);
-  licqConf.ReadStr("SysMsg", szOnParams[ON_EVENT_SYSMSG], DEF_SYSMSG);
-  licqConf.ReadStr("MsgSent", szOnParams[ON_EVENT_MSGSENT], DEF_MSGSENT);
-  m_xOnEventManager.SetParameters(szOnEventCommand, (const char **)szOnParams);
-  for (int i = 0; i < MAX_ON_EVENT; i++)
-    delete [] szOnParams[i];
+  licqConf.readString("Command", onEventCommand, "play");
+  licqConf.readString("Message", onEventParams[ON_EVENT_MSG], DEF_MESSAGE);
+  licqConf.readString("Url", onEventParams[ON_EVENT_URL], DEF_URL);
+  licqConf.readString("Chat", onEventParams[ON_EVENT_CHAT], DEF_CHAT);
+  licqConf.readString("File", onEventParams[ON_EVENT_FILE], DEF_FILE);
+  licqConf.readString("OnlineNotify", onEventParams[ON_EVENT_NOTIFY], DEF_NOTIFY);
+  licqConf.readString("SysMsg", onEventParams[ON_EVENT_SYSMSG], DEF_SYSMSG);
+  licqConf.readString("MsgSent", onEventParams[ON_EVENT_MSGSENT], DEF_MSGSENT);
+  m_xOnEventManager.setParameters(onEventCommand, onEventParams);
 
   // Statistics
   m_nResetTime = 0;
@@ -374,7 +378,7 @@ bool CICQDaemon::Start()
   char sz[MAX_FILENAME_LEN];
   int nResult = 0;
 
-  TCPSocket *s = new TCPSocket(0);
+  TCPSocket* s = new TCPSocket();
   m_nTCPSocketDesc = StartTCPServer(s);
   if (m_nTCPSocketDesc == -1)
   {
@@ -383,12 +387,12 @@ bool CICQDaemon::Start()
      return false;
   }
   gSocketManager.AddSocket(s);
-  ICQOwner *o = gUserManager.FetchOwner(LOCK_W);
+  ICQOwner* o = gUserManager.FetchOwner(LICQ_PPID, LOCK_W);
   if (o != NULL)
   {
     o->SetIntIp(s->LocalIp());
     o->SetPort(s->LocalPort());
-    gUserManager.DropOwner();
+    gUserManager.DropOwner(o);
   }
   CPacket::SetLocalPort(s->LocalPort());
   gSocketManager.DropSocket(s);
@@ -447,10 +451,23 @@ bool CICQDaemon::Start()
   nResult = pthread_create(&thread_updateusers, NULL, &UpdateUsers_tep, this);
   if (nResult != 0)
   {
-    gLog.Error("%sUnable to start ping thread:\n%s%s.\n", L_ERRORxSTR, L_BLANKxSTR, strerror(nResult));
+    gLog.Error("%sUnable to start users update thread:\n%s%s.\n", L_ERRORxSTR, L_BLANKxSTR, strerror(nResult));
     return false;
   }
 
+  if (UseServerSideBuddyIcons())
+  {
+    m_xBARTService = new COscarService(this, ICQ_SNACxFAM_BART);
+    nResult = pthread_create(&thread_ssbiservice, NULL,
+                             &OscarServiceSendQueue_tep, m_xBARTService);
+    if (nResult != 0)
+    {
+      gLog.Error(tr("%sUnable to start BART service thread:\n%s%s.\n"),
+                 L_ERRORxSTR, L_BLANKxSTR, strerror(nResult));
+      return false;
+    }
+  }
+
   return true;
 }
 
@@ -646,10 +663,10 @@ void CICQDaemon::UnregisterProtoPlugin()
   pthread_mutex_unlock(&licq->mutex_protoplugins);
 }
 
-char *CICQDaemon::ProtoPluginName(unsigned long _nPPID)
+const char* CICQDaemon::ProtoPluginName(unsigned long _nPPID) const
 {
   ProtoPluginsListIter it;
-  char *p = 0;
+  const char* p = 0;
 
   pthread_mutex_lock(&licq->mutex_protoplugins);
   for (it = licq->list_protoplugins.begin();
@@ -658,7 +675,7 @@ char *CICQDaemon::ProtoPluginName(unsigned long _nPPID)
   {
     if ((*it)->PPID() == _nPPID)
     {
-      p = (char *)(*it)->Name();
+      p = (*it)->Name();
       break;
     }
   }
@@ -681,7 +698,7 @@ void CICQDaemon::ProtoPluginShutdown(unsigned short _nId)
 
 //---Version-------------------------------------------------------------------
 /*! \brief Returns the version of Licq */
-const char *CICQDaemon::Version()
+const char* CICQDaemon::Version() const
 {
   return licq->Version();
 }
@@ -775,11 +792,15 @@ void CICQDaemon::SaveConf()
 
   // Utility tab
   licqConf.WriteStr("UrlViewer", m_szUrlViewer);
-  const char *pc = strrchr(gTranslator.getMapName(), '/');
-  if (pc != NULL)
-    pc++;
+  const char* pc = gTranslator.getMapName();
+  if (pc == NULL)
+    pc = "none";
   else
-    pc = gTranslator.getMapName();
+  {
+    const char* pc2 = strrchr(pc, '/');
+    if (pc2 != NULL)
+      pc = pc2++;
+  }
   licqConf.WriteStr("Translation", pc);
   licqConf.WriteStr("Terminal", m_szTerminal);
   licqConf.WriteStr("Errors", m_szErrorFile);
@@ -804,8 +825,10 @@ void CICQDaemon::SaveConf()
 
   // Misc
   licqConf.WriteBool("UseSS", m_bUseSS); // server side list
+  licqConf.WriteBool("UseBART", m_bUseBART); // server side buddy icons
   licqConf.WriteBool("SendTypingNotification", m_bSendTN); 
   licqConf.WriteBool("ReconnectAfterUinClash", m_bReconnectAfterUinClash);
+  licqConf.WriteStr("DefaultUserEncoding", gUserManager.DefaultUserEncoding());
 
   // save the sound stuff
   licqConf.SetSection("onevent");
@@ -813,14 +836,14 @@ void CICQDaemon::SaveConf()
   licqConf.WriteNum("Enable", oem->CommandType());
   licqConf.WriteBool("AlwaysOnlineNotify", m_bAlwaysOnlineNotify);
   oem->Lock();
-  licqConf.WriteStr("Command", oem->Command());
-  licqConf.WriteStr("Message", oem->Parameter(ON_EVENT_MSG));
-  licqConf.WriteStr("Url", oem->Parameter(ON_EVENT_URL));
-  licqConf.WriteStr("Chat",oem->Parameter(ON_EVENT_CHAT));
-  licqConf.WriteStr("File",oem->Parameter(ON_EVENT_FILE));
-  licqConf.WriteStr("OnlineNotify", oem->Parameter(ON_EVENT_NOTIFY));
-  licqConf.WriteStr("SysMsg", oem->Parameter(ON_EVENT_SYSMSG));
-  licqConf.WriteStr("MsgSent", oem->Parameter(ON_EVENT_MSGSENT));
+  licqConf.writeString("Command", oem->command());
+  licqConf.writeString("Message", oem->parameter(ON_EVENT_MSG));
+  licqConf.writeString("Url", oem->parameter(ON_EVENT_URL));
+  licqConf.writeString("Chat",oem->parameter(ON_EVENT_CHAT));
+  licqConf.writeString("File",oem->parameter(ON_EVENT_FILE));
+  licqConf.writeString("OnlineNotify", oem->parameter(ON_EVENT_NOTIFY));
+  licqConf.writeString("SysMsg", oem->parameter(ON_EVENT_SYSMSG));
+  licqConf.writeString("MsgSent", oem->parameter(ON_EVENT_MSGSENT));
   oem->Unlock();
 
   licqConf.FlushFile();
@@ -858,12 +881,10 @@ void CICQDaemon::SaveConf()
 
 //++++++NOT MT SAFE+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 
-const char *CICQDaemon::Terminal()       { return m_szTerminal; }
 void CICQDaemon::SetTerminal(const char *s)  { SetString(&m_szTerminal, s); }
-bool CICQDaemon::AlwaysOnlineNotify()  { return m_bAlwaysOnlineNotify; }
 void CICQDaemon::SetAlwaysOnlineNotify(bool b)  { m_bAlwaysOnlineNotify = b; }
 
-const char *CICQDaemon::getUrlViewer()
+const char* CICQDaemon::getUrlViewer() const
 {
   if ((strcmp(m_szUrlViewer, "none") == 0) || (strlen(m_szUrlViewer) == 0))
     return (NULL);
@@ -975,38 +996,46 @@ void CICQDaemon::InitProxy()
     delete m_xProxy;
     m_xProxy = NULL;
   }
+  m_xProxy = CreateProxy();
+}
+
+ProxyServer *CICQDaemon::CreateProxy()
+{
+  ProxyServer *Proxy = NULL;
 
   switch (m_nProxyType)
   {
     case PROXY_TYPE_HTTP :
-      m_xProxy = new HTTPProxyServer();
+      Proxy = new HTTPProxyServer();
       break;
     default:
       break;
   }
 
-  if (m_xProxy != NULL)
+  if (Proxy != NULL)
   {
     gLog.Info(tr("%sResolving proxy: %s:%d...\n"), L_INITxSTR, m_szProxyHost, m_nProxyPort);
-    if (!m_xProxy->SetProxyAddr(m_szProxyHost, m_nProxyPort)) {
+    if (!Proxy->SetProxyAddr(m_szProxyHost, m_nProxyPort))
+    {
       char buf[128];
 
       gLog.Warn(tr("%sUnable to resolve proxy server %s:\n%s%s.\n"), L_ERRORxSTR,
-                 m_szProxyHost, L_BLANKxSTR, m_xProxy->ErrorStr(buf, 128));
-      delete m_xProxy;
-      m_xProxy = NULL;
+                m_szProxyHost, L_BLANKxSTR, Proxy->ErrorStr(buf, 128));
+      delete Proxy;
+      Proxy = NULL;
     }
 
-    if (m_xProxy)
+    if (Proxy)
     {
       if (m_bProxyAuthEnabled)
-        m_xProxy->SetProxyAuth(m_szProxyLogin, m_szProxyPasswd);
+        Proxy->SetProxyAuth(m_szProxyLogin, m_szProxyPasswd);
 
-      m_xProxy->InitProxy();
+      Proxy->InitProxy();
     }
   }
-}
 
+  return Proxy;
+}
 
 unsigned short VersionToUse(unsigned short v_in)
 {
@@ -1111,8 +1140,27 @@ void CICQDaemon::SetIgnore(unsigned short n, bool b)
     m_nIgnoreTypes &= ~n;
 }
 
+void CICQDaemon::SetUseServerSideBuddyIcons(bool b)
+{
+  if (b && m_xBARTService == NULL)
+  {
+    m_xBARTService = new COscarService(this, ICQ_SNACxFAM_BART);
+    int nResult = pthread_create(&thread_ssbiservice, NULL,
+                                 &OscarServiceSendQueue_tep, m_xBARTService);
+    if (nResult != 0)
+    {
+      gLog.Error(tr("%sUnable to start BART service thread:\n%s%s.\n"),
+                 L_ERRORxSTR, L_BLANKxSTR, strerror(nResult));
+    }
+    else
+      m_bUseBART = true;
+  }
+  else
+    m_bUseBART = b;
+}
+                                    
 bool CICQDaemon::AddUserToList(const char *szId, unsigned long nPPID,
-                               bool bNotify, bool bTempUser)
+                               bool bNotify, bool bTempUser, unsigned short groupId)
 {
   // Don't add invalid uins
   if (szId == 0 || nPPID == 0) return false;
@@ -1132,11 +1180,11 @@ bool CICQDaemon::AddUserToList(const char *szId, unsigned long nPPID,
 
   // this notify is for local only adds
   if (nPPID == LICQ_PPID && m_nTCPSrvSocketDesc != -1 && bNotify && !bTempUser)
-    icqAddUser(szId);
+    icqAddUser(szId, false, groupId);
   else if (nPPID != LICQ_PPID && bNotify)
     PushProtoSignal(new CAddUserSignal(szId, false), nPPID);
   
-  PushPluginSignal(new CICQSignal(SIGNAL_UPDATExLIST, LIST_ADD, szId, nPPID));
+  PushPluginSignal(new CICQSignal(SIGNAL_UPDATExLIST, LIST_ADD, szId, nPPID, groupId));
 
   return true;
 }
@@ -1149,29 +1197,15 @@ bool CICQDaemon::AddUserToList(const char *szId, unsigned long nPPID,
  * \return Returns true on success, else returns false. Please note that when 
  * this call returns the user is not locked.
  */
-bool CICQDaemon::AddUserToList(unsigned long nUin, bool bNotify)
+bool CICQDaemon::AddUserToList(unsigned long nUin, bool bNotify, bool bTempUser,
+                               unsigned short groupId)
 {
   // Don't add invalid uins
   if (nUin == 0) return false;
 
-  // Don't add a user we already have
-  if (gUserManager.IsOnList(nUin))
-  {
-    gLog.Warn(tr("%sUser %lu already on contact list.\n"), L_WARNxSTR, nUin);
-    return false;
-  }
-
-  ICQUser *u = new ICQUser(nUin);
-  gUserManager.AddUser(u);
-  gUserManager.DropUser(u);
-  SaveUserList();
-
-  // this notify is for local only adds
-  if (m_nTCPSrvSocketDesc != -1 && bNotify)  icqAddUser(nUin);
-
-  PushPluginSignal(new CICQSignal(SIGNAL_UPDATExLIST, LIST_ADD, nUin));
-
-  return true;
+  char szUin[24];
+  sprintf(szUin, "%lu", nUin);
+  return AddUserToList(szUin, LICQ_PPID, bNotify, bTempUser, groupId);
 }
 
 
@@ -1214,17 +1248,14 @@ void CICQDaemon::AddUserToList(ICQUser *nu)
 //-----RemoveUserFromList-------------------------------------------------------
 void CICQDaemon::RemoveUserFromList(unsigned long _nUin)
 {
-  if (m_nTCPSrvSocketDesc != -1) icqRemoveUser(_nUin);
-
-  gUserManager.RemoveUser(_nUin);
-  SaveUserList();
-
-  PushPluginSignal(new CICQSignal(SIGNAL_UPDATExLIST, LIST_REMOVE, _nUin));
+  char szUin[24];
+  sprintf(szUin, "%lu", _nUin);
+  RemoveUserFromList(szUin, LICQ_PPID);
 }
 
 void CICQDaemon::RemoveUserFromList(const char *szId, unsigned long nPPID)
 {
-  ICQUser *u = gUserManager.FetchUser(szId, nPPID, LOCK_R);
+  const ICQUser* u = gUserManager.FetchUser(szId, nPPID, LOCK_R);
   if (!u) return;
   bool bTempUser = u->NotInList();
   gUserManager.DropUser(u);
@@ -1315,6 +1346,13 @@ bool CICQDaemon::AddUserEvent(ICQUser *u, CUserEvent *e)
  *
  *--------------------------------------------------------------------------*/
 void CICQDaemon::RejectEvent(unsigned long nUin, CUserEvent *e)
+{
+  char szUin[24];
+  sprintf(szUin, "%lu", nUin);
+  RejectEvent(szUin, e);
+}
+
+void CICQDaemon::RejectEvent(const char* id, CUserEvent* e)
 {
   if (m_szRejectFile == NULL) return;
 
@@ -1325,8 +1363,8 @@ void CICQDaemon::RejectEvent(unsigned long nUin, CUserEvent *e)
   }
   else
   {
-    fprintf(f, "Event from new user (%lu) rejected: \n%s\n--------------------\n\n",
-            nUin, e->Text());
+    fprintf(f, "Event from new user (%s) rejected: \n%s\n--------------------\n\n",
+        id, e->Text());
     chmod(m_szRejectFile, 00600);
     fclose(f);
   }
@@ -1334,23 +1372,6 @@ void CICQDaemon::RejectEvent(unsigned long nUin, CUserEvent *e)
   m_sStats[STATS_EventsRejected].Inc();
 }
 
-
-//---CheckBirthdays------------------------------------------------------------
-/*! \brief Sends a signal for each user whose birthday occurs in the next few days.
- *
- * This function is NOT IMPLEMENTED yet. Don't expect it to do anything usefull ;-P
- */
-void CICQDaemon::CheckBirthdays(UinList &uins)
-{
-  FOR_EACH_USER_START(LOCK_R)
-  {
-    if (pUser->Birthday(m_nBirthdayRange) != -1)
-      uins.push_back(pUser->Uin());
-  }
-  FOR_EACH_USER_END
-}
-
-
 /*----------------------------------------------------------------------------
  * CICQDaemon::SendEvent
  *
@@ -1424,44 +1445,12 @@ ICQEvent *CICQDaemon::SendExpectEvent_Server(const char *szId, unsigned long /*
 ICQEvent *CICQDaemon::SendExpectEvent_Server(unsigned long nUin, CPacket *packet,
    CUserEvent *ue, bool bExtendedEvent)
 {
-  // If we are already shutting down, don't start any events
-  if (m_bShuttingDown)
-  {
-    if (packet != NULL) delete packet;
-    if (ue != NULL) delete ue;
-    return NULL;
-  }
-
-  if (ue != NULL) ue->m_eDir = D_SENDER;
-  ICQEvent *e = new ICQEvent(this, m_nTCPSrvSocketDesc, packet, CONNECT_SERVER, nUin, ue);
-
-  if (e == NULL)  return NULL;
-
-  if (bExtendedEvent) PushExtendedEvent(e);
-
-  ICQEvent *result = SendExpectEvent(e, &ProcessRunningEvent_Server_tep);
-
-  // if an error occured, remove the event from the extended queue as well
-  if (result == NULL && bExtendedEvent)
-  {
-    pthread_mutex_lock(&mutex_extendedevents);
-    std::list<ICQEvent *>::iterator i;
-    for (i = m_lxExtendedEvents.begin(); i != m_lxExtendedEvents.end(); ++i)
-    {
-      if (*i == e)
-      {
-        m_lxExtendedEvents.erase(i);
-        break;
-      }
-    }
-    pthread_mutex_unlock(&mutex_extendedevents);
-  }
-
-  return result;
+  char szUin[24];
+  sprintf(szUin, "%lu", nUin);
+  return SendExpectEvent_Server(szUin, LICQ_PPID, packet, ue, bExtendedEvent);
 }
 
-
-ICQEvent *CICQDaemon::SendExpectEvent_Client(ICQUser *pUser, CPacket *packet,
+ICQEvent* CICQDaemon::SendExpectEvent_Client(const ICQUser* pUser, CPacket* packet,
    CUserEvent *ue)
 {
   // If we are already shutting down, don't start any events
@@ -1474,7 +1463,7 @@ ICQEvent *CICQDaemon::SendExpectEvent_Client(ICQUser *pUser, CPacket *packet,
 
   if (ue != NULL) ue->m_eDir = D_SENDER;
   ICQEvent *e = new ICQEvent(this, pUser->SocketDesc(packet->Channel()), packet,
-     CONNECT_USER, pUser->Uin(), ue);
+     CONNECT_USER, pUser->IdString(), pUser->PPID(), ue);
 
   if (e == NULL) return NULL;
 
@@ -1607,11 +1596,11 @@ void CICQDaemon::FailEvents(int sd, int err)
 /**
  * Search the running event queue for a specific event by subsequence.
  */
-bool CICQDaemon::hasServerEvent(unsigned long _nSubSequence)
+bool CICQDaemon::hasServerEvent(unsigned long _nSubSequence) const
 {
   bool hasEvent = false;
   pthread_mutex_lock(&mutex_runningevents);
-  list<ICQEvent *>::iterator iter;
+  list<ICQEvent*>::const_iterator iter;
   for (iter = m_lxRunningEvents.begin(); iter != m_lxRunningEvents.end(); ++iter)
   {
     if ((*iter)->CompareSubSequence(_nSubSequence))
@@ -1830,7 +1819,7 @@ void CICQDaemon::ProcessDoneEvent(ICQEvent *e)
       e->m_eResult == EVENT_ACKED &&
       e->m_nSubResult != ICQ_TCPxACK_RETURN)
   {
-    ICQUser *u = gUserManager.FetchUser(e->m_nDestinationUin, LOCK_R);
+    const ICQUser* u = gUserManager.FetchUser(e->m_nDestinationUin, LOCK_R);
     if (u != NULL)
     {
       e->m_pUserEvent->AddToHistory(u, LICQ_PPID, D_SENDER);
@@ -1877,9 +1866,9 @@ void CICQDaemon::ProcessDoneEvent(ICQEvent *e)
   case ICQ_CMDxSND_SETxSTATUS:
     if (e->m_eResult == EVENT_ACKED)
     {
-      ICQOwner *o = gUserManager.FetchOwner(LOCK_W);
+        ICQOwner* o = gUserManager.FetchOwner(LICQ_PPID, LOCK_W);
       ChangeUserStatus(o, ((CPU_SetStatus *)e->m_pPacket)->Status() );
-      gUserManager.DropOwner();
+        gUserManager.DropOwner(o);
     }
     PushPluginEvent(e);
     break;
@@ -1887,9 +1876,9 @@ void CICQDaemon::ProcessDoneEvent(ICQEvent *e)
   case ICQ_CMDxSND_SETxRANDOMxCHAT:
     if (e->m_eResult == EVENT_ACKED)
     {
-      ICQOwner *o = gUserManager.FetchOwner(LOCK_W);
+        ICQOwner* o = gUserManager.FetchOwner(LICQ_PPID, LOCK_W);
       o->SetRandomChatGroup(((CPU_SetRandomChatGroup *)e->m_pPacket)->Group());
-      gUserManager.DropOwner();
+        gUserManager.DropOwner(o);
     }
     PushPluginEvent(e);
     break;
@@ -2029,6 +2018,13 @@ ICQEvent *CICQDaemon::DoneExtendedEvent(unsigned long tag, EventResult _eResult)
   return(e);
 }
 
+void CICQDaemon::PushEvent(ICQEvent *e)
+{
+  assert(e != NULL);
+  pthread_mutex_lock(&mutex_runningevents);
+  m_lxRunningEvents.push_back(e);
+  pthread_mutex_unlock(&mutex_runningevents);
+}
 
 /*------------------------------------------------------------------------------
  * PushExtendedEvent
@@ -2218,11 +2214,11 @@ void CICQDaemon::CancelEvent(ICQEvent *e)
   e->m_eResult = EVENT_CANCELLED;
 
   if (e->m_nSubCommand == ICQ_CMDxSUB_CHAT)
-    icqChatRequestCancel(e->m_nDestinationUin, e->m_nSequence);
+    icqChatRequestCancel(e->Id(), e->m_nSequence);
   else if (e->m_nSubCommand == ICQ_CMDxSUB_FILE)
-    icqFileTransferCancel(e->m_nDestinationUin, e->m_nSequence);
+    icqFileTransferCancel(e->Id(), e->m_nSequence);
   else if (e->m_nSubCommand == ICQ_CMDxSUB_SECURExOPEN)
-    icqOpenSecureChannelCancel(e->m_nDestinationUin, e->m_nSequence);
+    icqOpenSecureChannelCancel(e->Id(), e->m_nSequence);
 
   ProcessDoneEvent(e);
 }
@@ -2231,11 +2227,11 @@ void CICQDaemon::CancelEvent(ICQEvent *e)
 //-----updateAllUsers-------------------------------------------------------------------------
 void CICQDaemon::UpdateAllUsers()
 {
-  FOR_EACH_UIN_START
+  FOR_EACH_USER_START(LOCK_R)
   {
-    icqRequestMetaInfo(nUin);
+    icqRequestMetaInfo(pUser->IdString());
   }
-  FOR_EACH_UIN_END
+  FOR_EACH_USER_END
 }
 
 
@@ -2245,7 +2241,7 @@ void CICQDaemon::UpdateAllUsersInGroup(GroupType g, unsigned short nGroup)
   {
     if (pUser->GetInGroup(g, nGroup))
     {
-      icqRequestMetaInfo(pUser->Uin());
+      icqRequestMetaInfo(pUser->IdString());
     }
   }
   FOR_EACH_USER_END
@@ -2410,8 +2406,7 @@ bool CICQDaemon::RemoveConversation(unsigned long _nCID)
 
 //-----ProcessMessage-----------------------------------------------------------
 void CICQDaemon::ProcessMessage(ICQUser *u, CBuffer &packet, char *message,
-                                unsigned short nMsgType, unsigned long nMask,
-                                unsigned long nMsgID[2],
+    unsigned short nMsgType, unsigned long nMask, const unsigned long nMsgID[2],
                                 unsigned short nSequence, bool bIsAck,
                                 bool &bNewUser)
 {
@@ -2423,9 +2418,9 @@ void CICQDaemon::ProcessMessage(ICQUser *u, CBuffer &packet, char *message,
   unsigned short nPort;
 
   // Do we accept it if we are in Occ or DND?
-  // ICQOwner *o = gUserManager.FetchOwner(LOCK_R);
+  // const ICQOwner* o = gUserManager.FetchOwner(LICQ_PPID, LOCK_R);
   // unsigned short nOwnerStatus = o->Status();
-  // gUserManager.DropOwner();
+  // gUserManager.DropOwner(o);
 
   unsigned short nLevel = nMask;
   unsigned long nFlags = ((nMask & ICQ_CMDxSUB_FxMULTIREC) ? E_MULTIxREC : 0)
@@ -2582,8 +2577,8 @@ void CICQDaemon::ProcessMessage(ICQUser *u, CBuffer &packet, char *message,
     }
     else
     {
-      gLog.Info(tr("%s%s (%lu) requested auto response.\n"), L_SRVxSTR,
-                u->GetAlias(), u->Uin());
+      gLog.Info(tr("%s%s (%s) requested auto response.\n"), L_SRVxSTR,
+          u->GetAlias(), u->IdString());
 
     CPU_AckGeneral *p = new CPU_AckGeneral(u, nMsgID[0], nMsgID[1],
                                            nSequence, nMsgType, true, nLevel);
@@ -2593,7 +2588,7 @@ void CICQDaemon::ProcessMessage(ICQUser *u, CBuffer &packet, char *message,
     u->SetLastCheckedAutoResponse();
 
       PushPluginSignal(new CICQSignal(SIGNAL_UPDATExUSER, USER_EVENTS,
-                       u->Uin()));
+          u->IdString(), u->PPID()));
     }
     u->Unlock();
     return;
@@ -2667,8 +2662,8 @@ void CICQDaemon::ProcessMessage(ICQUser *u, CBuffer &packet, char *message,
     {
       pAckEvent->m_pExtendedAck = pExtendedAck;
       pAckEvent->m_nSubResult = ICQ_TCPxACK_ACCEPT;
-      gLog.Info(tr("%s%s accepted from %s (%lu).\n"), L_SRVxSTR, szType,
-                u->GetAlias(), u->Uin());
+      gLog.Info(tr("%s%s accepted from %s (%s).\n"), L_SRVxSTR, szType,
+          u->GetAlias(), u->IdString());
       u->Unlock();
       ProcessDoneEvent(pAckEvent);
       u->Lock(LOCK_W);
@@ -2688,20 +2683,21 @@ void CICQDaemon::ProcessMessage(ICQUser *u, CBuffer &packet, char *message,
       {
         if (Ignore(IGNORE_NEWUSERS))
         {
-          gLog.Info(tr("%s%s from new user (%lu), ignoring.\n"), L_SRVxSTR,
-                    szType, u->Uin());
+          gLog.Info(tr("%s%s from new user (%s), ignoring.\n"), L_SRVxSTR,
+                    szType, u->IdString());
           if (szType)  free(szType);
-          RejectEvent(u->Uin(), pEvent);
+          RejectEvent(u->IdString(), pEvent);
           u->Unlock();
           return;
         }
-        gLog.Info(tr("%s%s from new user (%lu).\n"), L_SRVxSTR, szType, u->Uin());
-        AddUserToList(u);
+        gLog.Info(tr("%s%s from new user (%s).\n"), L_SRVxSTR, szType, u->IdString());
+        u->Unlock();
+        AddUserToList(u->IdString(), u->PPID(), false, true);
         bNewUser = false;
       }
       else
-        gLog.Info(tr("%s%s from %s (%lu).\n"), L_SRVxSTR, szType, u->GetAlias(),
-                  u->Uin());
+        gLog.Info(tr("%s%s from %s (%s).\n"), L_SRVxSTR, szType, u->GetAlias(),
+            u->IdString());
 
       if (AddUserEvent(u, pEvent))
         m_xOnEventManager.Do(nEventType, u);
@@ -2719,7 +2715,7 @@ void CICQDaemon::ProcessMessage(ICQUser *u, CBuffer &packet, char *message,
   if (szType)  free(szType);
 }
 
-bool CICQDaemon::WaitForReverseConnection(unsigned short id, unsigned long uin)
+bool CICQDaemon::WaitForReverseConnection(unsigned short id, const char* userId)
 {
   bool bSuccess = false;
   pthread_mutex_lock(&mutex_reverseconnect);
@@ -2728,7 +2724,7 @@ bool CICQDaemon::WaitForReverseConnection(unsigned short id, unsigned long uin)
   for (iter = m_lReverseConnect.begin(); iter != m_lReverseConnect.end();
     ++iter)
   {
-    if ((*iter)->nId == id && (*iter)->nUin == uin)
+    if ((*iter)->nId == id && (*iter)->myIdString == userId)
       break;
   }
 
@@ -2754,7 +2750,7 @@ bool CICQDaemon::WaitForReverseConnection(unsigned short id, unsigned long uin)
           L_WARNxSTR);
         goto done;
       }
-      if ((*iter)->nId == id && (*iter)->nUin == uin)
+      if ((*iter)->nId == id && (*iter)->myIdString == userId)
       {
         if ((*iter)->bFinished)
         {
@@ -2772,7 +2768,7 @@ bool CICQDaemon::WaitForReverseConnection(unsigned short id, unsigned long uin)
   for (iter = m_lReverseConnect.begin(); iter != m_lReverseConnect.end();
     ++iter)
   {
-    if ((*iter)->nId == id && (*iter)->nUin == uin)
+    if ((*iter)->nId == id && (*iter)->myIdString == userId)
     {
       delete *iter;
       m_lReverseConnect.erase(iter);
@@ -2785,11 +2781,11 @@ done:
   return bSuccess;
 }
 
-CReverseConnectToUserData::CReverseConnectToUserData(unsigned long uin, unsigned long id,
+CReverseConnectToUserData::CReverseConnectToUserData(const char* idString, unsigned long id,
       unsigned long data, unsigned long ip, unsigned short port,
       unsigned short version, unsigned short failedport, unsigned long msgid1,
       unsigned long msgid2) :
-  nUin(uin), nId(id), nData(data), nIp(ip), nPort(port),
+  myIdString(idString), nId(id), nData(data), nIp(ip), nPort(port),
   nFailedPort(failedport), nVersion(version), nMsgID1(msgid1),
   nMsgID2(msgid2), bSuccess(false), bFinished(false)
 {
@@ -2824,3 +2820,16 @@ bool ParseFE(char *szBuffer, char ***szSubStr, int nNumSubStr)
 
   return (!*pcEnd);
 }
+
+CUserProperties::CUserProperties()
+  : newAlias(NULL),
+    newCellular(NULL),
+    normalSid(0),
+    groupId(0),
+    visibleSid(0),
+    invisibleSid(0),
+    inIgnoreList(false),
+    awaitingAuth(false)
+{
+  tlvs.clear();
+}
