@@ -39,16 +39,16 @@ CSearchAck::CSearchAck(const char *_szId, unsigned long _nPPID)
 
 CSearchAck::~CSearchAck()
 {
-  if (m_szAlias != NULL) free(m_szAlias);
-  if (m_szFirstName != NULL) free(m_szFirstName);
-  if (m_szLastName != NULL) free(m_szLastName);
-  if (m_szEmail != NULL) free(m_szEmail);
-  if (m_szId != NULL) free(m_szId);
+  free(m_szAlias);
+  free(m_szFirstName);
+  free(m_szLastName);
+  free(m_szEmail);
+  free(m_szId);
 }
 
 
 //-----CExtendedAck----------------------------------------------------------
-CExtendedAck::CExtendedAck(bool bAccepted, unsigned short nPort, char *szResponse)
+CExtendedAck::CExtendedAck(bool bAccepted, unsigned short nPort, const char* szResponse)
 {
   m_bAccepted = bAccepted;
   m_nPort = nPort;
@@ -57,8 +57,7 @@ CExtendedAck::CExtendedAck(bool bAccepted, unsigned short nPort, char *szRespons
 
 CExtendedAck::~CExtendedAck()
 {
-  if (m_szResponse != NULL)
-    free(m_szResponse);
+  free(m_szResponse);
 }
 
 
@@ -110,18 +109,32 @@ ICQEvent::ICQEvent(CICQDaemon *_pDaemon, int _nSocketDesc, CPacket *p,
 //   : m_xBuffer(p.getBuffer())
 {
   // set up internal variables
-  m_pPacket = p;
   m_bCancelled = false;
   m_Deleted = false;
   m_NoAck = false;
-  m_nChannel = p->Channel();
-  m_nCommand = p->Command();
-  m_nSNAC = p->SNAC();
-  m_nSubCommand = p->SubCommand();
-  m_nSequence = p->Sequence();
-  m_nSubSequence = p->SubSequence();
-  m_nSubType = (p->SNAC() & 0xFFFF);
-  m_nExtraInfo = p->ExtraInfo();
+  if (p)
+  {
+    m_pPacket = p;
+    m_nChannel = p->Channel();
+    m_nCommand = p->Command();
+    m_nSNAC = p->SNAC();
+    m_nSubCommand = p->SubCommand();
+    m_nSequence = p->Sequence();
+    m_nSubSequence = p->SubSequence();
+    m_nSubType = (p->SNAC() & 0xFFFF);
+    m_nExtraInfo = p->ExtraInfo();
+  } else
+  {
+    m_pPacket = NULL;
+    m_nChannel = 0;
+    m_nCommand = 0;
+    m_nSNAC = 0;
+    m_nSubCommand = 0;
+    m_nSequence = 0;
+    m_nSubSequence = 0;
+    m_nSubType = 0;
+    m_nExtraInfo = 0;
+  }
   m_nDestinationUin = 0;
   m_szId = _szId ? strdup(_szId) : 0;
   m_nPPID = _nPPID;
@@ -142,7 +155,7 @@ ICQEvent::ICQEvent(CICQDaemon *_pDaemon, int _nSocketDesc, CPacket *p,
 }
 
 //-----ICQEvent::constructor----------------------------------------------------
-ICQEvent::ICQEvent(ICQEvent *e)
+ICQEvent::ICQEvent(const ICQEvent* e)
 //   : m_xBuffer(e->m_xBuffer)
 {
   m_nEventId = e->m_nEventId;
@@ -192,8 +205,7 @@ ICQEvent::~ICQEvent()
   assert(!m_Deleted);
   m_Deleted = true;
 
-  if (m_szId)
-    free(m_szId);
+  free(m_szId);
   delete m_pPacket;
   delete m_pUserEvent;
   delete m_pExtendedAck;
@@ -202,13 +214,27 @@ ICQEvent::~ICQEvent()
 }
 
 
+//-----ICQEvent::AttachPacket---------------------------------------------------
+void ICQEvent::AttachPacket(CPacket *p)
+{
+  m_pPacket = p;
+  m_nChannel = p->Channel();
+  m_nCommand = p->Command();
+  m_nSNAC = p->SNAC();
+  m_nSubCommand = p->SubCommand();
+  m_nSequence = p->Sequence();
+  m_nSubSequence = p->SubSequence();
+  m_nSubType = (p->SNAC() & 0xFFFF);
+  m_nExtraInfo = p->ExtraInfo();
+}
+  
 //-----ICQEvent::CompareEvent---------------------------------------------------
-bool ICQEvent::CompareEvent(int sockfd, unsigned long _nSequence) const
+bool ICQEvent::CompareEvent(int sockfd, unsigned short _nSequence) const
 {
    return(m_nSocketDesc == sockfd && m_nSequence == _nSequence);
 }
 
-bool ICQEvent::CompareEvent(unsigned long nEventId) const
+bool ICQEvent::CompareEvent(unsigned short nEventId) const
 {
   return (m_nEventId == nEventId);
 }
@@ -250,25 +276,26 @@ ICQUser *ICQEvent::GrabUnknownUser()
 
 //=====CICQSignal===============================================================
 CICQSignal::CICQSignal(unsigned long nSignal, unsigned long nSubSignal,
-                       const char *szId, unsigned long nPPID, int nArgument,char *nParameters)
+                       const char *szId, unsigned long nPPID,
+                       int nArg, unsigned long nCID)
 {
   m_nSignal = nSignal;
   m_nSubSignal = nSubSignal;
   m_nUin = 0;
   m_szId = szId ? strdup(szId) : 0;
   m_nPPID = nPPID;
-  m_nArgument = nArgument;
-  m_szParameters = (nParameters!=NULL)?strdup(nParameters):NULL;
+  m_nArgument = nArg;
+  m_nCID = nCID;
 }
 
 CICQSignal::CICQSignal(unsigned long nSignal, unsigned long nSubSignal,
-                       unsigned long nUin, int nArgument,char *nParameters)
+                       unsigned long nUin, int nArg, unsigned long nCID)
 {
   m_nSignal = nSignal;
   m_nSubSignal = nSubSignal;
   m_nUin = nUin;
-  m_nArgument = nArgument;
-  m_szParameters = (nParameters!=NULL)?strdup(nParameters):NULL;
+  m_nArgument = nArg;
+  m_nCID = nCID;
 
   char szUin[24];
   sprintf(szUin, "%lu", nUin);
@@ -276,7 +303,7 @@ CICQSignal::CICQSignal(unsigned long nSignal, unsigned long nSubSignal,
   m_nPPID = LICQ_PPID;
 }
 
-CICQSignal::CICQSignal(CICQSignal *s)
+CICQSignal::CICQSignal(const CICQSignal* s)
 {
   m_nSignal = s->Signal();
   m_nSubSignal = s->SubSignal();
@@ -284,22 +311,20 @@ CICQSignal::CICQSignal(CICQSignal *s)
   m_szId = s->Id() ? strdup(s->Id()) : 0;
   m_nPPID = s->PPID();
   m_nArgument = s->Argument();
-  m_szParameters = (s->Parameters()!= NULL)?strdup(s->Parameters()):NULL;
+  m_nCID = s->CID();
 }
 
 
 CICQSignal::~CICQSignal()
 {
-    if (m_szParameters != NULL) {
-        free(m_szParameters);
-    }
   if (m_szId)
     free(m_szId);
 }
 
-CSignal::CSignal(SIGNAL_TYPE e, const char *szId)
+CSignal::CSignal(SIGNAL_TYPE e, const char *szId, unsigned long nCID)
 {
   m_eType = e;
+  m_nCID = nCID;
   if (szId)
     m_szId = strdup(szId);
   else
@@ -310,14 +335,14 @@ CSignal::CSignal(SIGNAL_TYPE e, const char *szId)
 CSignal::CSignal(CSignal *s)
 {
   m_eType = s->m_eType;
+  m_nCID = s->m_nCID;
   m_szId = s->m_szId ? strdup(s->m_szId) : 0;
   thread_plugin = s->thread_plugin;
 }
 
 CSignal::~CSignal()
 {
-  if (m_szId)
-    free(m_szId);
+  free(m_szId);
 }
 
 CLogonSignal::CLogonSignal(unsigned long nLogonStatus)
@@ -326,43 +351,83 @@ CLogonSignal::CLogonSignal(unsigned long nLogonStatus)
   m_nLogonStatus = nLogonStatus;
 }
 
+CLogonSignal::~CLogonSignal()
+{
+  // Empty
+}
+
 CLogoffSignal::CLogoffSignal()
   : CSignal(PROTOxLOGOFF, 0)
 {
 }
 
+CLogoffSignal::~CLogoffSignal()
+{
+  // Empty
+}
+
 CChangeStatusSignal::CChangeStatusSignal(unsigned long nStatus)
   : CSignal(PROTOxCHANGE_STATUS, 0)
 {
   m_nStatus = nStatus;
 }
 
+CChangeStatusSignal::~CChangeStatusSignal()
+{
+  // Empty
+}
+
 CAddUserSignal::CAddUserSignal(const char *szId, bool bAuthRequired)
   : CSignal(PROTOxADD_USER, szId)
 {
   m_bAuthRequired = bAuthRequired;
 }
 
+CAddUserSignal::~CAddUserSignal()
+{
+  // Empty
+}
+
 CRemoveUserSignal::CRemoveUserSignal(const char *szId)
   : CSignal(PROTOxREM_USER, szId)
 {
 }
 
+CRemoveUserSignal::~CRemoveUserSignal()
+{
+  // Empty
+}
+
 CRenameUserSignal::CRenameUserSignal(const char *szId)
   : CSignal(PROTOxRENAME_USER, szId)
 {
 }
 
-CSendMessageSignal::CSendMessageSignal(const char *szId, const char *szMsg)
-  : CSignal(PROTOxSENDxMSG, szId)
+CRenameUserSignal::~CRenameUserSignal()
+{
+  // Empty
+}
+
+CSendMessageSignal::CSendMessageSignal(const char *szId, const char *szMsg, unsigned long nCID)
+  : CSignal(PROTOxSENDxMSG, szId, nCID)
 {
   m_szMsg = szMsg ? strdup(szMsg) : 0;
 }
 
-CTypingNotificationSignal::CTypingNotificationSignal(const char *szId,
-  bool bActive)
-    : CSignal(PROTOxSENDxTYPING_NOTIFICATION, szId), m_bActive(bActive)
+CSendMessageSignal::~CSendMessageSignal()
 {
+  free(m_szMsg);
+}
+
+CTypingNotificationSignal::CTypingNotificationSignal(const char *szId,
+  bool bActive, unsigned long nCID)
+    : CSignal(PROTOxSENDxTYPING_NOTIFICATION, szId, nCID), m_bActive(bActive)
+{
+}
+
+CTypingNotificationSignal::~CTypingNotificationSignal()
+{
+  // Empty
 }
 
 CGrantAuthSignal::CGrantAuthSignal(const char *szId, const char *szMsg)
@@ -371,9 +436,233 @@ CGrantAuthSignal::CGrantAuthSignal(const char *szId, const char *szMsg)
   m_szMsg = szMsg ? strdup(szMsg) : 0;
 }
 
+CGrantAuthSignal::~CGrantAuthSignal()
+{
+  free(m_szMsg);
+}
+
 CRefuseAuthSignal::CRefuseAuthSignal(const char *szId, const char *szMsg)
   : CSignal(PROTOxSENDxREFUSExAUTH, szId)
 {
   m_szMsg = szMsg ? strdup(szMsg) : 0;
 }
 
+CRefuseAuthSignal::~CRefuseAuthSignal()
+{
+  free(m_szMsg);
+}
+
+CRequestInfo::CRequestInfo(const char *szId)
+  : CSignal(PROTOxREQUESTxINFO, szId)
+{
+}
+
+CRequestInfo::~CRequestInfo()
+{
+  // Empty
+}
+
+CUpdateInfoSignal::CUpdateInfoSignal(const char *szAlias, const char *szFirstName,
+  const char *szLastName, const char *szEmail,
+  const char *szCity,
+  const char *szState, const char *szPhoneNumber,
+  const char *szFaxNumber, const char *szAddress,
+  const char *szCellNumber, const char *szZipCode)
+  : CSignal(PROTOxUPDATExINFO, 0)
+{
+  m_szAlias = szAlias ? strdup(szAlias) : 0;
+  m_szFirstName = szFirstName ? strdup(szFirstName) : 0;
+  m_szLastName = szLastName ? strdup(szLastName) : 0;
+  m_szEmail = szEmail ? strdup(szEmail) : 0;
+  m_szCity = szCity ? strdup(szCity) : 0;
+  m_szState = szState ? strdup(szState) : 0;
+  m_szPhoneNumber = szPhoneNumber ? strdup(szPhoneNumber) : 0;
+  m_szFaxNumber = szFaxNumber ? strdup(szFaxNumber) : 0;
+  m_szAddress = szAddress ? strdup(szAddress) : 0;
+  m_szCellNumber = szCellNumber ? strdup(szCellNumber) : 0;
+  m_szZipCode = szZipCode ? strdup(szZipCode) : 0;
+}
+
+CUpdateInfoSignal::~CUpdateInfoSignal()
+{
+  free(m_szAlias);
+  free(m_szFirstName);
+  free(m_szLastName);
+  free(m_szEmail);
+  free(m_szCity);
+  free(m_szState);
+  free(m_szPhoneNumber);
+  free(m_szFaxNumber);
+  free(m_szAddress);
+  free(m_szCellNumber);
+  free(m_szZipCode);
+}
+
+CRequestPicture::CRequestPicture(const char *szId)
+  : CSignal(PROTOxREQUESTxPICTURE, szId)
+{
+}
+
+CRequestPicture::~CRequestPicture()
+{
+}
+
+CBlockUserSignal::CBlockUserSignal(const char *szId)
+  : CSignal(PROTOxBLOCKxUSER, szId)
+{
+}
+
+CBlockUserSignal::~CBlockUserSignal()
+{
+  // Empty
+}
+
+CUnblockUserSignal::CUnblockUserSignal(const char *szId)
+  : CSignal(PROTOxUNBLOCKxUSER, szId)
+{
+}
+
+CUnblockUserSignal::~CUnblockUserSignal()
+{
+  // Empty
+}
+
+CAcceptUserSignal::CAcceptUserSignal(const char *szId)
+  : CSignal(PROTOxACCEPTxUSER, szId)
+{
+}
+
+CAcceptUserSignal::~CAcceptUserSignal()
+{
+  // Empty
+}
+
+CUnacceptUserSignal::CUnacceptUserSignal(const char *szId)
+  : CSignal(PROTOxUNACCEPTxUSER, szId)
+{
+}
+
+CUnacceptUserSignal::~CUnacceptUserSignal()
+{
+  // Empty
+}
+
+CIgnoreUserSignal::CIgnoreUserSignal(const char *szId)
+  : CSignal(PROTOxIGNORExUSER, szId)
+{
+}
+
+CIgnoreUserSignal::~CIgnoreUserSignal()
+{
+  // Empty
+}
+
+CUnignoreUserSignal::CUnignoreUserSignal(const char *szId)
+  : CSignal(PROTOxUNIGNORExUSER, szId)
+{
+}
+
+CUnignoreUserSignal::~CUnignoreUserSignal()
+{
+  // Empty
+}
+
+CSendFileSignal::CSendFileSignal(const char *szId, const char *szFile,
+                                 const char *szMessage, ConstFileList &_lFileList)
+  : CSignal(PROTOxSENDxFILE, szId)
+{
+  m_szFile = (szFile ? strdup(szFile) : NULL);
+
+  m_szMessage = (szMessage ? strdup(szMessage) : NULL);
+
+  m_lFileList = _lFileList;
+}
+
+CSendFileSignal::~CSendFileSignal()
+{
+  free(m_szFile);
+  free(m_szMessage);
+}
+
+CSendChatSignal::CSendChatSignal(const char *szId, const char *szMessage)
+  : CSignal(PROTOxSENDxCHAT, szId)
+{
+  m_szMessage = (szMessage ? strdup(szMessage) : 0);
+}
+
+CSendChatSignal::~CSendChatSignal()
+{
+  free(m_szMessage);
+}
+
+CCancelEventSignal::CCancelEventSignal(const char *szId, unsigned long nFlag)
+  : CSignal(PROTOxCANCELxEVENT, szId)
+{
+  m_nFlag = nFlag;
+}
+
+CCancelEventSignal::~CCancelEventSignal()
+{
+  // Empty
+}
+
+CSendEventReplySignal::CSendEventReplySignal(const char *szId,
+  const char *szMessage, bool bAccept, unsigned short nPort,
+  unsigned long nSequence, unsigned long nFlag1, unsigned long nFlag2,
+  bool bDirect)
+  : CSignal(PROTOxSENDxEVENTxREPLY, szId)
+{
+  m_szMessage = (szMessage ? strdup(szMessage) : 0);
+
+  m_bAccept = bAccept;
+  m_bDirect = bDirect;
+  m_nPort = nPort;
+  m_nSequence = nSequence;
+  m_nFlag = nFlag1;
+  m_nFlag2 = nFlag2;
+}
+
+CSendEventReplySignal::~CSendEventReplySignal()
+{
+  free(m_szMessage);
+}
+
+COpenedWindowSignal::COpenedWindowSignal(const char *szId)
+  : CSignal(PROTOxOPENEDxWINDOW, szId)
+{
+}
+
+COpenedWindowSignal::~COpenedWindowSignal()
+{
+  // Empty
+}
+
+CClosedWindowSignal::CClosedWindowSignal(const char *szId)
+  : CSignal(PROTOxCLOSEDxWINDOW, szId)
+{
+}
+
+CClosedWindowSignal::~CClosedWindowSignal()
+{
+  // Empty
+}
+
+COpenSecureSignal::COpenSecureSignal(const char *szId)
+  : CSignal(PROTOxOPENxSECURE, szId)
+{
+}
+
+COpenSecureSignal::~COpenSecureSignal()
+{
+  // Empty
+}
+
+CCloseSecureSignal::CCloseSecureSignal(const char *szId)
+  : CSignal(PROTOxCLOSExSECURE, szId)
+{
+}
+
+CCloseSecureSignal::~CCloseSecureSignal()
+{
+  // Empty
+}
