@@ -91,6 +91,14 @@ void licq_handle_sigabrt(int s)
     return;
   }
 
+  // When shutting down, calls to pthread_rdwr_destroy_np() may trigger another
+  // assert causing thish andler to be called again and backtrace files
+  // overwritten.
+  // Restore default signal handlers now, that way we won't get called again if
+  // this happens.
+  signal(SIGSEGV, SIG_DFL);
+  signal(SIGABRT, SIG_DFL);
+
   /*
    * Use gdb to try to generate a backtrace for all threads and save
    * it in BASE_DIR/licq.backtrace.gdb.
@@ -131,7 +139,7 @@ void licq_handle_sigabrt(int s)
       FILE* file = fopen(filename, "a");
       if (file != NULL)
       {
-        fprintf(file, "\ntime: %lu\n", time(NULL));
+        fprintf(file, "\ntime: %lu\n", (unsigned long)time(NULL));
         fclose(file);
       }
 
@@ -160,17 +168,19 @@ void licq_handle_sigabrt(int s)
     void *array[size];
     int n = backtrace(array, size);
     char **res = backtrace_symbols(array, n);
-    for (i = 0; i < n; i++)
+    if (res == NULL)
+      fprintf(stderr, tr("Failed to retrive backtrace symbols"));
+    else
     {
-      fprintf(stderr, "%s\n", res[i]);
-      if (file != NULL)
-        fprintf(file, "%s\n", res[i]);
-    }
+      for (i = 0; i < n; i++)
+      {
+        fprintf(stderr, "%s\n", res[i]);
+        if (file != NULL)
+          fprintf(file, "%s\n", res[i]);
+      }
 
-    free(res);
-    /*array[0] = si->si_addr;
-    res = backtrace_symbols(array, 1);
-    fprintf(stderr, "%s\n", res[0]);*/
+      free(res);
+    }
   }
   fprintf(stderr, tr("Attempting to generate core file.\n"));
 
@@ -188,7 +198,8 @@ void licq_handle_sigabrt(int s)
            "Licq has encountered a fatal error.\n"
            "Please report this error either by creating a new ticket "
            "at http://trac.licq.org/ or by sending an e-mail to the mailing list "
-           "licq-dev@googlegroups.com (see http://trac.licq.org/wiki/MailingList).\n"
+           "licq-dev@googlegroups.com (you must be registered to be able to post, "
+           "see http://trac.licq.org/wiki/MailingList).\n"
            "\n"
            "To help us debug the error, please include a full description of "
            "what you did when the error occurred. Additionally, please include "
