@@ -41,6 +41,7 @@ extern int h_errno;
 #include "licq_user.h"
 #include "support.h"
 #include "licq_icqd.h"
+#include "licq_proxy.h"
 
 #ifdef USE_OPENSSL
 #include <openssl/ssl.h>
@@ -177,32 +178,10 @@ char *INetSocket::RemoteIpStr(char *buf)
   return (inet_ntoa_r(*(struct in_addr *)&m_sRemoteAddr.sin_addr.s_addr, buf));
 }
 
-//-----INetSocket::SetOwner---------------------------------------------------
-void INetSocket::SetOwner(unsigned long _nOwner)
-{
-  m_nOwner = _nOwner;
-  if (_nOwner)
-  {
-    char szUin[24];
-    sprintf(szUin, "%lu", _nOwner);
-    m_szOwnerId = strdup(szUin);
-    m_nOwnerPPID = LICQ_PPID;
-  }
-  else
-  {
-    m_szOwnerId = 0;
-    m_nOwnerPPID = 0;
-  }
-}
-
 void INetSocket::SetOwner(const char *_szOwnerId, unsigned long _nOwnerPPID)
 {
   m_szOwnerId = strdup(_szOwnerId);
   m_nOwnerPPID = _nOwnerPPID;
-  if (m_nOwnerPPID == LICQ_PPID)
-    m_nOwner = strtoul(_szOwnerId, (char **)NULL, 10);
-  else
-    m_nOwner = 0;
 }
 
 //-----INetSocket::Error------------------------------------------------------
@@ -264,24 +243,6 @@ char *INetSocket::ErrorStr(char *buf, int buflen)
 }
 
 
-//-----INetSocket::constructor--------------------------------------------------
-INetSocket::INetSocket(unsigned long _nOwner)
-{
-  m_nDescriptor = -1;
-  SetOwner(_nOwner);
-
-  m_nVersion = 0;
-  m_nErrorType = SOCK_ERROR_none;
-  memset(&m_sRemoteAddr, 0, sizeof(struct sockaddr_in));
-  memset(&m_sLocalAddr, 0, sizeof(struct sockaddr_in));
-  m_szRemoteName = NULL;
-  m_xProxy = NULL;
-  m_nChannel = ICQ_CHNxNONE;
-  
-  // Initialise the mutex
-  pthread_mutex_init(&mutex, NULL);
-}
-
 INetSocket::INetSocket(const char *_szOwnerId, unsigned long _nOwnerPPID)
 {
   m_nDescriptor = -1;
@@ -290,10 +251,6 @@ INetSocket::INetSocket(const char *_szOwnerId, unsigned long _nOwnerPPID)
   else
     m_szOwnerId = 0;
   m_nOwnerPPID = _nOwnerPPID;
-  if (m_nOwnerPPID == LICQ_PPID)
-    m_nOwner = strtoul(_szOwnerId, (char **)NULL, 10);
-  else
-    m_nOwner = 0;
   m_nVersion = 0;
   m_nErrorType = SOCK_ERROR_none;
   memset(&m_sRemoteAddr, 0, sizeof(struct sockaddr_in));
@@ -636,12 +593,6 @@ bool INetSocket::RecvRaw()
 
 //=====SrvSocket===============================================================
 
-SrvSocket::SrvSocket(unsigned long _nOwner) : INetSocket(_nOwner)
-{
-  strcpy(m_szID, "SRV");
-  m_nSockType = SOCK_STREAM;
-}
-
 SrvSocket::SrvSocket(const char *s, unsigned long n) : INetSocket(s, n)
 {
   strcpy(m_szID, "SRV");
@@ -781,14 +732,51 @@ bool SrvSocket::RecvPacket()
   return (true);
 }
 
-//=====TCPSocket===============================================================
-TCPSocket::TCPSocket(unsigned long _nOwner) : INetSocket(_nOwner)
+/*-----SrvSocket::ConnectTo--------------------------------------------------
+* Establish connection to server (through proxy or not)
+*---------------------------------------------------------------------------*/
+bool SrvSocket::ConnectTo(const char* server, unsigned short port,
+                          ProxyServer *xProxy)
 {
-  strcpy(m_szID, "TCP");
-  m_nSockType = SOCK_STREAM;
-  m_p_SSL = NULL;
+  char ipbuf[32];
+
+  if (xProxy == NULL)
+  {
+    gLog.Info(tr("%sResolving %s port %d...\n"), L_SRVxSTR, server, port);
+    if (!SetRemoteAddr(server, port)) {
+      char buf[128];
+      gLog.Warn(tr("%sUnable to resolve %s:\n%s%s.\n"), L_ERRORxSTR,
+                server, L_BLANKxSTR, ErrorStr(buf, 128));
+      return false;
+    }
+    gLog.Info(tr("%sICQ server found at %s:%d.\n"), L_SRVxSTR,
+              RemoteIpStr(ipbuf), RemotePort());
+  }
+  else
+  {
+    // It doesn't matter if it resolves or not, the proxy should do it then
+    SetProxy(xProxy);
+    SetRemoteAddr(server, port);
+  }
+
+  if (xProxy == NULL)
+    gLog.Info(tr("%sOpening socket to server.\n"), L_SRVxSTR);
+  else
+    gLog.Info(tr("%sOpening socket to server %s:%d via proxy.\n"),
+              L_SRVxSTR, server, port);
+  if (!OpenConnection())
+  {
+    char buf[128];
+    gLog.Warn(tr("%sUnable to connect to %s:%d:\n%s%s.\n"), L_ERRORxSTR,
+              RemoteIpStr(ipbuf), RemotePort(), L_BLANKxSTR,
+              ErrorStr(buf, 128));
+    return false;
+  }
+
+  return true;
 }
 
+//=====TCPSocket===============================================================
 TCPSocket::TCPSocket(const char *s, unsigned long n) : INetSocket(s, n)
 {
   strcpy(m_szID, "TCP");
@@ -813,12 +801,32 @@ TCPSocket::~TCPSocket()
  * Called to set up a given TCPSocket from an incoming connection on the
  * current TCPSocket
  *---------------------------------------------------------------------------*/
-void TCPSocket::RecvConnection(TCPSocket &newSocket)
+bool TCPSocket::RecvConnection(TCPSocket &newSocket)
 {
   socklen_t sizeofSockaddr = sizeof(struct sockaddr_in);
+  bool success = false;
 
-  newSocket.m_nDescriptor = accept(m_nDescriptor, (struct sockaddr *)&newSocket.m_sRemoteAddr, &sizeofSockaddr);
-  newSocket.SetLocalAddress();
+  // Make sure we stay under FD_SETSIZE
+  // See:
+  // * http://www.securityfocus.com/archive/1/490711
+  // * http://securityvulns.com/docs7669.html
+  // for more details
+  // This probably has no affect, since we are using multiple threads, but keep it here 
+  // to be used as a sanity check.
+  int newDesc = accept(m_nDescriptor, (struct sockaddr *)&newSocket.m_sRemoteAddr, &sizeofSockaddr);
+  if (newDesc < FD_SETSIZE)
+  {
+    newSocket.m_nDescriptor = newDesc;
+    newSocket.SetLocalAddress();
+    success = true;
+  }
+  else
+  {
+    gLog.Error(tr("%sCannot accept new connection, too many descriptors in use.\n"), L_ERRORxSTR);
+    close(newDesc);
+  }
+
+  return success;
 }
 
 #define m_pSSL ((SSL *) m_p_SSL)
@@ -841,9 +849,6 @@ void TCPSocket::TransferConnectionFrom(TCPSocket &from)
     m_szOwnerId = 0;
   m_nOwnerPPID = from.m_nOwnerPPID;
 
-  if (m_nOwnerPPID == LICQ_PPID)
-    m_nOwner = from.m_nOwner;
-
   m_nVersion = from.m_nVersion;
   if (from.m_p_SSL)
   {
@@ -1310,7 +1315,8 @@ void TCPSocket::SecureStop()
 
 #endif /*-----End of OpenSSL code------------------------------------------*/
 
-UDPSocket::UDPSocket(unsigned long _nOwner) : INetSocket(_nOwner)
+UDPSocket::UDPSocket(const char* ownerId, unsigned long ownerPpid)
+    : INetSocket(ownerId, ownerPpid)
 {
   strcpy(m_szID, "UDP");
   m_nSockType = SOCK_DGRAM;
