@@ -47,7 +47,7 @@ void SetString(char **szDest, const char *szSource)
 
 char *ParseDigits(char *szDest, const char *szSource, unsigned int nLen)
 {
-  int n = 0;
+  unsigned int n = 0;
   char *szCur = szDest;
 
   while ((*szSource) && (n < nLen))
@@ -109,7 +109,63 @@ int UinString(char *_szBuf, size_t _nSize, unsigned long _nUin)
   int nRet = snprintf(_szBuf, _nSize - 1, "%lu", _nUin);
   _szBuf[_nSize - 1] = '\0';
 
-  return (nRet == -1) ? _nSize - 1 : nRet; 
+  return (nRet == -1) ? (int)_nSize - 1 : nRet; 
+}
+
+char *PrintHex(char *szPrint, const char *szHex, size_t nSize)
+{
+  size_t i, j = 0;
+
+  for(i = 0; i < nSize ; i++)
+  {
+    unsigned char byte = (unsigned char)szHex[i];
+
+    unsigned char high = (byte >> 4) & 0x0f;
+    unsigned char low = byte & 0x0f;
+
+    if (high > 9)
+      szPrint[j++] = 'A' + (high - 10);
+    else
+      szPrint[j++] = '0' + high;
+
+    if (low > 9)
+      szPrint[j++] = 'A' + (low - 10);
+    else
+      szPrint[j++] = '0' + low;
+  }
+  szPrint[j] = '\0';
+
+  return szPrint;
+}
+
+char *ReadHex(char *szHex, const char *szRead, size_t nSize)
+{
+  size_t i, j = 0;
+  for(i = 0; i < nSize ; i++)
+  {
+    unsigned char digit, high, low;
+
+    if (szRead[j] == '\0') break;
+    digit = (unsigned char)szRead[j++];
+    high = 0;
+    if (digit >= 'A')
+      high = digit - 'A' + 10;
+    else if (digit >= '0')
+      high = digit - '0';
+    high <<= 4;
+
+    if (szRead[j] == '\0') break;
+    digit = (unsigned char)szRead[j++];
+    low = 0;
+    if (digit >= 'A')
+      low = digit - 'A' + 10;
+    else if (digit >= '0')
+      low = digit - '0';
+
+    szHex[i] = high + low;
+  }
+  
+  return szHex;
 }
 
 int Redirect(const char *_szFile)
@@ -249,6 +305,8 @@ int gethostbyname_r_portable(const char *szHostName, struct hostent *h, char *bu
 #else
 #warning "I don't know how to do reentrant gethostbyname on this machine."
 #warning "Using thread-unsafe version."
+  (void)buf;
+  (void)buflen;
   struct hostent *h_buf;
   h_buf = gethostbyname(szHostName);
   if (h_buf != NULL) memcpy(h, h_buf, sizeof(struct hostent));
